#!/usr/bin/env python
####################################################################
# (C) Copyright 2017 Hewlett Packard Enterprise Development LP.
# @(#) MC DRR Storae Prepare script.
# @(#) Product Name                : Metrocluster Dress Rehearsal
# @(#) Product Version             : B.12.10.00
# @(#) Patch Name                  :
#
# NOTE:    This script is not configurable!  Any changes made to this
#          script will be overwritten when you upgrade to the next
#          release of Serviceguard metrocluster addons.
#
###################################################################

"""Usage:-
    cmdrrstorageutils.py [START|STOP|DLS_RDM] [PKG_NAME] [DRR_ID] [DEBUG 0|1|2]\
 [ERR_FILE] [NODE_NAME optional]
 
Example:-
    cmdrrstorageutils.py START PKG_NAME DRR_ID 0 /home/err node_name

where-
    START     - To start rehearsal
    STOP      - To stop  rehearsal
    DLS_RDM   - To be used internally by the script to execute at remote node
    PKG_NAME  - Package name going to be rehearsed
    DRR_ID    - DR Rehearsal id 
    ERR_FILE  - Error file name where error messages will be dumped
    NODE_NAME - Current node name. This is an optional parameter
    DEBUG     - 
       0 : Default mode to not print any debug messages
       1 : Debug messages will be printed to STDOUT
       2 : Debug messages will be written to SGRUN/DRxR/mc_debug.log
    """

## All imported modules

import sys, os
COMMON_UTIL  = "/opt/drenabler/utils/drr_commonutils/"
ST_3PAR_UTIL = "/opt/drenabler/utils/drr_storageutils/3par/"
sys.path.append(COMMON_UTIL)
sys.path.append(ST_3PAR_UTIL)

from drrhelper    import Helper
from drrpkgutils  import Package
from drrhostutils import HOST

###
# Class STORAGE_ARRAY()
###
class STORAGE_ARRAY(Helper):

    ###
    # INIT
    ###
    def __init__(self, pkg_name, drr_id, DEBUG, node_name, err_file):

        Helper.__init__(self, DEBUG, pkg_name, drr_id, err_file)
        self.ret_code              = ""
        self.roll_back_flag        = False
        self.err_flag              = False
        self.error_file            = err_file
        self.node_type             = 'PHYSICAL'
        self.virt_type             = 'NA'
        self.pkg_type              = 'NA'
        self.pkg_id                = '0'
        self.pv_wwid_array         = []
        self.node_hba_arr          = []
        self.pkg_array             = []
        self.pri_sec_array_wwid    = {}
        self.vmfs_orig_snap_pv_arr = {}
        self.in_file_name          = p.in_file_name
        self.out_file_name         = p.out_file_name
        self.out_file_name_bk      = p.out_file_name + "_bk"
        drr_directory              = p.SGRUN  + "/DRxR/" + drr_id + '/'
        self.drr_start_mc_op       = drr_directory + '.mc_' + pkg_name + '_start_op'
        self.drr_cleanup_mc        = drr_directory + '.mc_' + pkg_name + '_cleanup_done'
        self.vmfs_attach_file      = drr_directory + '.mc_' + pkg_name + '_roll_attach'
        self.vmfs_detach_file      = drr_directory + '.mc_' + pkg_name + '_roll_detach'

    # EOF __init__()


    ###
    # Function to roll back steps in-case of a failure
    ###
    def _roll_back_(self):

        self.ret_error = "NA"

        ## Check if input file exists
        if self._check_file_exist_(self.out_file_name_bk):
            # Use backup file to complete the remaining cleanup
            self.out_file_name = self.out_file_name_bk

        if self._read_output_file():
            return 210

        #print self.rehearsal_array
        try:
            # We can get the package type from 1st element of the array
            self.pkg_type = self.rehearsal_array[0]['PKG_TYPE']
        except KeyError:
            _, e, _ = sys.exc_info()
            self.ret_error = "Unable to get the package type"
            return 211

        RC = 0
        return_code = 0

        ## Condition for different package types
        #  Condition for 3PAR
        if self.pkg_type == "MC3PAR":

            if not return_code :
                if RC:
                    return_code = RC

            # Import MC3PAR
            from drr3par import MC3PAR
            mc_3par = MC3PAR(pkg_name, drr_id, self.rehearsal_array, "", self.DEBUG, self.error_file)

            # Iterate through rehearsal_array 
            for reh_Array in self.rehearsal_array:

                # Default roll_back status will be "NA". No need to roll_back 
                # the steps for the status "passed"
                self._DEBUG_("reh_Array Before: ", reh_Array)
                steps_completed = reh_Array['STEPS_COMPLETED']
                if (steps_completed == '1'):
                        # Don't do anything for this steps. Just mark the status
                        # as pass.
                        reh_Array['roll_back']= "passed"

                elif (reh_Array['roll_back'] != 'passed'):
                    mc_3par.s_cli_ssh = reh_Array['ARRAY_CON']
                    snap_vv_name = reh_Array['SV_NAME']

                    if (steps_completed == '2'):
                        # 1. Remove snapshot vv
                        if mc_3par._remove_snapshot_vv_(snap_vv_name):
                            reh_Array['roll_back']= "failed"
                            self.ret_error = mc_3par.ret_error
                            RC = 212
                        else:
                            reh_Array['roll_back']= "passed"

                    elif (steps_completed == '3'):
                        # 1. Un-expose snapshot vv
                        # 2. Remove snapshot vv

                        ## 1
                        if mc_3par._remove_vlun_cmd_(reh_Array, "SNAPSHOT"):
                            reh_Array['roll_back']= "failed"
                            self.ret_error = mc_3par.ret_error
                            RC = 213
                        else:
                            ## 2
                            if mc_3par._remove_snapshot_vv_(snap_vv_name):
                                # Snap vv has been exposed, so change the steps 
                                # to 2
                                reh_Array['STEPS_COMPLETED'] = 2
                                reh_Array['roll_back']= "failed"
                                self.ret_error = mc_3par.ret_error
                                RC = 214
                            else:
                                reh_Array['roll_back']= "passed"
                    # This condition is extra as of now, but better to keep this
                    # for future.
                    elif (steps_completed == '4'):
                        # 1. Expose original
                        # 2. Un-expose snapshot vv
                        # 3. Remove snapshot vv

                        ## 1
                        if mc_3par._create_vlun_cmd_(reh_Array, "ORIGINAL"):
                            reh_Array['roll_back']= "failed"
                            self.ret_error = mc_3par.ret_error
                            RC = 215
                        else:
                            ## 2
                            reh_Array['STEPS_COMPLETED'] = 3
                            if mc_3par._remove_vlun_cmd_(reh_Array, "SNAPSHOT"):
                                reh_Array['roll_back']= "failed"
                                self.ret_error = mc_3par.ret_error
                                RC = 216
                            else:
                                ## 2
                                if mc_3par._remove_snapshot_vv_(snap_vv_name):
                                    # Snap vv has been exposed, so change the 
                                    # steps to 2
                                    reh_Array['STEPS_COMPLETED'] = 2
                                    reh_Array['roll_back']= "failed"
                                    self.ret_error = mc_3par.ret_error
                                    RC = 217
                                else:
                                    reh_Array['roll_back']= "passed"
                    else:
                        self.ret_error = "PROBLEM"
                        RC = 218
                # End of IF condition
                self._DEBUG_("reh_Array After: ", reh_Array)
            # End of rehearsal_array for loop

            # Write to the out_file_name_bk
            out_f_write = open(self.out_file_name_bk, 'w')
            for arr_val in self.rehearsal_array:
                for val in arr_val:
                    out_f_write.write(str(val) + '=' + str(arr_val[val]) + '\n')
                out_f_write.write('^^^\n')
            out_f_write.close()

        else:
            self.ret_error = "Not implemented yet for " + self.pkg_type
            RC = 219

        if RC :
            return RC
        else:
            return 0
    # EOF _roll_back_()

    ###
    # Function to populate details for VIRTUAL DLS
    ###
    def _pop_3par_vmware_dls_(self, mc_3par):
        # Get original, snapshot rcvg name
        snap_storage_rcvg_name = mc_3par.s_storage_rcvg_name
        orig_storage_rcvg_name = mc_3par.p_storage_rcvg_name

        # Get original & snapshot cli_ssh connection
        rc1, rc2, mc_3par.s_cli_ssh, mc_3par.p_cli_ssh = mc_3par._get_cli_ssh_()
        if rc1 or rc2:
            self.ret_error = "Unable to get the CLI or SSH details for DC1,DC2"
            return  113
        snap_cli_ssh = mc_3par.s_cli_ssh
        orig_cli_ssh = mc_3par.p_cli_ssh

        # Get rcvg vv wwid array for snapshot array rcvg
        mc_3par._get_rcvg_vv_wwid_(snap_storage_rcvg_name, snap_cli_ssh, 'snap')
        vm_snap_vv_wwid_arr = mc_3par.vm_vv_wwid_array

        # Get rcvg vv wwid array for original array rcvg
        mc_3par._get_rcvg_vv_wwid_(orig_storage_rcvg_name, orig_cli_ssh, 'orig')
        vm_orig_vv_wwid_arr = mc_3par.vm_vv_wwid_array

        # Get original:snapshot volume details from original rcvg
        if mc_3par._get_rcvg_sync_vv_nam_(orig_storage_rcvg_name, orig_cli_ssh):
            self.ret_error = mc_3par.ret_error
            return 1

        self._DEBUG_("vm_snap_vv_wwid_arr: ", vm_snap_vv_wwid_arr)
        self._DEBUG_("vm_orig_vv_wwid_arr: ", vm_orig_vv_wwid_arr)


        if not vm_snap_vv_wwid_arr:
            self.ret_error = "Unable to get vv details at snapshot array."
            return 1
        elif not vm_orig_vv_wwid_arr:
            self.ret_error = "Unable to get vv details at the original array"
            return 1
        else:
            self.vm_snap_vv_wwid_arr = vm_snap_vv_wwid_arr
            self.vm_orig_vv_wwid_arr = vm_orig_vv_wwid_arr

        pv_wwid_old_array  = []
        pv_wwid_old_array  = self.pv_wwid_array
        self.pv_wwid_array = []

        ## 
        for pv_wwid in pv_wwid_old_array:
            self._DEBUG_("pv_wwid_old: ", pv_wwid)
            
            # Get the associated pv_name from the original pkg array
            orig_vv_name = vm_orig_vv_wwid_arr[pv_wwid.upper()]

            # Get snapshot vv name using the orig_pv_name
            snap_vv_name = mc_3par.orig_vv_replicated_vv_array[orig_vv_name]

            # Get snapshot vv wwid using the snapshot vv name
            snap_vv_wwid = vm_snap_vv_wwid_arr[snap_vv_name]

            # Append snapshot vv wwid to the new pv_wwid_array
            self.pv_wwid_array.append(snap_vv_wwid.lower())

            # Append to the self.pri_sec_array_wwid to be used for DLS pkg types
            self.pri_sec_array_wwid[snap_vv_wwid.upper()] = pv_wwid

        self._DEBUG_("pv_wwid_array_new: ", self.pv_wwid_array)
        self._DEBUG_("pri_sec_array_wwid: ", self.pri_sec_array_wwid)

        # Return non-zero value if the newly created pv_wwid_array is empty
        if not self.pv_wwid_array:
            self.ret_error = "Unable to populate the pv wwid"
            return 1

        return 0
    # EOF _pop_3par_vmware_dls_()

    ###
    # Function to write in an error file in-case of a failure.
    ###
    def _get_node_type_(self):
        ## Read the input file to get the node type
        if 'NODE_TYPE:VIRTUAL' in open(self.in_file_name).read():
            self.node_type = 'VIRTUAL'

            if 'VIRT_TYPE:DLS' in open(self.in_file_name).read():
                self.virt_type = 'DLS'

            if 'VIRT_TYPE:SLS' in open(self.in_file_name).read():
                self.virt_type = 'SLS'
    # EOF _get_node_type_()


    ###
    # Function to read input file & populate host related information.
    ###
    def _read_input_file_(self, input_file):
        # Check if input file exists
        if not self._check_file_exist_(input_file):
            error = "Input file " + input_file + " doesn't exist."
            self._write_to_file_(error, 'a')
            return 51

        # Read the input file content in an array
        fh = open(self.in_file_name)
        file_array = [file_array for file_array in fh.readlines() \
                                                          if file_array.strip()]
        file_array = [x.strip() for x in file_array]
        fh.close()
        self._DEBUG_("file_array: ", file_array)

        # Iterate through main host file array to get PV, HBA & Other details
        for file_array_val in file_array:
            file_array_new = file_array_val.split(':')
            if (file_array_new[0] == 'PV'):
                #pv_arr = file_array_new[1].split('$')

                # Append pv name & wwid to the array
                self.pv_wwid_array.append(file_array_new[1].split('$')[1])

            elif(file_array_new[0] == 'HBA'):
                if ('iqn' in file_array_new[1]):
                    self.node_hba_arr.append(file_array_new[1] + ":" + \
                                             file_array_new[2])
                else:
                    self.node_hba_arr.append(file_array_new[1])
            elif(file_array_new[0] == 'PKG_TYPE'):
                self.pkg_type = file_array_new[1]

        self._DEBUG_("pkg_type: ", self.pkg_type)
        self._DEBUG_("pv_wwid_array: ", self.pv_wwid_array)
        self._DEBUG_("node_hba_arr: ", self.node_hba_arr)

        # If pv_wwid_array array is empty then error & exit
        if not self.pv_wwid_array:
            error = "Unable to get the associated pv wwid details for pkg vg."
            self._write_to_file_(error, 'a')
            return 1

        return 0
    # EOF _read_input_file_()

    ###
    # Function to start DRR related activities
    # This function will perform two major steps-
    # STEP:A- To Perform all host related operations to start the rehearsal. 
    # STEP:B- To Perform all storage related operations. 
    # STEP:C- To Perform all host related operations after the storage preparation
    #         is done.
    # STEP:B has more sub steps:-
    # STEP:B.1- To get all details from the array for a given pkg pv wwid's
    # STEP:B.2- To create snapshot of all pvs
    # STEP:B.3- To Expose snapshot virtual volumes
    ###
    def _START_DRR_(self):

        self._DEBUG_("STARTING DRESS REHEARSAL FOR THE PACKAGE : ", pkg_name)
        ## STEP:A- To Perform all host related operations to start the rehearsal.
        # Populate host related information
        self._DEBUG_("Populating host related information for pkg-", pkg_name)
        RC = h._pop_host_details_()
        if RC:
            self._write_to_file_(h.ret_error, 'a')
            return RC

        # Get node type to be use later-on
        self._get_node_type_()

        # Remove the backup file if exist during the start rehearsal_array
        try:
            os.remove(self.out_file_name_bk)
        except OSError:
            pass


        # Populate pkg array
        self.pkg_array = p._get_pkg_details_()

        # Get package id
        self.pkg_id = p._get_pkg_id_(pkg_name);
        self._DEBUG_("pkg_id :", self.pkg_id);
        if (self.pkg_id == -1): return 1
           

        # Get package node name
        pkg_node_name, rcode  = p._get_pkg_node_name_()

        # Read created host input file & populate all information to be use
        RC = self._read_input_file_(p.in_file_name)
        if RC: return RC

        ## STEP: B
        # Populate storage array related information & make storage available
        self._DEBUG_("Populating storage array information for pkg-", pkg_name)

        if (self.pkg_type == "MC3PAR"):

            ## STEP:B.1- To get all details from the array for a given pkg pv wwid's
            ## Import 3PAR 
            from drr3par import MC3PAR
            mc_3par = MC3PAR(pkg_name, drr_id, self.pkg_array, pkg_node_name, self.DEBUG, self.error_file)

            if self.virt_type == 'DLS':
                # Making 3par env variables use for further usage
                RC = mc_3par._get_rcvg_details_()
                if RC: 
                    self._write_to_file_(mc_3par.ret_error, 'a')
                    return RC

                # Get values for vmware dls configurations
                if self._pop_3par_vmware_dls_(mc_3par): 
                    return 1

            ## Call 3par function to populate all required information for the
            #  pv array passed to be used for creating & exposing snapshot vvs
            RC = mc_3par._get_rcvg_vv_details_(self.pv_wwid_array, self.node_hba_arr)
            if RC: 
                self._write_to_file_(mc_3par.ret_error, 'a')
                return RC

            ## create an empty to be used later-on to detect the rehearsal status
            #  and to perform the cleanup accordigly
            open(self.drr_start_mc_op, 'a').close()

            # Take snapshot of all virtual volumes
            count = 0
            for virt_vol_array in mc_3par.rcvg_vv_array:
                # We need to set this value 1st to use while roll_back	
                virt_vol_array['roll_back'] = "NA"

                ## Don't iterate through the array if the roll_back_flag is TRUE
                if self.roll_back_flag: 
                    break

                # Assign default values for the snapshot volume
                virt_vol_array['SV_NAME']    = "NA"
                virt_vol_array['SV_LUN_ID']  = "NA"
                virt_vol_array['SV_HOST']    = "NA"
                virt_vol_array['SV_PORT']    = "NA"
                virt_vol_array['ARRAY_CON']  = mc_3par.s_cli_ssh
                virt_vol_array['PKG_TYPE']   = self.pkg_type
                virt_vol_array['ARRAY_CON_P_Array']  = mc_3par.p_cli_ssh

                # For drr_id is DRxR_1302601300958_150 trimmed_drr_id would
                # be D1302601300958 to eliminate possibility of failure of 
                # snapshot volume creation because of more than 32 char. 
                _sep = '_'
                _temp = drr_id.split(_sep, 1)[1].split(_sep, 1)[0]

                trimmed_drr_id = "D"+_temp 
                snap_vv_name = "drr_" + trimmed_drr_id + "_" + \
                                             self.pkg_id + "_" + str(count)
                count += 1

                # STEP:B.2- To create snapshot of all pvs
                self._DEBUG_("STEP:2 Taking Snapshot for", \
                                                      virt_vol_array['VV_NAME'])
                RC = mc_3par._create_snapshot_(virt_vol_array['VV_NAME'], \
                                          virt_vol_array['HOST'], snap_vv_name)
                if RC:
                    if (RC == 121): 
                        self.ret_code = RC
                        self.ret_error = mc_3par.ret_error
                        self.roll_back_flag = True
                        # Need to mark this steps so that roll back will delete
                        # the snapshot volume
                        #virt_vol_array['STEPS_COMPLETED'] = 2
                    else :
                        self.ret_error = "Unable to take the " + \
                                     "snapshot for " + virt_vol_array['VV_NAME']
                        self.ret_code = RC
                        self.roll_back_flag = True
                else : 
                    virt_vol_array['STEPS_COMPLETED'] = 2
                    #Get SNAPSHOT vv wwid from array
                    RC = mc_3par._get_vv_wwid_(virt_vol_array['ARRAY_CON'], \
                                                                  snap_vv_name)
                    if RC: 
                        self.ret_code = RC
                        self.ret_error = mc_3par.ret_error
                        self.roll_back_flag = True
                    virt_vol_array['SV_WWID'] = mc_3par.sv_wwid

                    ## If we have a virtual machine then also append values to
                    #  be used for SLS/DLS RDM & VMFS.
                    if (self.node_type == 'VIRTUAL'):
                        orig_pv_wwid = virt_vol_array['VV_WWID']
                        # Need to populate array[pri_node_wwid] = snap_wwid
                        if self.virt_type == 'DLS':
                            primary_wwid = self.pri_sec_array_wwid[orig_pv_wwid]
                            self.vmfs_orig_snap_pv_arr[primary_wwid]= \
                                                                 mc_3par.sv_wwid
                        else:
                            self.vmfs_orig_snap_pv_arr[orig_pv_wwid]= \
                                                                 mc_3par.sv_wwid

                # Add snapshot vv name to the array
                virt_vol_array['SV_NAME'] = snap_vv_name

            ## STEP:B.3- To Expose snapshot virtual volumes
            ## Expose all snapshot virtual volumes to the host
            for virt_vol_array in mc_3par.rcvg_vv_array:
                ## Don't iterate through the array if the roll_back_flag is TRUE
                if self.roll_back_flag: 
                    break

                self._DEBUG_("STEP:3 Exposing snapshot volume", snap_vv_name +\
                                          "to the host-",virt_vol_array['HOST'])

                RC = mc_3par._create_vlun_cmd_(virt_vol_array, "SNAPSHOT")
                if RC:
                    self.ret_code = RC
                    self.ret_error = mc_3par.ret_error
                    self.roll_back_flag = True
                    break
                else : virt_vol_array['STEPS_COMPLETED'] = 3

                # Get the snapshot volume details from array
                if mc_3par._show_vlun_\
                                    (virt_vol_array, virt_vol_array['SV_NAME']):
                    self.ret_error = "Unable to get the active "+ \
                                     "vlun details for the snapshot vv- "   + \
                                     virt_vol_array['SV_NAME']
                    self.ret_code = RC
                    self.roll_back_flag = True

            ## Note:- Will keep this code as of now to be use in future if req.
            # Un-expose all original virtual volumes from the host
            #for virt_vol_array in mc_3par.rcvg_vv_array:
            #    self._DEBUG_("STEP:4 Un-exposing orig volume" + \
            #                       virt_vol_array['VV_NAME'] + \
            #                       "to the host-",virt_vol_array['HOST'])
            #    # Remove/un-export the original disk from the array to the host
            #    RC = mc_3par._remove_vlun_cmd_(virt_vol_array, "ORIGINAL")
            #    if RC: return RC
            #    else : virt_vol_array['STEPS_COMPLETED'] = 4

            self._DEBUG_("mc_3par.rcvg_vv_array: ", mc_3par.rcvg_vv_array)

        elif (self.pkg_type == "MCXP"):
            pass
        elif (self.pkg_type == "MCEVA"):
            pass
        elif (self.pkg_type == "MCSRDF"):
            pass

        # Write to the out_file_name
        out_f_write = open(self.out_file_name, 'w')
        for arr_val in  mc_3par.rcvg_vv_array:
            for val in arr_val:
                out_f_write.write(str(val) + '=' + str(arr_val[val]) + '\n')
            out_f_write.write('^^^\n')
        out_f_write.close()

        ## change the file permissions
        cmd = "chmod 644 " + self.out_file_name
        output, p_status, err = self._cmd_exec_(name = cmd, value = " ")

        ## STEP:C- To Perform all host related operations after the storage preparation
        #          is done.
        ## Check & roll_back as the storage configurations has been changed
        if self.roll_back_flag:
            self._DEBUG_("ret_error", self.ret_error)
            self._write_to_file_(self.ret_error, 'a')

            ## Call roll_back function
            self._roll_back_()

            # _roll_back_ will re-initialize self.ret_error
            if (self.ret_error != "NA"):
                self._DEBUG_("ret_error: ", self.ret_error)
                self._write_to_file_(self.ret_error, 'a')
            return self.ret_code

        ## STEP:C Sub-steps
        # For physical serers we have to call the _rescan_at_host()
        # For SLS RDM we have to -
        #          i. Rescan at the esx/vcenter
        #          ii. Detach the Original RDM disks
        #          iii. Attach the RDM disks
        ## 1. For physical servers
        if (self.node_type == 'PHYSICAL'):
            RC = h._rescan_at_host_('PHYSICAL')
            if RC:
                self._write_to_file_(h.ret_error, 'a')
                return RC

        ## 2. For Virtual servers
        else:
            # If node is of SLS RDM type
            if self.virt_type == 'SLS':
                # Attach snapshot vmdk's to the host
                RC = h._sls_rdm_setup_('START', self.vmfs_orig_snap_pv_arr)
                if RC:
                    # Remove storage
                    self._roll_back_()
                    return RC

            # If node is of DLS type
            else:
                RC = h._dls_vmfs_rdm_setup_('START', self.vmfs_orig_snap_pv_arr)
                if RC:
                    # If host roll_back_flag is True then remove storage
                    self._roll_back_()
                    return RC

            # Peform rescan
            RC = h._rescan_at_host_('VIRTUAL')
            if RC:
                return RC

        self._DEBUG_("DRESS REHEARSAL FOR THE PACKAGE : ", pkg_name, " IS SUCCESSFUL.")
        return 0
    # EOF _START_DRR_()


    ###
    # _STOP_DRR_()
    # 1.  If node type is physical-
    # 1.1 Do cleanup
    # 1.2 Call Storage Roll backup
    #
    # 2   If node type is virtual sls-
    # 2.1 Do sls rdm cleanup
    # 2.2 Call storage roll back
    # 2.3 Call esx/vcenter rescan
    ###
    def _STOP_DRR_(self):

        self._DEBUG_("STOPPING DRESS REHEARSAL FOR THE PACKAGE : ", pkg_name)

        ## In-case we are calling the STOP multiple times & rehearsal cleanup
        #  was already done, then will just return the success
        if self._check_file_exist_(self.drr_cleanup_mc):
            return 0

        ## If the input file exists then only perform any operation otherwise 
        #  just return success.
        if not self._check_file_exist_(self.in_file_name):
            return 0

        ## Get node type to use the correct step to do cleanup
        self._get_node_type_()

        ## Check if the output file exists, if it doesn't exist then check for 
        #  existance of the drr_start_mc_op & return failure accordigly.
        #  If it exists then verify the array connectivity & ESX/Vcenter connectivity
        #  for the vm's.
        if not self._check_file_exist_(self.out_file_name):
            # Check if drr_start_mc_op file exist
            if self._check_file_exist_(self.drr_start_mc_op):
                err_msg = "Some or all of the snapshot virtual volumes created "+\
                          "for rehearsing the package "+ pkg_name +" could not be removed."
                self._write_to_file_(err_msg, 'a')

                ## Get to be deleted rehearsal matching volume names.
                # Get package id
                self.pkg_id = p._get_pkg_id_(pkg_name);
                self._DEBUG_("pkg_id :", self.pkg_id);
                if (self.pkg_id == -1): return 1
                _sep = '_'
                _temp = drr_id.split(_sep, 1)[1].split(_sep, 1)[0]
                trimmed_drr_id = "D" + _temp 
                snap_vv_name = "drr_" + trimmed_drr_id + "_" + \
                                             self.pkg_id + "_*"
                err_msg = "Use the storage vendor recommended methods to clean-up"+\
                  " all the snapshot virtual volumes matching the string " + snap_vv_name
                self._write_to_file_(err_msg, 'a')

                ## Create the file to not call the STOP again.
                open(self.drr_cleanup_mc, 'a').close()
                return 2
            else:
                err_msg = "Rehearsal failed before creating any snapshot virtual" +\
                          " volumes"
                self._write_to_file_(err_msg, 'a')
                return 0
        else:
            ## Verify Array connection for all types of servers
            if self._get_array_from_output_file_():
                self._write_to_file_(self.ret_error, 'a')
                return 4
               
            ## Verify ESX or Vcenter connectivity only for VM's
            if self.node_type != 'PHYSICAL':
                if h._verify_esx_vcenter_conn_():
                    return 5

        ## 1.  If node type is physical-
        if self.node_type == 'PHYSICAL':
        # 1.1 Do cleanup
            if h._cleanup_():
                self._write_to_file_(h.ret_error, 'a')
                return 6

         # 1.2 Call Storage Roll backup
            # Call roll back to undo all storage parts
            if self._roll_back_():
                self._write_to_file_(self.ret_error, 'a')
                return 6

            if h._rescan_scsi_():
                self._write_to_file_(h.ret_error, 'a')
                return 6
            
        ## 2 If node type is virtual sls-
        #    Compare the values to exactly detect & continue with the node type
        if self.node_type == 'VIRTUAL':
            ## Call SLS DLS cleanup only if the file was created
            if self._check_file_exist_(self.vmfs_attach_file) or \
                                 self._check_file_exist_(self.vmfs_detach_file):
            # 2.1 Do sls rdm cleanup
                if self.virt_type == 'SLS':
                    if h._sls_rdm_cleanup_('STOP'):
                        return 7

                # Do DLS cleanup
                else:
                    if h._dls_cleanup_('STOP'):
                        return 7

            # 2.2 Call storage roll back
            # Call roll back to undo all storage parts
            if self._roll_back_():
                self._write_to_file_(self.ret_error, 'a')
                return 7
            else:
                if self._roll_back_():
                    self._write_to_file_(h.ret_error, 'a')
                    return 7

            # 2.3 Call esx/vcenter rescan
            if self.virt_type == 'SLS':
                if h._sls_rdm_cleanup_rescan_():
                    self._write_to_file_(h.ret_error, 'a')
                    return 7

        ## create an empty file determine the rehearsal cleanup status.
        open(self.drr_cleanup_mc, 'a').close()

        self._DEBUG_("SUCCESSFULLY  STOPPED DRESS REHEARSAL FOR THE PACKAGE : ", pkg_name)
        return 0
    #_STOP_DRR_()


    ###
    # Function to call _get_pv_details_pkg_node_() from the host util to 
    # populate the pv related information for the DLS RDM
    ###
    def _DLS_RDM_PV_INFO_(self):
        ## Call _get_pv_details_pkg_node_() from host util script
        if h._dls_rdm_pv_info_():
            self._write_to_file_(h.ret_error, 'a')
            return 11
        return 0
    # EOF _DLS_RDM_PV_INFO_()

# EOC STORAGE_ARRAY()


################################################################################
#######################---------------MAIN---------------#######################
################################################################################

def main(argv=None):

    if "-h" in argv or "--help" in argv:
        print(__doc__)
        return 0
    
    global p, pkg_name, h, drr_id, DEBUG, err_file, node_name
    pkg_name  = sys.argv[2]
    drr_id    = sys.argv[3]
    DEBUG     = sys.argv[4]
    err_file  = sys.argv[5]
    if (len(sys.argv) == 7):
        node_name = sys.argv[6]
    else:
        node_name = "NA"

    if (DEBUG != '0') and (DEBUG != '1') and (DEBUG != '2'):
        print("ERROR: Debug mode " + DEBUG + " not supported")
        return 1

    # Create object for package, host & storage_array.
    p = Package(pkg_name, drr_id, int(DEBUG), err_file)
    h = HOST(pkg_name, drr_id, int(DEBUG), node_name, err_file)
    arr = STORAGE_ARRAY(pkg_name, drr_id, DEBUG, node_name, err_file)
    
    ## To start the rehearsal
    if sys.argv[1] == "START" :
        RC = arr._START_DRR_()

    ## To stop the rehearsal
    elif (sys.argv[1] == "STOP") :
        RC = arr._STOP_DRR_()

    ## To get information about PV at the package running node.
    elif (sys.argv[1] == "DLS_RDM"):
        RC = arr._DLS_RDM_PV_INFO_()

    if RC: 
        return RC
    else: 
        return 0

if __name__ == '__main__':
    sys.exit(main(sys.argv))

