#######################################################################
#!/usr/bin/perl
# Easy deployment code for Metrocluster 3PAR RC 
# 
# (C) Copyright 2016-2017 Hewlett Packard Enterprise Development Company, L.P.
# (C) Copyright 2013-2015 Hewlett-Packard Development Company, L.P.
# @(#) Metrocluster XPCA Module  Script.
# @(#) Product Name                :  HP Metrocluster Addons for Linux
# @(#) Product Version             :  B.12.10.00
# @(#) Patch Name                  :  
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

package MC3PAR_ED_linux;    # The module for 3PARRC parameters Easy Deployment
use warnings;
use strict;

our $error     = 0;     # Package globals
our $error_map = {};    # An hash to store the error messages indexed by $error
####################### globals for this package block ########################

my $par_param_map = {};    # An empty hash ref to hold 3PARRC params & values
our $rc_volume_group_map = {};    # An empty hash ref to hold device group params alone

my $par_found = 0;
my @properly_named_disks;
my @par_disks;
my @par_wwn;
my $global_domain;
my (@output);

my $exit_status;
my (@replicated_disks);
my (@non_replicated_disks);
my @instance_nos;
my @disk;
my $CLICMD = "/usr/bin/cli";
my $SGCONF;
my $SGBIN;

my (@wwn_names);
my (@disk_names);
my $system;
my $pwf_dir;
my $user;
my $hostname="";
my (@values);
my (@hosts);
my $rcvg;
my $vol;
my $global_role;
my $global_rcvg2;
my $global_target2;
my $global_hosts2;
my $global_user2;
my $global_system2;
my $global_rcvg1;
my $global_target1;
my $global_hosts1; 
my $global_user1;
my $global_system1;
my $global_systemid;
my @global_output;
my $target;
my @local_nodes;
my @remote_nodes;
my $remote_vol;

BEGIN
{
$ENV{HOME} = "/root";
if ( -e "/etc/redhat-release" )
{
$SGCONF = "/usr/local/cmcluster/conf";
$SGBIN  = "/usr/local/cmcluster/bin";
}
else
{
$SGCONF = "/opt/cmcluster/conf";
$SGBIN  = "/opt/cmcluster/bin";
}

}

################# End of Globals #####################################

#######################################################################
#
# Function Name: do_parameter_discovery
# Input Parameters: An array of names of physical disks
# Return value: A hash reference that has all params and values
# Interface type: Public
# Description: This routine is called by the module consumers to fillin the
#        parameter->value hash map and return it
######################################################################

sub do_parameter_discovery {
    (@wwn_names) = @_;
    my $ret_val;
    $error = 0;    #set error to 0
    @replicated_disks = ();
    @non_replicated_disks = ();

    insert_into_map_default_parameters();
    $ret_val = do_discovery(@wwn_names);
    if ( $ret_val eq -1 ) {
        return $par_param_map;    # returning empty hash
    }
    
    if( $#non_replicated_disks == $#wwn_names ) {
    $error  = 10;
    $error_map->{$error} = " No disks are replicated";
    }

    return ($par_param_map);

}

#################### End of Routine ##################################################


#######################################################################################
#
# Function Name: trim
# Input parameter: the string the needs to be trimmed
# Return value: the trimmed string
# Description: This function removes the leading and trailing whitespaces from the
#        string. whitespace implies [:space:] as defined by POSIX
#######################################################################################

sub trim {
    my $string = shift;
    $string =~ s/^\s+|\s+$//g;
    return $string;
}

################ End of routine #########################################################


#######################################################################################
#
# Function Name: get_user_and_system
# Input parameter: the password file name with full path
# Return value: user and system for 3PAR array
# Description: This function parses the user ans system
# 
#######################################################################################

sub get_user_and_system {
    my $filename=$_[0];
            chomp($filename);
            my $file = $filename;
                my $dashCharIndex = index( $filename, "_" );
                my $dotCharIndex = index( $filename, "." );
                my $pwfIndex = index( $filename, ".pwf");
                my $pwfPath = substr ($filename, 0 , $pwfIndex);
            my $pwf_dir = "$SGCONF/3parrc/";
            
            my $tempIndex = index($pwfPath, $pwf_dir);
            my $len=length($SGCONF)+8;
            my $tempStr = substr ($pwfPath, $tempIndex+$len);
            $filename =~ s/$pwf_dir//;
            $filename = $file;
                $system = substr( $pwfPath, $dashCharIndex + 1 );
            $dashCharIndex = index( $tempStr, "_");
            $user = substr ( $tempStr, 0, $dashCharIndex );
            $hostname = `/bin/hostname`;
            chomp($hostname);
            my $hostIndex = index($hostname , "." );
            my $host;
            if ( $hostIndex > 0 )
            {
            $host = substr($hostname, 0, $hostIndex);
            $hostname = $host;
            }
        }

################ End of routine #########################################################
        
#######################################################################################
#
# Function Name: execute_cli_command 
# Input parameter: the password file name with full path and CLI command string with options
# Return value: output of CLI command
# Description: This function gets the user and system details and 
#         connects to the cli and runs the command and returns the output. 
# 
#######################################################################################
    
sub execute_cli_command {    
            my $filename=$_[0];
            my $cmd = $_[1];
            my $rh5 = 0;
            my $cliFlag=0;
            my $sshFlag=0;
            chomp($filename);
            get_user_and_system($filename);
            if ( -e "/etc/redhat-release" ) {
                my $os       = `/bin/cat "/etc/redhat-release"`;
                my $dotIndex = index( $os, "." );
                my $name     = substr( $os, 0, $dotIndex );
                my $ver = substr( $name, $dotIndex - 1, $dotIndex );
                if ( $ver == 5 ) {
                    $rh5     = 1;
                }
            }
            if (! (-e "/usr/bin/cli" ))     
            {       
                if ($rh5 == 1) {
                    $error=3;
                    $error_map->{$error} = "CLI binary doesnt exist at /usr/bin/cli. Create a symbolic link /usr/bin/cli to the installed Inform OS CLI binary.";
                    return -1;
                }
                my $ssh_cmd="/usr/bin/ssh -l $user $system showversion -s";     
                my $ssh_op=`$ssh_cmd`;      
                my $ssh_ret=$?;     
                chomp $ssh_op;
                if ($ssh_ret != 0) {        
                    $error=3;       
                    $error_map->{$error} = "CLI binary doesnt exist at /usr/bin/cli. Create a symbolic link /usr/bin/cli to the installed Inform OS CLI binary.";        
                    $error_map->{$error} = "Also passwordless ssh is not set for $user to $system. If the InformOS version is 3.3.1 or higher, please set the passwordless ssh using the $SGBIN/3parrc/setupMC3PARsetsshkeys utility. The /usr/bin/cli need not be created if the InformOS version is 3.3.1 or higher.";        
                    return -1;      
                } 
                elsif ($ssh_op ge "3.3.1"){       
                     $sshFlag=1;      
                } 
                elsif ($ssh_op lt "3.3.1"){        
                    $error=3;       
                    $error_map->{$error} = "CLI binary doesnt exist at /usr/bin/cli. Create a symbolic link /usr/bin/cli to the installed Inform OS CLI binary.";        
                    return -1;      
                }       

            } 
            else {        
                my $cli_cmd="$CLICMD -sys $system -sockssl -pwf $filename showversion -s";      
                my $cli_op=`$cli_cmd`;      
                my $cli_ret=$?;     
                chomp $cli_op;      
                if ( $cli_ret != 0 ){  
                    if ( $rh5 == 1 ) {
                        $error=4;       
                        $error_map->{$error} = "CLI connection failed. Check for the existence of password file under ${SGCONF}/3parrc directory";      
                        return -1; 
                    }     
                    my $ssh_cmd="/usr/bin/ssh -l $user $system showversion -s";     
                    my $ssh_op=`$ssh_cmd`;      
                    my $ssh_ret=$?;     
                    chomp $ssh_op;      
                    if (($ssh_op ge "3.3.1") && ($ssh_ret == 0)) {      
                        $sshFlag=1;      
                    } 
                    else{     
                        $error=4;       
                        $error_map->{$error} = "CLI connection failed. Check for the existence of password file under ${SGCONF}/3parrc directory";      
                        return -1;      
                    }
                }
                else {
                    if (($cli_op ge "3.3.1") && ($rh5 != 1))  {      
                        my $ssh_cmd="/usr/bin/ssh -l $user $system showversion -s";     
                        my $ssh_op=`$ssh_cmd`;      
                        my $ssh_ret=$?;     
                        if ($ssh_ret == 0){     
                            $sshFlag=1;      
                        } 
                        else {     
                            $error_map->{$error} = "Passwordless ssh is not set for $user to $system. If the InformOS version is 3.3.1 or higher, please set the passwordless ssh using the $SGBIN/3parrc/setupMC3PARsetsshkeys utility. The /usr/bin/cli need not be created if the InformOS version is 3.3.1 or higher.";        
                            return -1;      
                        }
                    }                  
                    else {     
                        #Setting cli flag to one if OS is RH5 or if InformOS version is lesser than 3.3.1
                        $cliFlag=1;       
                    }       
                }    
            }       
            if ($cliFlag == 1) {
                my @cmd_output =`$CLICMD -sys $system -sockssl -pwf $filename $cmd 2>&1`;
                if ($#cmd_output > 0)
                {
                    my $first_line = $cmd_output[0];
                    #check for listed error message in the output
                    if ( index($first_line, "Could not open connection to CLI server process") >=0)
                    {
                        #add error
                                        $error=4;
                                        $error_map->{$error} = "CLI connection failed. Check for the existence of password file under ${SGCONF}/3parrc directory";

                        print "CLI connection failed. Check for the existence of password file under ${SGCONF}/3parrc directory";
                        return -1;
                    }
                    if ( index($first_line, "Too many") >=0)
                    {
                        #add error
                                        $error=5;
                                        $error_map->{$error} = "CLI connection failed as CLI connections are exhausted.";
                        print "CLI connection failed as CLI connections are exhausted";
                        return -1;
                    }
                    if ( index($first_line, "Badly formatted status response") >=0)
                    {
                        #add error
                                        $error=6;
                                        $error_map->{$error} = "CLI connection failed as CLI connections are exhausted.";
                        print "CLI connection failed as CLI connections are exhausted";
                        return -1;
                    }
                    @global_output=@cmd_output;
                    return 0;
                }
            }   
            elsif ($sshFlag == 1){     
                my @cmd_output =`/usr/bin/ssh -l $user $system $cmd 2>&1`;
                if ($#cmd_output > 0)       
                {       
                    my $first_line = $cmd_output[0];        
                    #check for listed error message in the output       
                       
                    if ( index($first_line, "Too many") >=0)        
                    {       
                        #add error      
                        $error=5;       
                        $error_map->{$error} = "SSH connection failed as SSH connections are exhausted.";       
                        print "SSH connection failed as SSH connections are exhausted";     
                        return -1;      
                    }       
                    if ( index($first_line, "Badly formatted status response") >=0)     
                    {       
                        #add error      
                        $error=6;       
                        $error_map->{$error} = "SSH connection failed as SSH connections are exhausted.";       
                        print "SSH connection failed as SSH connections are exhausted";     
                        return -1;      
                    }       
                    @global_output=@cmd_output;     
                    return 0;       
                }       
            }
}

sub get_module_version {
        my $final = 2;
        my $module = $_[0];
        my @output = split(" ", $module);
        my $lscmd = "/bin/ls $SGCONF/modules/$output[1]*"; # if linux modules will be in ${SGCONF}/modules/ 
        my $var = "$SGCONF/modules/$output[1]";
        my @cmd = `$lscmd`;
        foreach (@cmd) {
                my $value = $_;
        chomp($value);
                if ( length($value) > length($var))
                {
                my $test = substr ($value, length($var)+1);
                if ( $test == "1" || $test == "2" )
                {
                        $final = $test;
                }
                else
                {
                        $final = 2;
                }
}
}
return $final;
}

#######################################################################################
#
# Function Name: do_discovery
# Input parameter: list of disks with wwns
# Return value: 3PAR module parameters discovered.
# Description: This function does the auto discovery of all 3PAR package 
#         configuration parameters given the disk wwn. 
#         It also checks if the disks are replicated and categorizes them into
#         replication disks and non-replication disks.
#         It finds the nodes that belong to local and remote data center.
# 
#######################################################################################

sub do_discovery {
    my (@wwn_names) = @_;
    my $return_value = -1;
    my @pwf_files = `/bin/ls $SGCONF/3parrc/*.pwf`;
     $error_map->{0}="";
     $error_map->{1}="";
     $error_map->{2}="";
     $error_map->{3}="";
     $error_map->{4}="";
     $error_map->{5}="";
        my $discovery;
        my $count = 0;
    for my $wwn_disk_name (@wwn_names) {
        my($wwn_name,$disk_name) = split(":", $wwn_disk_name);
        $discovery = 0;
        for my $filename (@pwf_files) {
        if ( $discovery eq 1)
        {
            last;
        }
            get_user_and_system($filename);

            @global_output = ();
            my $cmd = " showvv -host $hostname -rcopy -showcols Name,Domain,VV_WWN,RcopyStatus,RcopyGroup -csvtable -listdom 2>&1";
            my $value = execute_cli_command($filename, $cmd);
            if ( $value ne -1)
            {
            my @showvv_output;
            @showvv_output = @global_output;
            for my $vvoutput (@showvv_output) {
            chomp($vvoutput);

                #check for novv listed error message in the output
                if ( index($vvoutput, "no vv") >= 0 )
                {
                #No VV in this host for this array. Try with another array
                last;
                }
                if ( index($vvoutput, "Name,Domain") >= 0  ||
                        index($vvoutput, "total,,") >= 0 ||index($vvoutput,"----------------")>=0 )
                {
                }
                else
                {
                my @values = split (',', $vvoutput);
                if ( $#values <3)
                {
                $error = 1;

                $error_map->{$error} = $error_map->{$error}."Command error in InformOS CLI \n$vvoutput\nMake sure that Virtual Volume is part of a Remote Copy volume group.";
                next;
                }

                else
                {
                $vol = $values[0];
                $global_domain = $values[1];
                my $wwn = $values[2];
                my $role = $values[3];
                $global_role = $role;
                $rcvg = $values[4];
                chomp($rcvg);
                chomp($wwn_name);
                chomp($wwn);

                if ( uc($wwn_name) eq uc($wwn) )
                {
                # WWN matched. Only user needs to be verified.
                #check user and domain here
                if (check_user($filename) eq 0)
                {
                $discovery = 1;
                get_target($filename);
                get_hosts($filename);
                my $idIndex = index($rcvg, $global_systemid );
                my $rcvg2;

                if (  $idIndex > 0 )
                {
                    $rcvg2 = substr ($rcvg, 0, $idIndex);
                    chomp($rcvg2);
                }
                else
                {
                    $rcvg2 = $rcvg.$global_systemid;
                    chomp($rcvg2);
                }
                if ( (lc($role) eq "primary") || (lc($role) eq "primary-rev"))
                {
                $global_rcvg2 = $rcvg2;

                push (@replicated_disks, $disk_name);
                $par_param_map->{'DC1_RC_VOLUME_GROUP'} = $rcvg;
                $par_param_map->{'DC1_STORAGE_SYSTEM_USER'} = $user;
                $par_param_map->{'DC1_STORAGE_SYSTEM_NAME'} = $system;
                $par_param_map->{'DC1_RC_TARGET_FOR_DC2'} = $target;
                $par_param_map->{'DC1_NODE_LIST'} = $global_hosts1;

                my $tempUser = $user;
                my $tempSystem = $system;

                get_remote_vg($filename);
                $par_param_map->{'DC2_RC_VOLUME_GROUP'} = $global_rcvg2;
                $par_param_map->{'DC2_RC_TARGET_FOR_DC1'} = $global_target2;
                $par_param_map->{'DC2_STORAGE_SYSTEM_USER'} = $global_user2;
                $par_param_map->{'DC2_STORAGE_SYSTEM_NAME'} = $global_system2;
                $par_param_map->{'DC2_NODE_LIST'} = $global_hosts2;

                $user = $tempUser;
                $system = $tempSystem;

                $return_value = 0;
                last;
                }
                elsif ( (lc($role) eq "secondary") || (lc($role) eq "secondary-rev"))
                {
                $discovery = 1;
                $global_rcvg1 = $rcvg2;
                push (@replicated_disks, $disk_name);
                $par_param_map->{'DC2_RC_VOLUME_GROUP'} = $rcvg;
                $par_param_map->{'DC2_STORAGE_SYSTEM_USER'} = $user;
                $par_param_map->{'DC2_STORAGE_SYSTEM_NAME'} = $system;
                $par_param_map->{'DC2_RC_TARGET_FOR_DC1'} = $target;
                $par_param_map->{'DC2_NODE_LIST'} = $global_hosts1;

                my $tempUser = $user;
                                my $tempSystem = $system;

                get_primary_vg($filename);

                $par_param_map->{'DC1_RC_VOLUME_GROUP'} = $global_rcvg1;
                $par_param_map->{'DC1_STORAGE_SYSTEM_USER'} = $global_user1;
                $par_param_map->{'DC1_STORAGE_SYSTEM_NAME'} = $global_system1;
                $par_param_map->{'DC1_RC_TARGET_FOR_DC2'} = $global_target1;
                $par_param_map->{'DC1_NODE_LIST'} = $global_hosts2;

                $return_value = 0;
                last;
                #return $return_value;
                }
                else
                {
                #add error
                                $error=2;
                $error_map->{$error} = $error_map->{$error} ."Disk $disk_name is not replicated. ";
                push(@non_replicated_disks, $disk_name);
                }
                } #if loop for check_user.
                #}
                else
                {
                    #User doesnt have correct privilege. Trying with another password file
                    last;
                }
                } #if loop for wwn matching
                }#else loop for values count > 12
                }#else loop for headers
            }#FOR LOOP FOR vv output
        }
        }#for loop for pwf files
        if ( $discovery eq 0)
        {
        #add error
        $error=2;
        $error_map->{$error} = $error_map->{$error} ."Disk $disk_name is not replicated. ";
        push(@non_replicated_disks, $disk_name);
        }
    }#for loop for wwn names
return $return_value;
}
                         
################ End of routine #########################################################

#############################################################################################
#
# Function Name: get_remote_hosts
# Input Parameter: 
# Return Value: 
# Interface Type: Public
# Description: This rotuine will be called by ED script to get 
#         nodes belonging to the cluster and 
#         categorizes them into local and remote datacenter
#############################################################################################
sub get_remote_hosts {
    my $filename=$_[0];
    $global_hosts2=" ";
    $vol = $remote_vol;
    my $cmd = " showvlun -a -v $vol -csvtable -listdom 2>&1 ";
            @global_output = ();
    my $value = execute_cli_command($filename, $cmd);
    if ( $value ne -1)
    {
        for my $useroutput (@global_output) {
            my $validoutput = index($useroutput, $vol.",");
            if ( $validoutput >= 0 )
            {
            @values = split (',', $useroutput);
            my $host = $values[3];
            if ( index($host, ".") > 0)
            {
            my @temp = split(".", $host);
            $host = $values[0];    
            }
            
            if ( index($global_hosts2, $host) < 0 )
            {
            $global_hosts2 = $global_hosts2." ".$host;
            push(@hosts, $host);
            if ( (lc($global_role) eq "primary") || (lc($global_role) eq "primary-rev"))
            {
                                push(@remote_nodes, $host);
            }
            elsif ( (lc($global_role) eq "secondary") || (lc($global_role) eq "secondary-rev"))
            {
                push(@local_nodes, $host);
            }
            else
            {
                push(@local_nodes, $host);
                push(@remote_nodes, $host);
            }
            }
            }
        }
    }
}
################ End of routine #########################################################

#############################################################################################
#
# Function Name: get_remote_vg
# Input Parameter: password file name
# Return Value: 
# Interface Type: Public
# Description: This rotuine will be called by ED script to get 
#         Remote Copy VG in the remote array
#############################################################################################
sub get_remote_vg {
    my $current_pwf_file = $_[0];
    chomp($current_pwf_file);

    my @pwf_files = `/bin/ls ${SGCONF}/3parrc/*.pwf`;
    for my $file (@pwf_files) {
        chomp($file);
        if ( $current_pwf_file ne $file )
        {
        get_user_and_system($file); #$filename);
        my $cmd = " showrcopy groups $global_rcvg2 -csvtable -listdom 2>&1 ";
            @global_output = ();
        my $value = execute_cli_command($file, $cmd);
        if ( $value ne -1)
        {
            #my $rcvgStr = $rcvg.".";
            my $found=0;

        for my $useroutput (@global_output) {
            if ( index($useroutput, ",RemoteVV") >= 0) {
                            $found = 1;
                            next;
                        }
                        if ( $found == 1 ) {
                if ( index($useroutput, $vol) >=0)
                {
                            my @line_token = split ( ',' , $useroutput ) ;
                $remote_vol=$line_token[1];
                chomp($remote_vol);
                get_remote_hosts($file);
                return;
                }
                else
                {
                    next;
                }
                        }
            if (index($useroutput, $global_rcvg2) >= 0) 
            {
            @values = split (',', $useroutput);
            if ( $#values > 2)
            {
                #$global_rcvg2 = $values[0];
            $global_target2 = $values[1];
            $global_user2 = $user;
            $global_system2=$system;
            }
            }
        }#for loop
        }
    }
    }#for pwf file loop
}
################ End of routine #########################################################

#############################################################################################
#
# Function Name: get_primary_vg
# Input Parameter: password file name
# Return Value: 
# Interface Type: Public
# Description: This rotuine will be called by ED script to get the primary Remote Copy VG
#############################################################################################
sub get_primary_vg {
        my $current_pwf_file = $_[0];
        my @pwf_files = `/bin/ls $SGCONF/3parrc/*.pwf`;
    chomp($current_pwf_file);
        for my $file (@pwf_files) {
        chomp($file);
                if ( $current_pwf_file ne $file )
        {
            #my $rcvgStr = substr ($rcvg, 0, index($rcvg, "."));
        my $cmd = " showrcopy groups $global_rcvg1 -csvtable -listdom 2>&1";
            @global_output = ();
        my $value = execute_cli_command($file, $cmd);
        if ( $value ne -1)
        {
            my $found=0;
                for my $useroutput (@global_output) {
            if ( index($useroutput, "LocalVV") >= 0 ) {
                            $found = 1;
                            next;
                        }
                        if ( $found == 1 ) {
                if ( index($useroutput, $vol) >=0)
                                {
                                my @line_token = split ( ',' , $useroutput ) ;
                                $remote_vol=$line_token[1];
                                chomp($remote_vol);
                                get_remote_hosts($file);
                                return;
                                }
                                else
                                {
                                        next;
                                }
                        }
                        if (index($useroutput, $global_rcvg1) >= 0)
                        {
                        @values = split (',', $useroutput);
            #$global_rcvg1 = $values[0];
                        $global_target1 = $values[1];
            $global_user1 = $user;
            $global_system1=$system;
                        }
                }
        }
    }
        }
}
################ End of routine #########################################################


#############################################################################################
#
# Function Name: get_target
# Input Parameter: password file name
# Return Value: 
# Interface Type: Public
# Description: This rotuine will be called by ED script to get the target value for the 
#         local vv and remote vv
#############################################################################################
sub get_target {
    my $flag = 2;
    my $filename=$_[0];
    my $cmd = " showrcopy groups $rcvg -csvtable -listdom 2>&1 ";
            @global_output = ();
    my $value = execute_cli_command($filename, $cmd);
    if ( $value ne -1)
    {
            for my $useroutput (@global_output) {
                if ( $flag == 0)
                    {
                    my $validoutput = index($useroutput, $vol);
                    if ( $validoutput >= 0)
                    {
                    @values = split (',', $useroutput);
                    my $pvol = $values[1];
                    my $svol = $values[3];
                    $flag = 2;
                    }
                    }
                    elsif ( $flag == 1)
                    {
                    $flag = 0;
                    }
                    else 
                    {
                    my $validoutput = index($useroutput, $rcvg);
                    if ( $validoutput >= 0 )
                    {
                    @values = split (',', $useroutput);
                    $target = $values[1];
                    last;
                    #if ( index($rcvg, '.') >= 0)
                    #{
                    #$flag = 1;
                    #}
                    }
                }
        }
    }
    my $cmd_targetid = " showsys -d -csvtable 2>&1 ";
            @global_output = ();
    $value = execute_cli_command($filename, $cmd_targetid);
    if ( $value ne -1)
    {
        for my $useroutput (@global_output) {

                    my $validoutput = index($useroutput, "System ID");
                    if ( $validoutput >= 0)
                    {
                    @values = split (',', $useroutput);
                    my $str = $values[1];
                    chomp($str);
                    my $dec_str = hex($str);
                    $global_systemid =  ".r".$dec_str;
                    last;
                    #my $id = $values[2];
                    }
                    }
    }
}
################ End of routine #########################################################

#############################################################################################
#
# Function Name: get_hosts
# Input Parameter: password file name
# Return Value: 
# Interface Type: Public
# Description: This rotuine will be called by ED script to get local_nodes and remote_nodes
#         of the datacenters.
#############################################################################################
sub get_hosts {
    my $filename=$_[0];
    $global_hosts1=" ";
    my $cmd = " showvlun -a -v $vol -csvtable -listdom 2>&1 ";
            @global_output = ();
    my $value = execute_cli_command($filename, $cmd);
    if ( $value ne -1)
    {
        for my $useroutput (@global_output) {
            my $validoutput = index($useroutput, $vol.",");
            if ( $validoutput >= 0 )
            {
            @values = split (',', $useroutput);
            my $host = $values[3];
            if ( index($host, ".") > 0)
            {
            my @temp = split(".", $host);
            $host = $values[0];    
            }
            if ( index($global_hosts1, $host) < 0 )
            {     
                $global_hosts1 = $global_hosts1." ".$host;
                push(@hosts, $host);
                
                if ( (lc($global_role) eq "primary") || (lc($global_role) eq "primary-rev"))
                {
                    push(@local_nodes, $host);
                }
                elsif ( (lc($global_role) eq "secondary") ||  (lc($global_role) eq "secondary-rev"))
                {
                                    push(@remote_nodes, $host);
                }
                else
                {
                    push(@local_nodes, $host);
                    push(@remote_nodes, $host);
                }
            }
            }
        }
    }
}
################ End of routine #########################################################


#############################################################################################
#
# Function Name: check_user
# Input Parameter: Password file name
# Return Value: 0 if the user has "Super" or "Edit" privilege 
# Interface Type: Public
# Description: This rotuine will be called by easy deployment script to get the user who has write privilege to the 3PAR vv
#        for the disk name supplied to ED script    
#############################################################################################
sub check_user {
    my $filename=$_[0];
    my $cmd = " showuser $user -csvtable -listdom 2>&1 ";
            @global_output = ();
    my $value = execute_cli_command($filename, $cmd);
    if ( $value ne -1)
    {
    for my $useroutput  (@global_output) {

    my $return = index($useroutput, $user.",");

    if ($return >= 0)
    {
        my $commaCharIndex = index( $useroutput, "," );
        my @values = split (',', $useroutput);
        my $domain = $values[1];
        my $perm = $values[2];

        if ($perm eq "super" || $perm eq "edit" || $perm eq "Super" || $perm eq "Edit")
        {
            #check domain
            if ($domain eq "all" || $domain eq $global_domain)
            {
                #reached a place to discover.
                return 0;
            }
            else
            {
                return -1;
                # go to previous for loop to try for another password file
            }
        }
        else
        {
            return -1;# go to previous for loop to try for another password file    
            #last; # 
        }
    }
    }
}
return -1;
}
################ End of routine #########################################################


#############################################################################################
#
# Function Name: get_replicated_and_non_replicated_disks
# Input Parameter: an array of names of physical disks
# Return Value: four array references;First reference for replicated disks and second for
#        non-replicated disks, third for Local nodes and fourth for remote nodes
# Interface Type: Public
# Description: This rotuine will be called by module consumers to get the list of
#        of replicated and non-replicated disks
#############################################################################################

sub get_replicated_and_non_replicated_disks {
    my (@disk) = @_;
    my $ret_val;
    $error                = 0;     # set error to 0
    @non_replicated_disks = ();    # reset the global array
    @replicated_disks     = ();    # reset the global array

    $ret_val = do_discovery(@disk);

    return ( \@replicated_disks, \@non_replicated_disks, \@local_nodes, \@remote_nodes );

}

############### End of rotuine #########################################################


sub insert_into_map_default_parameters {

    $par_param_map->{'AUTO_NONCURDATA'}                        = 0;
    $par_param_map->{'RESYNC_WAIT_TIMEOUT'}                    = 0;
    $par_param_map->{'PERIODIC_ASYNC_LINK_DOWN_TIMEOUT'}       = 250;
    my $version = get_module_version("module_name dts/3parrc");
    if ($version > 1)
    {
    $par_param_map->{'DR_NOTIFICATION_CHOICES'}       = "SYSLOG" ;
    }
}

###################### End of routine ####################################


1;


