#!/usr/bin/perl
######################################################################
# Easy deployment code for Metrocluster
# (C) Copyright 2016-2017 Hewlett Packard Enterprise Development Company, L.P.
# (C)  Copyright 2013-2015 Hewlett-Packard Development Company, L.P.
# @(#)  Metrocluster Easy Deployment script.
# @(#)  Product Name                :  HP Metrocluster
# @(#)  Product Version             :  B.12.10.00
# @(#)  Patch Name                  :
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HP Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
###################################################################

package  MCEDTools;
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(get_local_remote_nodes  mc_verify_storage);
no strict;

use lib '/opt/drenabler/utils';
use utils;
use MC3PAR_ED_linux;

my @properly_named_disks;
my @nodes;
my @par_disks;
my $par_found = 0;

sub mc_verify_storage {
    @nodes       = @{ $_[0] };
    @localnodes  = @{ $_[1] };
    @remotenodes = @{ $_[2] };
    $opt_vgname  = $_[3];
    $vg_type     = $_[4];
    @tot_pvs     = @{ $_[5] };

    if ( $#nodes == $#remotenodes ) {
        return 0;
    }

}

sub get_local_remote_nodes {
    print
      "Metrocluster:Determining remote nodes and local nodes for the disks \n";
    my @disks = @{ $_[0] };
    @nodes = @{ $_[1] };
    my @split_cols;

    parse_and_store_disks(\@disks);
    my ( $local_nodes, $remote_nodes ) = do_initial_discovery();
my %seen=();
my @unique_local = grep { ! $seen{$_}++ } @$local_nodes;

   %seen=();
my @unique_remote = grep { ! $seen{$_}++ } @$remote_nodes;

	@$local_nodes = @unique_local;
	@$remote_nodes = @unique_remote;

	if ( scalar(@$local_nodes) != scalar(@nodes) ) {
	my $hostname_cmd = '/bin/hostname | awk -F\'.\' \'{print $1}\'';
        my @hostname_arr = `$hostname_cmd`;
        $exit_val = $? >> 8;

        if ( $exit_val == 0 ) {
            chomp $hostname_arr[0];
            my $hostname     = $hostname_arr[0];
            if ( grep { $_ eq $hostname } @$remote_nodes ) {
                print
"Metrocluster: The state of the devices indicate that the node $hostname belongs to the destination site of replication\n";
                print
"Metrocluster: Run the command in one of the following nodes \n";
                $local_nodes_str = join( ' ', @$local_nodes );
                print $local_nodes_str, "\n";
                print "Metrocluster: Exiting...............................\n";
                exit 1;
            }
            else {
                return ( $local_nodes, $remote_nodes );
            }
        }
        else {
            print
"Metrocluster: Failed to get local hostname using /bin/hostname \n";
            print
"Metrocluster: Unable to determine if the current node belongs to the source or destination site of replication \n ";
            print
"Metrocluster: Assuming it as a non Metrocluster environment............\n";
####################sg expects remote_nodes hash has to be empty for recognizing it as normal cluster
            $remote_nodes = [];
            return ( $local_nodes, $remote_nodes );
        }
    }
    else {
####################sg expects remote_nodes hash has to be empty for recognizing it as normal cluster
    print 
"Metrocluster: Assuming it as a non Metrocluster environment............\n";
    $remote_nodes = [];
    return ( $local_nodes, $remote_nodes );
    }

}

##############################################################################
sub parse_and_store_disks {
    my @disks =  @{ $_[0] };
    foreach my $disk (@disks) {
            push( @properly_named_disks, $disk );
            next;
    }
}


sub do_initial_discovery {
    $par_installed = `rpm -qa | grep serviceguard-metrocluster-3parrc`;
    chomp($par_installed);
    my $local_nodes;
    my $remote_nodes;

    if (   ( index( $par_installed, "serviceguard-metrocluster-3parrc" ) >= 0 )
        && ( check_if_3par() != -1 ) )
    {
        ( $local_nodes, $remote_nodes ) = do_3par();
        return ( $local_nodes, $remote_nodes );
    }
    else {
        print "Metrocluster: No Metrocluster product with Easy Deployment support found. Assuming it as a non Metrocluster environment\n";
        return ( \@nodes, \@nodes );
    }
}

sub check_if_3par {
    my_log( "Checking for 3PAR disks", -1 );
    my $ver=0;
    my $rh5=0;
    my $CMCLUSTER="";
    my $SCSI_ID="";
    my $CONF="";
    my $BIN="";
 
   if ( -e "/etc/redhat-release" ) {
        $CMCLUSTER = "/usr/local/cmcluster";
        $SCSI_ID   = "/sbin/scsi_id";
        $CONF      = "/usr/local/cmcluster/conf";
        $BIN       = "/usr/local/cmcluster/bin";
        my $os       = `/bin/cat "/etc/redhat-release"`;
        my $dotIndex = index( $os, "." );
        my $name     = substr( $os, 0, $dotIndex );
        $ver = substr( $name, $dotIndex - 1, $dotIndex );

        if ( -e "/lib/udev/scsi_id" ) {
            my_log( "found SCSI-ID binary", -1 );
            $SCSI_ID="/lib/udev/scsi_id";
        }
        if ( $ver == 5 ) {
            $rh5     = 1;
            $SCSI_ID = "/lib/udev/scsi_id -g -s /block/";
        }
        elsif( -e "/usr/lib/udev/scsi_id" )
            {
               my_log( "found SCSI-ID binary", -1 );
               $SCSI_ID="/usr/lib/udev/scsi_id"; 
            }
         else {
            my_log( "SCSI-ID binary not found", 1 );
            my_log(
"Unable to determine 3PAR disks using disk WWID. Install scsi-id.",
                -1
            );
            return -1;
        }

    }
    else {
        $CMCLUSTER = "/opt/cmcluster";
        $SCSI_ID   = "/lib/udev/scsi_id";
        $CONF      = "/opt/cmcluster/conf";
        $BIN       = "/opt/cmcluster/bin/";

        if ( -e "$SCSI_ID" ) {
            my_log( "found SCSI-ID binary", -1 );
            $SCSI_ID="/lib/udev/scsi_id";
        }
        elsif( -e "/usr/lib/udev/scsi_id" )
            {
               my_log( "found SCSI-ID binary", -1 );
               $SCSI_ID="/usr/lib/udev/scsi_id"; 
            }
        else {
            my_log( "SCSI-ID binary not found", 1 );
            my_log(
"Unable to determine 3PAR disks using disk WWID. Install scsi-id.",
                -1
            );
            return -1;
        }
    }
    foreach my $disk (@properly_named_disks) 
    {
            check_if_multipath( $disk);
            if ( $? == 0 )
            {              #if multipath use dmsetup to get the wwn
                my $cm = "/sbin/dmsetup info $disk | grep UUID | sed \'s/-/ /g\' | awk \'{print \$NF}\'";
                $disk_wwn_pkgvg=uc(substr(`$cm`, 1));
            }
            else
            {
                $dev=`/bin/basename $disk | sed 's/[0-9].*//'`;
                if ( $rh5 == 1)
                {
                    #For non multipath device use scsi_id command after stripping the partition number if any
                    my $cm = "$SCSI_ID$dev";
                    $disk_wwn_pkgvg=uc(substr(`$cm`, 1));
                }
                else
                {
                    $disk="/dev/".$dev;
                    my $cm="$SCSI_ID --whitelisted $disk";
                    $disk_wwn_pkgvg=uc(substr(`$cm`, 1));
                }
            }
            my_log("3parDisk: ".$disk,-1);
            $par_found = 1;
            my_log("3PAR WWN: ".$disk_wwn_pkgvg,-1);
        my $disk_wwn = "$disk_wwn_pkgvg:$disk";
        push(@par_disks, $disk_wwn);
    }

    if ( $#par_disks < 0 ) {
        my_log( "No valid disks found", -1 );
        return -1;
    }
}

sub check_if_multipath {
    my $device = $_[0];

    `echo $device | grep -q -e "/dev/mapper/" -e "/dev/mpath/"`;
    if ( $? == 0 ) {
        return 0;
    }
    else {
        $udev_tmp = `readlink $device`;
        if ( $? == 0 ) {
            `echo $udev_tmp | grep -q "dm-"`;
            if ( $? == 0 ) {
                return 0;
            }
        }
        return 1;
    }
}

#######################################################################
#
# Function Name: do_3par
# Input Parameters: An array of names of physical disks, An array of nodes
# Return value: Two array references for local nodes and remote nodes
# Interface type: Public
# Description: This routine will be called by cmpreparestg for MC support
#
######################################################################

sub do_3par {

    my ( $rep_disks, $non_rep_disks, $local, $remote ) =
      MC3PAR_ED_linux::get_replicated_and_non_replicated_disks(@par_disks);

    @non_rep      = @$non_rep_disks;
    @rep_disk     = @$rep_disks;
    @local_nodes  = @$local;
    @remote_nodes = @$remote;

    $non_rep_size = scalar @non_rep_disks;
    $par_size     = scalar @par_disks;

    if ( $non_rep_size == $par_size ) {
        @remote_nodes = @nodes;
        @local_nodes  = @nodes;
        return ( \@local_nodes, \@remote_nodes );
    }

    if ( $non_rep_size != 0 && $non_rep_size < $par_size ) {
        print "Metrocluster:The following disks are not replicated \n";
        $rep_str = join( ' ', @non_rep_disks );
        print $rep_str, "\n";
        print
          "Metrocluster: Exiting.........................................\n";
        exit 1;
    }

#    if ($MC3PAR_ED_linux::error) {
#        print
#"Metrocluster:  $MC3PAR_ED_linux::error_map->{$MC3PAR_ED_linux::error} \n";
##        print "Metrocluster: Assuming disks are not replicated  \n";
##        @remote_nodes = @nodes;
#        @local_nodes  = @nodes;
#        return ( \@local_nodes, \@remote_nodes );
#    }

    if ($MC3PAR_ED_linux::error) {
        print
"Metrocluster: The following error occurred when determining remote/local nodes \n";
        print
"Metrocluster:  $MC3PAR_ED_linux::error_map->{$MC3PAR_ED_linux::error} \n";
        print "Metrocluster: Assuming disks are not replicated  \n";
        @remote_nodes = @nodes;
        @local_nodes  = @nodes;
        return ( \@local_nodes, \@remote_nodes );
    }
    return ( \@local_nodes, \@remote_nodes );
}

1;

