#!/usr/bin/env python
####################################################################
# (C) Copyright 2017 Hewlett Packard Enterprise Development LP.
# @(#) MC DRR Helper Script.
# @(#) Product Name                :
# @(#) Product Version             : B.12.10.00
# @(#) Patch Name                  :
#
# NOTE:    This script is not configurable!  Any changes made to this
#          script will be overwritten when you upgrade to the next
#          release of Serviceguard metrocluster addons.
#
###################################################################

## All imported modules
from subprocess import PIPE, Popen
from signal import alarm, signal, SIGALRM, SIGKILL
from os import kill
import os.path, platform, sys

###
# Class Helper
###
class Helper(object):
       
    ###
    # INIT
    ###
    def __init__(self, DEBUG, pkg_name, drr_id, error_file):
        self.DEBUG = int(DEBUG)
        self.drr_id           = drr_id
        self.pkg_name         = pkg_name
        self.error_file       = error_file
        self.hostname_wdomain = ""
        self.rehearsal_array  = []
        self.out_file_name    = ""
        self.ret_error        = "NA"
        if "suse" in str(platform.dist()).lower():
            self.SGRUN  = "/opt/cmcluster/run"
            self.SGCONF = "/opt/cmcluster/conf/"
        else:
            self.SGRUN  = "/usr/local/cmcluster/run"
            self.SGCONF = "/usr/local/cmcluster/conf/"
        self.vmfs_out         = 'NA'

    # EOF __init__()


    ###
    # Function to print debug messages when required to debug the code. 
    # Usage : _DEBUG_("messages")
    ###
    def _DEBUG_(self, *args):
        
        if self.DEBUG:
            if self.DEBUG == 1:
                print(' '.join(str(x) for x in args))
            else:
                # Write to the debug file
                DEBUG_File_Name = self.SGRUN + '/DRxR/' + self.drr_id + '/.' + \
                                                 self.pkg_name + '_mc_debug.log'
                dfh = open(DEBUG_File_Name, 'a+')
                print(' '.join(str(x) for x in args))
                for arg in args:
                    dfh.write(str(arg) + ' ')
                dfh.write('\n\n')
                dfh.close()
            sys.stdout.write('\n')
        return 0
    # EOF _DEBUG_()


    ###
    # Function to check & return system os version
    # This is required for RH5 validation, as we need CLI for that.
    ###
    def _get_os_version_(self):
        cmd = "cat"
        value = "/etc/redhat-release | awk -F \"[ .]\" '{print $7}'"
        output, rcode, err = self._cmd_exec_(name = cmd, value = value)
        if output == '5':
            self.os_version = '5'
        else:
            self.os_version = 'NA'
        return 0
    # _get_os_version_()


    ###
    # Function to execute bash command & to get the return code & output
    ###
    def _cmd_exec_(self, **args):
        stdout    = ""
        shell     = True
        kill_tree = True
        env = None
        self._DEBUG_(" __________________Executing commands__________________")
        self._DEBUG_("|______________________________________________________|")
        self._DEBUG_("command name           :", args.get("name"))
        self._DEBUG_("command value          :", args.get("value"))

        timeout=args.get("timeout")
        try:
           timeout
        except NameError:
           pass
        if ( timeout is None ): timeout = 300

        # Build the command to be executed
        args = args.get("name") + ' ' + args.get("value")
        self._DEBUG_("command to be executed :", args)
        self._DEBUG_("command timeout value  :", timeout)

        class Alarm(Exception):
            pass
        def alarm_handler(signum, frame):
            raise Alarm
        proc = Popen(args, shell = shell, cwd = None, stdout = PIPE, stderr = PIPE, env = env, universal_newlines=True)
        if timeout != -1:
            signal(SIGALRM, alarm_handler)
            alarm(timeout)
        try:
            stdout, stderr = proc.communicate()
            if timeout != 300:
                alarm(0)
        except Alarm:
            pids = [proc.pid]
            if kill_tree:
                pids.extend(self.get_process_children(proc.pid))
            for pid in pids:
                # process might have died before getting to this line
                # so wrap to avoid OSError: no such process
                try: 
                    kill(pid, SIGKILL)
                except OSError:
                    pass
            self._DEBUG_("Command output         :")
            self._DEBUG_(stdout)
            self._DEBUG_("Command exit ret code  :", proc.returncode)
            self._DEBUG_("________________________DONE____________________________")
            return '', -9, ''

        self._DEBUG_("Command exit ret code  :", proc.returncode)
        self._DEBUG_("Command output         :")
        self._DEBUG_(stdout)
        self._DEBUG_("________________________DONE____________________________")
        if proc.returncode == 254 or proc.returncode == 255:
            err_msg = "SSH to HPE 3PAR array failed. This is a known issue with "+\
                      "HPE 3PAR. Refer to troubleshooting section of HPE " +\
                      "Metrocluster 3PAR user guide"
            self._write_to_file_(err_msg, 'a')
        return  stdout.rstrip("\n"), proc.returncode, stderr
    # EOF _cmd_exec_(**cmd)


    ###
    # Function to get process id of the children prorcess
    ###
    def get_process_children(self, pid):
        proc = Popen('ps --no-headers -o pid --ppid %d' % pid, shell = True,
                  stdout = PIPE, stderr = PIPE)
        stdout, stderr = proc.communicate()
        return [int(proc) for proc in stdout.split()]
    # EOF get_process_children()


    ###
    # Function to print all parameters defined
    ###
    def __str__(self):
        str = "Parameters- \n"  + \
               "       Debug Level: %s \n" % self.DEBUG
        return str
    # EOF


    ###
    # Function to check if a file exist
    # Usage : _check_file_exist_(file_name)
    ###
    def _check_file_exist_(self, file_name):
        self._DEBUG_("Inside function _check_file_exist_")
        if (os.path.isfile(file_name)): return True
        else : return False
    # EOF _check_file_exist_(file_name)

    
    ###
    # Function to check if a directory exist
    # Usage : _check_dir_exist_(dir_name)
    ###
    def _check_dir_exist_(self, dir_name):
        if (os.path.exists(dir_name)): return True
        else : return False
    # EOF _check_dir_exist_(dir_name)


    ###
    # Function to get the current hostname without domain
    ###
    def _hostname_wdomain_(self):
        self._DEBUG_("Inside function _hostname_wdomain_")
        value = "${HOSTNAME%%.*}"
        output, rcode, err = Helper._cmd_exec_(self,name= "/bin/echo", \
                                                  value= value, time_out = "5" )
        if not output: 
            return 1
        else:
            self.hostname_wdomain = output
        return 0
    # EOF _hostname_wdomain_()


    ###
    # Function to source a file as an environment variables 
    # Usage : _source_config_file_(file_name)
    # Note: Only a file with variables kind of enteries can be sourced. 
    ###
    def _source_config_file_(self, fileName):
        self._DEBUG_("Inside function _source_config_file_")
        for line in open(fileName):
            li=line.strip()
            if not li.startswith("#"):
                if line.strip():
                    gVariableArray = line.split("=")
                    globals()['%s' % gVariableArray[0].rstrip("\n")] = \
                                                  gVariableArray[1].rstrip("\n")
        if SGCONF : self.SGCONF = SGCONF
        if SGRUN  : self.SGRUN  = SGRUN
        if SGLIB  : self.SGLIB  = SGLIB
        if SGROOT : self.SGROOT = SGROOT
        if SGSBIN : self.SGSBIN = SGSBIN

        return 0
    # EOF _source_config_file_()


    ###
    #_read_output_file()
    ###
    def _read_output_file(self):
        count=0
        fh = open(self.out_file_name, "r")
        fh_array =  [x.strip() for x in fh.readlines()]
        #with open(self.out_file_name, "r") as ins:
        newarr = {}
        for line in fh_array:
        #for line in ins:
            if not '^^^' in line.splitlines():
                val=line.splitlines()[0].split("=")
                key=val[0]
                value=val[1]
                newarr[key]=value
            else:
                self.rehearsal_array.append(newarr)
                newarr = {}
                count+=1
                continue

        if not self.rehearsal_array:
            self.ret_error = "Unable to get the required details in the output" +\
                             " file to do the cleanup"
            return 210
        else:
            #print self.rehearsal_array
            return 0
    # EOF _read_output_file()


    ###
    # Function _get_array_from_output_file_()
    ###
    def _get_array_from_output_file_(self):
        count=0
        fh = open(self.out_file_name, "r")
        fh_array =  [x.strip() for x in fh.readlines()]
        #with open(self.out_file_name, "r") as ins:
        pwf_file  = ""
        array_str = ""
        arr_con_find_flag = False
        newarr = {}
        for line in fh_array:
            if 'ARRAY_CON=' in line:
                val=line.splitlines()[0].split("=")
                array_str = val[1]
                if '.pwf' in array_str:
                    # /usr/bin/cli -sys X -pwf /A/B/Y.pwf
                    pwf_file = array_str.split()[4]
                else:
                    # ARRAY_CON="ssh -l user system"
                    array_str_arr = array_str.split()
                    pwf_file = self.SGCONF + '3parrc/' + array_str_arr[2] + \
                                                 '_' + array_str_arr[3] + '.pwf'

                ## Verify if .pwf file is present & return an error if its not
                #  present as the CLI connection to the array will hang. Also,
                #  .pwf file is must for 3PAR Oracle easy deployment point of 
                #  view, so it should be always present for all 3PAR versions.
                self._DEBUG_(pwf_file)
                if not self._check_file_exist_(pwf_file):
                    self.ret_error = "SG MC configuration has been modified &"+\
                    " it is strongly recommended to execute setupMC3PARpasswdfile" +\
                    " utility if the HPE 3PAR Inform OS version is <= 3.2.2, otherwise"+\
                    " execute setupMC3PARsetsshkeys utility. Refer to the SG MC 3PAR"+\
                    " user guide for more details on these utilities."
                    return 1
                
                ## Execute command to verify the connection status
                cmd   = array_str
                value = "showversion"
                output, rcode, err = self._cmd_exec_(name = cmd, value = value)
                if rcode:
                    self.ret_error = "Unable to connect to the array to perform" +\
                                 " cleanup. Please verify the array connection."
                    return 1
                arr_con_find_flag = True
                break

        if not arr_con_find_flag:
            self.ret_error = "Unable to find the array connection information" +\
                " in the output file. Need manual intervention to do the cleanup."
            return 1
        return 0
    # EOF _get_array_from_output_file_()


    ###
    # Function to write in an error file in-case of a failure.
    ###
    def _write_to_file_(self, msg, file_mode):
        ## create the file handler object to write errors into the error file
        fh = open(self.error_file, file_mode)
        fh.write(self.pkg_name + '|storage|' + msg + '\n')
        fh.close()
    # EOF _write_to_file_()

# EOC Helper()
