#!/usr/bin/env python
####################################################################
# (C) Copyright 2017 Hewlett Packard Enterprise Development LP.
# @(#) MC DRR Host utility script.
# @(#) Product Name                :
# @(#) Product Version             : B.12.10.00
# @(#) Patch Name                  :
#
# NOTE:    This script is not configurable!  Any changes made to this
#          script will be overwritten when you upgrade to the next
#          release of Serviceguard metrocluster addons.
#
###################################################################

## All imported modules
from drrpkgutils import Package
from drrhelper import Helper
import sys
import os

os.environ['PATH'] = "/usr/lib64/qt-3.3/bin:/usr/local/sbin:" + \
                         "/usr/local/bin:/usr/sbin:/usr/bin:" + \
            "/usr/local/cmcluster/bin:/opt/cmcluster/bin:/root/bin:/sbin/:/bin:"
os.environ['HOME'] = "/root/"


###
# CLASS HOST
###
class HOST(Helper):

    ###
    # INIT
    ###
    def __init__(self, pkg_name, drr_id, DEBUG, node_name, error_file):
        Helper.__init__(self, DEBUG, pkg_name, drr_id, error_file)
        self.pv_name = ""
        self.pv_array = []
        self.dev_state = ""
        self.p = Package(pkg_name, drr_id, int(DEBUG), error_file)
        self.pv_wwid_array         = []
        self.node_hba_details      = []
        self.dls_pv_wwid_array     = []
        self.dls_rdm_flag          = False
        self.multipath_enabled     = False
        self.device_wwid_array     = {}
        self.drr_id                = drr_id
        self.node_name             = node_name
        self.roll_back_file        = ''
        self.error_file            = error_file
        self.dls_rdm_file          = self.p.dls_rdm_file
        self.in_file_name          = self.p.in_file_name
        self.out_file_name         = self.p.out_file_name
        self.dls_rdm_pv_wwid       = self.p.dls_rdm_pv_wwid
        self.dls_rdm_pv_info       = self.p.dls_rdm_pv_info
        self.pv_wwid_file_name_in  = self.p.pv_wwid_file_name_in
        self.pv_wwid_file_name_out = self.p.pv_wwid_file_name_out
    # EOF __init__()


    ###
    # Function to check if a device is a multipath device
    ###
    def _check_if_multipath_(self, DEV):
        self._DEBUG_("Inside function _check_if_multipath_")
        if "/dev/mapper/" in DEV or "/dev/mpath/" in DEV:
            self.multipath_enabled = True
            return 0
        else:
            udev_tmp, p_status, err = self._cmd_exec_\
                                                (name = "readlink", value = DEV)
            self._DEBUG_("udev_tmp: ",udev_tmp,"p_status: ",p_status)
            if ( p_status == 0):
                if "dm-" in udev_tmp:
                    return 0
            return 1
    # EOF _check_if_multipath_(DEV)

    ###
    # Function to get the scsi path of a device
    ###
    def _get_scsi_id_path_(self):
        self._DEBUG_("Inside function _get_scsi_id_path_")
        #SCSI_ID command path is changed on RHEL7 & SLES12
        rh5 = 0
        if self._check_file_exist_("/usr/lib/udev/scsi_id"):
            SCSI_ID="/usr/lib/udev/scsi_id"
            #CMCLUSTER=SGROOT
        else:
            if self._check_file_exist_("/etc/redhat-release"):
                #CMCLUSTER="/usr/local/cmcluster"
                SCSI_ID="/sbin/scsi_id"
                value = "/etc/redhat-release|awk -F" + '"[ .]"'+ " '{print $7}'"
                ver,p_status,err= self._cmd_exec_(name= "/bin/cat",value= value)
                if (ver == "5" ):
                    rh5=1
                    SCSI_ID="/lib/udev/scsi_id -g -s /block/"
            else:
                #CMCLUSTER="/opt/cmcluster"
                SCSI_ID="/lib/udev/scsi_id"
        return SCSI_ID, rh5
    # EOF _get_scsi_id_path_()


    ###
    # Function to list all available block devices with there wwid
    ###
    def _pop_dev_name_wwid_(self, file_name):
        cmd = "lsscsi -t | grep -i ' disk ' "
        value = "| awk '{print $NF}' | sed -e /^$/d -e's%/dev/%%'"
        output, p_status, err = self._cmd_exec_(name = cmd, value = value)
        output = output.split("\n")
        pv_handler = open(file_name, 'w')
        for dev in output:
            cmd = 'udevadm info -q all -p'
            value= "/block/" + dev + " | grep by-id/wwn | grep -v DEVLINKS"+\
                          " | awk -F'/' '{print $3}' | awk -F'-0x' '{print $2}'"
            wwid, p_status, err = self._cmd_exec_(name = cmd, value = value)
            pv_handler.write(dev + '=' + wwid + '\n')
            self._DEBUG_("device: ", dev)
            self._DEBUG_("wwid: ", wwid)
        pv_handler.close()
        return 0
		
		
    ###
    # Function to get pv details for a given pv
    ###
    def _get_vg_pv_wwid_details_(self):

        self._DEBUG_("Inside function _get_vg_pv_details_")

        VG_ARRAY     = {}
        VG_NEW_ARRAY = {}

        VG_ARRAY_VAL = filter(lambda x:'SG_VG[' in x, self.p.pkg_array)
        VG_ARRAY     = list(VG_ARRAY_VAL)
        if not VG_ARRAY:
            self.ret_error = "ERROR: Unable to get the VG information. Please"+\
                       " check the FS details in the package configuration file"
            return 1

        self._DEBUG_("VG_ARRAY: ", VG_ARRAY)
        # ['SG_VG[0]', 'SG_VG[1]']
        #self.pv_wwid_array = [] 
        for VG_VAL in VG_ARRAY :
            self._DEBUG_("VG_VAL: ", VG_VAL)

            # Get pv information from vgdisplay
            value= "-v " + self.p.pkg_array[VG_VAL] + " 2> /dev/null | fgrep "+\
                      '"PV Name"' + " | grep -v " + '"unknown device"' +   \
                      " | awk '{print $NF}'"
            output, p_status, err = self._cmd_exec_(name = "vgdisplay",    \
                                                                  value = value)
            list_of_devices = output.splitlines()
            self._DEBUG_("list_of_devices: ", list_of_devices)

            for DEV in list_of_devices : 
                self._DEBUG_("DEV: ", DEV)

                RC = self._check_if_multipath_(DEV)
                if(RC == 0):
                    #if multipath use dmsetup to get the wwn
                    value = DEV + " | grep UUID | sed 's/-/ /g' | "+\
                            "awk '{print $NF}' | awk '{print substr($0,2)}'| "+\
                            "awk '{print toupper($0)}'"
                    disk_wwn_pkgvg, p_status, err = self._cmd_exec_(name = \
                                           "/sbin/dmsetup info ", value = value)
                else:
                    #For non multipath dev, use scsi_id command after stripping 
                    #the partition number if any
                    value = DEV + " | sed 's/[0-9].*//'"
                    disk,p_status,err =self._cmd_exec_(name= "/bin/basename ", \
                                                       value = value)
           
                    SCSI_ID, rh5 = self._get_scsi_id_path_()
                    #print SCSI_ID, rh5
                    #In RH5 scsi_id has different options and also it does not 
                    #work on multipath devices
                    if(rh5 == "1"):
                        value=" -g -s /block/" + disk.rstrip("\n") + " | \
                                            awk '{print substr($0,2)}' | \
                                            awk '{print toupper($0)}'"
                        disk_wwn_pkgvg, p_status, err = self._cmd_exec_(name = \
                                            "/lib/udev/scsi_id ", value = value)
                    else:
                        value = "--whitelisted /dev/"+ disk + \
                                            " | awk '{print substr($0,2)}' | "+\
                                                  "awk '{print toupper($0)}'"
                        disk_wwn_pkgvg, p_status, err = \
                                  self._cmd_exec_(name = SCSI_ID, value = value)

                disk_wwn_pkgvg_value = filter(lambda x:disk_wwn_pkgvg in \
                                                    x, self.pv_wwid_array)
                disk_wwn_pkgvg_array = list(disk_wwn_pkgvg_value)
                if not disk_wwn_pkgvg_array:

                    # append to the pv_wwid_array for the future usage
                    self.pv_wwid_array.append(disk_wwn_pkgvg)

                    # Create associative array with wwid & pv or mpath dev name
                    self.device_wwid_array[disk_wwn_pkgvg] = DEV

                self._DEBUG_("disk_wwn_pkgvg: ", disk_wwn_pkgvg)

        self._DEBUG_("self.pv_wwid_array: ", self.pv_wwid_array)

        if self.pv_wwid_array: 
            return 0
        else: 
            self.ret_error = "Unable to get the package PV WWID details"
            return 80
        #return self.pv_wwid_array
    #EOF _get_vg_pv_wwid_details_()


    ###
    # Function to get the node/host HBA wwid
    # Note: It will work only for the physical nodes
    ###
    def _get_node_hba_wwn_(self, node_name, node_type):
        self._DEBUG_("Inside function _get_node_hba_wwn_")
        ## Execute commands only for the physical servers
        if (node_type == "PHYSICAL"):
            value = "/sys/class/fc_host/host*/port_name " + \
                                                       "| grep ^0x* | cut -c 3-"
            self._DEBUG_("Value: ", value)
            output, r_code, err = self._cmd_exec_(name = "cat", value = value)
            self.node_hba_details = output.upper().split("\n")

        ## Add iscsi adapter details to this array
        value = "/etc/iscsi/initiatorname.iscsi | grep -ve \"^#\" " + \
                                     "| awk -F\"=\" \'{print $2}\' 2> /dev/null"
        output, r_code, err = self._cmd_exec_(name = "cat", value = value)
        if (not r_code) and output:
            self.node_hba_details.append(output)
            
        self._DEBUG_("node_hba_details: ", self.node_hba_details)
        return 0
    # EOF _get_node_hba_wwn_(node_name)

 
    ###
    # Function to get the current node type [PHYSICAL:VIRTUAL]
    # Usage : _node_type_(node_name)
    ###
    def _node_type_(self, node_name):
        self._DEBUG_("Inside function _node_type")
        return_val = " "
        self.p._cache_pkg_cmviewcl_env_()
        value = self.p.pkg_cmviewcl_file + " | grep -i 'node:'" + \
                                                     node_name + "'|virt_tech='"
        output, rcode, err = self._cmd_exec_(name = "cat", value = value)
        output_array = output.split("=")
        if ((output_array[1] == "vmware") or (output_array[1] == "KVM")):
            return_val = "VIRTUAL"
        else :
            return_val = "PHYSICAL"
        return return_val
    # EOF _node_type_(node_name)



    ###
    # Function to get a package node UUID
    ###
    def _get_node_uuid_(self, node_name):
        self._DEBUG_("Inside function _get_node_uuid_.")
        pkg_cmviewcl_file = self.p._tmp_pkg_cmviewcl_file_()
        value = pkg_cmviewcl_file + " | grep -i 'node:'" + node_name +"'|uuid='"
        output, rcode, err = self._cmd_exec_(name = "cat", value = value)
        output_array = output.split("=")
        return output_array[1]
    # EOF _get_node_uuid_(node_name)


    ###
    # Funtion to get the pv name for a given wwid. It will iterate through all
    # the devices available to the host & return the matching PV set name 
    # (pv set, as incase of multipath- more than one pvs can have the same wwid)
    # Usage: _get_pv_name_(wwid)
    ###
    def _get_pv_name_(self, wwid, file_name):
        self._DEBUG_("Inside function _get_pv_name_")
        self._DEBUG_("wwid to get the pv name is: ", wwid)

        lines = [line.rstrip('\n') for line in open(file_name)]
        for line_val in lines:
            pv_name = line_val.split('=')[0]
            pv_wwid = line_val.split('=')[1]

            if (pv_wwid.lower() == wwid.lower()):
               self._DEBUG_("Matching pv_name: ", pv_name)
               self._DEBUG_("Matching pv_wwid: ", pv_wwid)
               self.pv_array.append(pv_name)

        self._DEBUG_("Matching pv_array: ", self.pv_array)
        if not self.pv_array: 
            return 1

        return 0
    # EOF _get_pv_name_()

    ###
    # Function _verify_dls_rdm_vmfs_same_pkg_()- To fail the rehearsal if a pkg
    # has both DLS:VMFS & DLS:RDM enteries. Also it creates a dls_rdm_flag
    # This is with refernce to the SG user guide 3.7.6.3 Supported Configuration
    # Note:- Configuring both DLS based RDM disks and VMFS disks in the same 
    # package is not supported.
    ###
    def _verify_dls_rdm_vmfs_same_pkg_(self):
        self._DEBUG_("Inside function _verify_dls_rdm_vmfs_same_pkg_")

        rdm_search  = "package:" + self.pkg_name + "|.*vmdk|disk_type=RDM"
        value = self.p.pkg_cmviewcl_file + ' | grep \"' + rdm_search + '\" | wc -l'
        rdm_count, p_status, err = self._cmd_exec_(name = "cat", value=value)

        vmfs_search = "package:" + self.pkg_name + "|.*vmdk|disk_type=VMFS"
        value = self.p.pkg_cmviewcl_file + ' | grep \"' + vmfs_search + '\" | wc -l'
        vmfs_count, p_status, err = self._cmd_exec_(name = "cat", value=value)

        self._DEBUG_("rdm_count",rdm_count)
        self._DEBUG_("vmfs_count",vmfs_count)

        ## Fail if both count is greater than 0
        if int(rdm_count) > 0 and int(vmfs_count) > 0:
            self.ret_error = "Package has DLS RDM & DLS VMFS configuration, "+\
                              "is not supported"
            return 1

        ## Create a RDM flag to use for DLS RDM package check
        if int(rdm_count) > 0:
            self.dls_rdm_flag = True

        return 0
    # EOF _verify_dls_rdm_vmfs_same_pkg_()


    ###
    #
    ###
    def _get_device_state_(self, device_name):
        value = "/sys/block/" + device_name + "/device/state"
        self.dev_state, p_status, err = self._cmd_exec_\
                                                   (name = "cat", value = value)
        if p_status:
            return 1
        else:
            return 0
    # EOF _get_device_state_()


    ###
    # Function to just get the pv_wwid details at the node. This will
    # be helpful while working with DLS RDM. Because we need the DLS RDM pv 
    # details at the node. This function will write the enteries into the 
    # dls_rdm_pv_info file
    ###
    def _dls_rdm_pv_info_(self):
        ## Get the package details
        self.p._get_pkg_details_()
        if not self.p.pkg_array:
            self.ret_error = "Unable to popualate the pkg details"
            return 33
            
        ## Get pv wwid details for the pkg
        if self._get_vg_pv_wwid_details_(): 
            self.ret_error = "Unable to get the pv information for the vg"
            return 33

        self._DEBUG_("pv_wwid_array: ", self.pv_wwid_array)
        self._DEBUG_("dls_rdm_pv_info: ", self.dls_rdm_pv_info)

        # Open the file to write inputs
        fw = open(self.dls_rdm_pv_info, 'w')

        ## Iterate through the pv_wwid array & write to the info file        
        for pv_wwid in self.pv_wwid_array:
            self._DEBUG_("pv_wwid: ", pv_wwid)
            fw.write('PV_WWID|wwid:' + pv_wwid + '\n')

        fw.close()
        return 0
    # EOF


    ###
    # Function to execute script at remote node & to get all pv related info.
    ###
    def _get_pv_details_pkg_node_(self):
        CMDO = self.p.SGSBIN + '/cmdo'
        CMCP = self.p.SGSBIN + '/cmcp'
        self._DEBUG_("CMDO: ", CMDO)
        self._DEBUG_("CMCP: ", CMCP)

        # Get hostname without domain
        if self._hostname_wdomain_(): 
            self.ret_error = "Unable to get hostname without domain"
            return 72

        if (self.p._check_if_pkg_exist()): 
            self.ret_error = "Package doesn't exist."
            return 74

        # Get the current host node type
        node_type = self._node_type_(self.hostname_wdomain)

        pkg_node_name, rcode  = self.p._get_pkg_node_name_()
        if rcode:
            self.ret_error = "Unable to get the package node name"
            return 75

        ## Execute python script with DLS_RDM parameter at the current pkg node
        cmd = CMDO + ' -n ' + pkg_node_name
        value = 'python /opt/drenabler/bin/cmdrrprepstorage.py DLS_RDM' + ' ' +\
                self.p.pkg_name + ' ' +  self.drr_id + ' 2 /var/tmp/mc_drr_err'

        self._DEBUG_("cmd: ", cmd)
        self._DEBUG_("value: ", value)

        output, p_status, err = self._cmd_exec_(name = cmd, value = value)
        if p_status:
            self.ret_error = 'Execution of script failed at node-' + \
                                                                  pkg_node_name
            return 1

        ## Copy the file at the rehearsal node
        cmd = CMCP
        value = pkg_node_name + ':' + self.dls_rdm_pv_info + ' ' +\
                self.hostname_wdomain + ':' + self.dls_rdm_pv_info

        self._DEBUG_("cmd: ", cmd)
        self._DEBUG_("value: ", value)

        output, p_status, err = self._cmd_exec_(name = cmd, value = value)
        if p_status:
            self.ret_error = 'Execution of script failed at node-' + \
                                                                  pkg_node_name
            return 1
        return 0
    # EOF _get_pv_details_pkg_node_()


    ###
    # Function to parse the dls_rdm_pv_info file & get the pv wwid into an array
    ###
    def parse_dls_rdm_pv_info(self):
        ## Read the dls_rdm_pv_info file
        file_arr = []
        #with open(self.dls_rdm_pv_info) as fr:
        #    file_arr = fr.readlines()
        #file_arr = [x.strip() for x in file_arr]
        fh = open(self.dls_rdm_pv_info, "r")
        file_arr =  [x.strip() for x in fh.readlines()]

        for line in file_arr:
            if 'PV_WWID' in line:
                line = line.split(':')
                pv_wwid = line[1]
                self._DEBUG_("pv_wwid: ", pv_wwid)
                self.dls_pv_wwid_array.append(pv_wwid)
        
        if not self.dls_pv_wwid_array:
            self.ret_error = "Didn't get any pv wwid from the file"
            return 1

        return 0
    # EOF parse_dls_rdm_pv_info()


    ###
    # Function _pop_host_details_
    # 1. Get the pv wwid at the current node for physical & SLS RDM
    # 2. Call vmfs util if pkg is of virtual type
    # 3. Populate pv_wwid array differntly for PHYSICAL, SLS & DLS config.
    # 4. Write other entries into the input file.
    ###
    def _pop_host_details_(self):
    # 1. Get the pv wwid at the current node for physical & SLS RDM
        # Check & create if DRR directory doesn't exist
        drr_directory = self.p.SGRUN + "/DRxR/" + self.drr_id
        if not os.path.exists(drr_directory):
            os.makedirs(drr_directory)

        # Get hostname without domain
        if self._hostname_wdomain_(): 
            self.ret_error = "Unable to get hostname without domain"
            return 72

        # Get the current host node type
        node_type = self._node_type_(self.hostname_wdomain)

        # Get the host wwpn & iscsi initiatorname into an array
        if self._get_node_hba_wwn_(self.hostname_wdomain, node_type):
            self.ret_error = "Unable to get host wwpn & iscsi detail"
            return 73

        if (self.p._check_if_pkg_exist()): 
            self.ret_error = "Package doesn't exist."
            return 74

        pkg_node_name, rcode  = self.p._get_pkg_node_name_()
        if rcode:
            self.ret_error = "Unable to get the package node name"
            return 75

        if (self.hostname_wdomain == pkg_node_name):
            self.ret_error = "Current host name & pkg node name are same"
            return 76

        if (self.node_name != "NA"):
            if (pkg_node_name == self.node_name):
                self.ret_error = "Given DRR node name & pkg node name are same"
                return 77

        self._DEBUG_("pkg_node_name: ", pkg_node_name)

        # Get File system informations if exist, it will populate pkg_array
        self.p._get_pkg_details_()
        if not self.p.pkg_array:
            self.ret_error = "Unable to get the package details"
            return 78

        if self.p._get_pkg_type_():
            self.ret_error = "Unable to get the package MC module name"
            return 79

        ## Return if a DLS pkg has both VMFS & RDM parameters
        if self._verify_dls_rdm_vmfs_same_pkg_(): return 1

        ## Get the pv details from the pkg running node
        if self.dls_rdm_flag:
            ## create an empty file determine this pkg as dls rdm everywhere easily
            open(self.dls_rdm_file, 'a').close()

            # call the function to execute the commands
            if self._get_pv_details_pkg_node_():
                return 1
            
            # Verify if the self.dls_rdm_pv_info file exists
            if not self._check_file_exist_(self.dls_rdm_pv_info):
                self.ret_error = "Unable to find file " + self.dls_rdm_pv_info
                return 1

            # Read the file & populate pv_wwid_array to be used later on
            if self.parse_dls_rdm_pv_info():
                return 1

        VG_count = self.p._pkg_val_occurances_count_("SG_VG[")
        if not VG_count:
            self.ret_error = "DRR is supported only on packages with VG."
            return 80

        ## Get VMDK count for all packages. If the pkg is not of DLS type then
        #  count will be default as 0
        vmdk_cnt = self.p._pkg_val_occurances_count_("SG_VMDK_FILE_NAME[")
        if vmdk_cnt == 0:
            ### Get pv wwid details for the physical & sls rdm
            if self._get_vg_pv_wwid_details_(): return 81

        # Open the file to write inputs
        fw = open(self.in_file_name, 'w')

        # Start writing to input file begins here
        fw.write('PKG_TYPE:' + self.p.pkg_type + '\n')

    # 2. Call vmfs util if pkg is of virtual type
        ## Condition for physical servers only
        if (node_type == "PHYSICAL"):
            ## write NODE_TYPE into the package input file
            virt_pkg_type = 'NA'
            fw.write('NODE_TYPE:PHYSICAL\n')

        else:
            ## get snapshot & pkg current node uuid to be used with vmwutil api
            node_uuid = self._get_node_uuid_(self.hostname_wdomain)
            pkg_node_uuid = self._get_node_uuid_(pkg_node_name)

            ## write details into the package input file
            fw.write('NODE_TYPE:VIRTUAL\n')
            fw.write('SNAP_NODE_NAME:' + self.hostname_wdomain + '\n')
            fw.write('SNAP_NODE_UUID:' + node_uuid + '\n')
            fw.write('ORIG_NODE_NAME:' + pkg_node_name + '\n')
            fw.write('ORIG_NODE_UUID:' + pkg_node_uuid + '\n')
            fw.write('VMDK_COUNT:' + str(vmdk_cnt) + '\n')

            ## Import drrvmfs 
            from drrvmfs import VMWARE
            vm = VMWARE(self.p.pkg_name, self.drr_id, self.p.pkg_array, \
                        self.DEBUG, vmdk_cnt, self.hostname_wdomain, node_uuid,\
                        pkg_node_name, pkg_node_uuid, self.pv_wwid_array, self.error_file)

            ## Call setup_esx function to check the connection for esx/vcenter
            #  and return all the required information to continue such as:-
            #  hba wwpn_iscsi details, DS names, DS wwid if required e.t.c.
            if vm._setup_esx_():
                self.ret_error = vm.ret_error
                return 1
            else:
                self.vmfs_out = vm.vmfs_out
                # Iterate through the vm hba array & append it to the main
                # host array. Better to keep on file for everything.
                for hba_val in vm.hba_array:
                    self.node_hba_details.append(hba_val)

                # set the virt_pkg_type as SLS or DLS, also required for STEP:3
                virt_pkg_type = vm.pkg_type

                ## Condition for DLS
                if virt_pkg_type == 'DLS':
                    # Iterate through the vm DS array & append it to the main
                    # host array. Better to keep on file for everything.
                    for ds_wwn in vm.ds_wwn_array:
                        self.pv_wwid_array.append(ds_wwn)

                    # Condition only for DLS RDM
                    # Iterate through self.dls_pv_wwid_array & add pv wwid's to
                    # the array
                    if self.dls_rdm_flag:
                        pv_rdm = open(self.dls_rdm_pv_wwid, 'w')
                        for dls_rdm_wwid in self.dls_pv_wwid_array:
                            self.pv_wwid_array.append(dls_rdm_wwid)
                            pv_rdm.write(dls_rdm_wwid + '\n')
                        pv_rdm.close()

                ## Condition for SLS
                else:
                    # This array will be only populated for SLS RDM
                    for sls_rdm in vm.sls_dls_rdm_info:
                        fw.write(sls_rdm + '\n')

                # write virt type in the input file
                fw.write('VIRT_TYPE:' + virt_pkg_type + '\n')

    # 3. Populate pv_wwid array differntly for PHYSICAL, SLS & DLS config.
        ## Applicable for Physical servers & vmware SLS RDM configuration
        # Note- Can't do this before because we need the pv details
        if virt_pkg_type != 'DLS':

            self._pop_dev_name_wwid_(self.pv_wwid_file_name_in)
            for pv_wwid in self.pv_wwid_array:
                self.pv_array = []
                self._get_pv_name_(pv_wwid, self.pv_wwid_file_name_in)
                self._DEBUG_("pv_array: ", self.pv_array)

                for pv_val in self.pv_array:
                    self._DEBUG_("pv_val: ", pv_val)
                    self._get_device_state_(pv_val)
                    fw.write('PV:' + pv_val + '$' + pv_wwid + '$' + \
                                                          self.dev_state + '\n')

        else:
            for pv_wwid in self.pv_wwid_array:
                fw.write('PV:NA$' + pv_wwid + '\n')

    # 4. Write other entries into the input file.
        if self.multipath_enabled:
            fw.write('MULTIPATH:' + "enabled" + '\n')
        else:
            fw.write('MULTIPATH:' + "disabled" + '\n')

        fw.write('NODE_NAME:' + self.node_name + '\n')

        for hba_wwpn in self.node_hba_details:
            fw.write('HBA:' + str(hba_wwpn) + '\n')

        fw.close()

        ## change the file permissions
        cmd = "chmod 644 " + self.in_file_name
        output, p_status, err = self._cmd_exec_(name = cmd, value = " ")

        return 0
    # EOF _pop_host_details_()


    ###
    # Function to perform scsi rescan at the host
    ###
    def _rescan_scsi_(self):
        # Do a rescan at the host
        output, p_status, err = self._cmd_exec_\
                           (name="rescan-scsi-bus.sh &> /dev/null", value = " ")
    # EOF _rescan_scsi_()


    ###
    # Function to change the device state & perform the rescan at the host
    ###
    def _rescan_at_host_(self, node_type):
        # Check if the input file exists
        if not self._check_file_exist_(self.in_file_name):
            self.ret_error= "Input file " + self.in_file_name + " doesn't exist" 
            return 240

        ## Read the file content in an array
        file_array = []
        fh = open(self.in_file_name)
        file_array = [file_array for file_array in fh.readlines() \
                                                          if file_array.strip()]
        file_array = [x.strip() for x in file_array]
        fh.close()
        self._DEBUG_("file_array: ", file_array)

        # Do a rescan at the host
        self._rescan_scsi_()

        #Iterate through the file array & put the pvs into offline state
        if node_type == 'PHYSICAL':
            for file_array_val in file_array:
                # split with ":"
                file_array_new = file_array_val.split(':')
                # get lines which contains 'PV'
                if (file_array_new[0] == 'PV') or (file_array_new[0] == 'MPATH'):

                    # split the 2 elements with '$' to get "exists"
                    if len(file_array_new[1].split('$')) == 4 :
                        pass
                    else:
                        pv_value = file_array_new[1].split('$')[0]
                        cmd ="echo offline > /sys/block/"+ pv_value +"/device/state"
                        output, p_status, err = \
                                          self._cmd_exec_(name = cmd, value = " ")
                        self._DEBUG_("pv_value: ", pv_value, "p_status: ", p_status)


        # Sleep for 5 seconds followed by a vgscan
        output, p_status, err = self._cmd_exec_\
                           (name="sleep 5 &> /dev/null", value = " ")

        ## Get before & after pvs value
        output, p_status, err = self._cmd_exec_\
                           (name="pvs", value = " ")
        output, p_status, err = self._cmd_exec_\
                           (name="pvscan &> /dev/null", value = " ")
        output, p_status, err = self._cmd_exec_\
                           (name="vgscan &> /dev/null", value = " ")
        output, p_status, err = self._cmd_exec_\
                           (name="pvs", value = " ")                                                                    
        return 0
    # EOF _rescan_at_host_()


    ###
    # Function to change the device state & perform the rescan at the host
    # Need to call only for Physical & SLS RDM setups
    ###
    def _cleanup_(self):

        ## Read the file content in an array
        file_array = []
        fh = open(self.in_file_name)
        file_array = [file_array for file_array in fh.readlines() \
                                                          if file_array.strip()]
        file_array = [x.strip() for x in file_array]
        fh.close()
        output, p_status, err = self._cmd_exec_\
                           (name="sleep 2 &> /dev/null", value = " ")

        self._DEBUG_("file_array: ", file_array)
        #self.out_file_name = "out_file_name"
        if self._read_output_file():
            return 210

        ## Delete the snapshot pv at the host
        self._pop_dev_name_wwid_(self.pv_wwid_file_name_out)
        for reh_array in self.rehearsal_array:
            if reh_array['STEPS_COMPLETED'] == '3':
                self._DEBUG_("reh_array['SV_WWID'] :", reh_array['SV_WWID'])

                # Get snapshot pv name
                self.pv_array = []
                self._get_pv_name_(reh_array['SV_WWID'],self.pv_wwid_file_name_out)
                self._DEBUG_("pv_array: ", self.pv_array)

                for pv_val in self.pv_array:
                    self._DEBUG_("self.pv_name: ", pv_val)
                    # Delete the snapshot pv at the host
                    cmd ="echo 1 > /sys/block/"+ pv_val +"/device/delete"
                    output, p_status, err = \
                                      self._cmd_exec_(name = cmd, value = " ")


        ##Iterate through the file array
        for file_array_val in file_array:
            # split with ":"
            file_array_new = file_array_val.split(':')
            # get lines which contains 'PV'
            if (file_array_new[0] == 'PV') or (file_array_new[0] == 'MPATH'):

                # split the 2 elements with '$' to get "exists"
                if len(file_array_new[1].split('$')) == 4 :
                    pass
                else:
                    pv_value = file_array_new[1].split('$')[0]
                    pv_state = file_array_new[1].split('$')[2]
                    cmd ="echo " + pv_state + " > /sys/block/"+ \
                                                       pv_value +"/device/state"
                    output, p_status, err = \
                                      self._cmd_exec_(name = cmd, value = " ")
                    self._DEBUG_("pv_value, pv_state :", pv_value, ',',pv_state)

        return 0
    # EOF _cleanup_()


    ###
    # Function _sls_rdm_setup_() to do rescan, detach & attach vmdks
    # This function will do the following-
    # 1. rescan at esx/vcenter
    # 2. detach the vmdks from the VM
    # 3. Create & attach the vmdks as per the pv_wwid array passed to function
    ###
    def _sls_rdm_setup_(self, call_type, vmfs_orig_snap_pv_arr):

        ## Import drrvmfs
        from drrvmfs import VMWARE

        # Get hostname without domain
        self._hostname_wdomain_()

        vmdk_cnt      = 0
        pkg_node_name = ''
        pkg_node_uuid = ''
        node_uuid     = self._get_node_uuid_(self.hostname_wdomain)

        vm = VMWARE(self.p.pkg_name, self.drr_id, self.p.pkg_array, \
             self.DEBUG, vmdk_cnt, self.hostname_wdomain, node_uuid,\
             pkg_node_name, pkg_node_uuid, self.pv_wwid_array, self.error_file)

    ## 1. rescan at esx/vcenter as HBA type only to get snapshot vvs
        if vm._rescan_esx_vc_('hba'):
            return 1

    ## 2. detach the original vmdks from the vm.
        if vm._detach_vmdk_(call_type):
            ## Roll_back steps:-
            #  Call vm_attach_vmdk_ with roll_back parameter to -
            #  2.1 attach all original vmdks which got detached
            #  Host roll_back flag will be True as we can remove the storage from
            #  backend.
            vm._attach_vmdk_('roll_back')

            ## Return non-zero as detach failed
            return 1

    ## 3. Create & attach vmdks as per the pv_wwid array passed to function
        if vm._create_vmdk_(vmfs_orig_snap_pv_arr):
            ## Roll_back steps
            #  Call vm._attach_vmdk_ with roll_back parameter to -
            #  3.1 detach all snapshot vmdks which got attached
            #  3.2 attach all original vmdks which got detached
            #  Host roll_back flag will be False when  rollback will fail & TRUE
            #  when roll_back is successful

            ## Failed while creating & attaching vmdks. Return actual error
            #  3.1 detach all snapshot vmdks which got attached
            #  If vm.sls_dls_rdm_info array is empty then don't detach vmdks
            if vm.sls_dls_rdm_info:
                vm._detach_vmdk_('roll_back')

            #  3.2 attach all original vmdks which got detached
            #  Call attach vmdk with roll_back
            vm._attach_vmdk_('roll_back')

            ## Return non-zero as detach failed
            return 1
        return 0
    # EOF _sls_rdm_setup_()

    ###
    # Function _sls_rdm_cleanup_() to do do the cleanup
    # This function will do the following-
    # 1. Detach the snapshot vmdks by reading the final out file
    # 2. Destroy the created snapshot vmdks.
    # 3. Attach the original vmdk back to vms by reading the output file
    ###
    def _sls_rdm_cleanup_(self, call_type):
        ## Import drrvmfs
        from drrvmfs import VMWARE

        # Get hostname without domain
        self._hostname_wdomain_()

        vmdk_cnt = 0
        pkg_node_name = ''
        node_uuid = self._get_node_uuid_(self.hostname_wdomain)
        pkg_node_uuid = ''
        vm = VMWARE(self.p.pkg_name, self.drr_id, self.p.pkg_array, \
             self.DEBUG, vmdk_cnt, self.hostname_wdomain, node_uuid,\
             pkg_node_name, pkg_node_uuid, self.pv_wwid_array, self.error_file)

        ## Populate esx/vcenter to setup values for below functions
        vm._populate_info_()

    ## 1. Detach the snapshot vmdks by reading the final out file
        if vm._detach_vmdk_(call_type):
            ## Roll_back steps
            #  Call vm._attach_vmdk_ with roll_back parameter to -
            #  1.1 detach all snapshot vmdks which got attached
            #  1.2 attach all original vmdks which got detached

            ## Failed while detaching snapshot vmdks. Return actual error
            #  1.1 detach all snapshot vmdks which got attached
            if vm.sls_dls_rdm_info:
                vm._detach_vmdk_('roll_back')

            #  1.2 attach all original vmdks which got detached
            #  Call attach vmdk with roll_back
            vm._attach_vmdk_('roll_back')

            ## Return non-zero as detach failed
            return 1

    ## 2. Destroy the created snapshot vmdks.
        vm._destroy_vmdk_()

    ## 3. Attach the original vmdk back to vms by reading the output file
        if vm._attach_vmdk_(call_type):
            # vm.sls_dls_rdm_other_info array will have a list of vmdks for which
            # action was not SUCCEED. This can be used for cleanup.
            return 1

        return 0
    # EOF _sls_rdm_cleanup_()


    ###
    # Function _sls_rdm_cleanup_rescan_() to do do the cleanup
    # This function will do the following-
    # 1. Rescan at esx/vcenter, rescan as 'all' to rescan hba & vmfs both
    ###
    def _sls_rdm_cleanup_rescan_(self):
        ## Import drrvmfs
        from drrvmfs import VMWARE

        # Get hostname without domain
        self._hostname_wdomain_()

        vmdk_cnt = 0
        pkg_node_name = ''
        node_uuid = self._get_node_uuid_(self.hostname_wdomain)
        pkg_node_uuid = ''
        vm = VMWARE(self.p.pkg_name, self.drr_id, self.p.pkg_array, \
             self.DEBUG, vmdk_cnt, self.hostname_wdomain, node_uuid,\
             pkg_node_name, pkg_node_uuid, self.pv_wwid_array, self.error_file)


    ## 1. rescan at esx/vcenter, rescan as 'all' to rescan hba & vmfs both
        if vm._rescan_esx_vc_('all'):
            return 1

        return 0
    # EOF _sls_rdm_cleanup_rescan_()


    ###
    # Function _dls_vmfs_rdm_setup_() to do rescan, keep same-signaturing, 
    # detach & attach vmdks
    # This function will do the following-
    # 1. rescan at esx/vcenter
    # 2. Resignature the existing dev ds 
    # 3. Attach snapshot vmdks to the vm's
    # 3.1. Create & attach for DLS RDM pkgs
    # 3.2. Attach for DLS VMFS pkgs
    ###
    def _dls_vmfs_rdm_setup_(self, call_type, vmfs_orig_snap_pv_arr):
        self._DEBUG_("Inside function _dls_vmfs_rdm_setup_")
        self._DEBUG_(vmfs_orig_snap_pv_arr)
        if self._check_file_exist_(self.dls_rdm_file):
            other_arr       = {}
            associative_arr = {}
            lines = [line.rstrip('\n') for line in open(self.dls_rdm_pv_wwid)]
            self._DEBUG_(lines)
            for pv_val in vmfs_orig_snap_pv_arr:
                self._DEBUG_(pv_val)
                found = False
                for line in lines:
                    if line == pv_val:
                       found = True
                if found:
                   other_arr[pv_val] = vmfs_orig_snap_pv_arr[pv_val]
                else:
                   associative_arr[pv_val] = vmfs_orig_snap_pv_arr[pv_val]

            self._DEBUG_(other_arr)
            self._DEBUG_(associative_arr)

            vmfs_orig_snap_pv_arr = associative_arr

        ## Import drrvmfs
        from drrvmfs import VMWARE

        # Get hostname without domain
        self._hostname_wdomain_()

        self.p._get_pkg_details_()
        vmdk_cnt = self.p._pkg_val_occurances_count_("SG_VMDK_FILE_NAME[")
        pkg_node_name, rcode  = self.p._get_pkg_node_name_()
        if rcode:
            error = "Unable to get the package node name"
            self._write_to_file_(error, 'a')
            return 75

        node_uuid = self._get_node_uuid_(self.hostname_wdomain)
        pkg_node_uuid = self._get_node_uuid_(pkg_node_name)

        vm = VMWARE(self.p.pkg_name, self.drr_id, self.p.pkg_array, \
             self.DEBUG, vmdk_cnt, self.hostname_wdomain, node_uuid,\
             pkg_node_name, pkg_node_uuid, self.pv_wwid_array, self.error_file)

        vm._populate_info_()
    ## 1. rescan at esx/vcenter as HBA type only to get snapshot vvs
        if vm._rescan_esx_vc_('hba'):
            return 1

    ## 2. Resignature the existing dev ds
    #     As Resignaturing is done by us & its internal, so will not give any 
    #     reason for its failure excep a generic message.
        if vm._resignature_dev_ds_(vmfs_orig_snap_pv_arr):
            return 1

    ## 3. Attach snapshot vmdks to the vm's
        # 3.1. Create & attach for DLS RDM pkgs
        if self._check_file_exist_(self.dls_rdm_file):
            if vm._create_vmdk_(other_arr):
                return 1

        # 3.2. Attach for DLS VMFS pkgs
        else:
            if vm._attach_vmdk_(call_type):
                ## Roll_back steps
                #  Call vm._detach_vmdk_ with roll_back parameter to -
                #  3.1 detach all snapshot vmdks if attached
                vm._detach_vmdk_('roll_back')

                ## Return non-zero as attach failed
                return 1

        return 0
    # EOF _dls_vmfs_rdm_setup_()


    ###
    # Function _dls_cleanup_() to do do the cleanup
    # This function will do the following-
    # 1. Detach the snapshot vmdks by reading the final out file
    # 2. rescan at esx/vcenter, rescan as 'all' to rescan hba & vmfs both
    ###
    def _dls_cleanup_(self, call_type):
        ## Import drrvmfs
        from drrvmfs import VMWARE

        # Get hostname without domain
        self._hostname_wdomain_()

        vmdk_cnt = 0
        pkg_node_name = ''
        node_uuid = self._get_node_uuid_(self.hostname_wdomain)
        pkg_node_uuid = ''
        vm = VMWARE(self.p.pkg_name, self.drr_id, self.p.pkg_array, \
             self.DEBUG, vmdk_cnt, self.hostname_wdomain, node_uuid,\
             pkg_node_name, pkg_node_uuid, self.pv_wwid_array, self.error_file)

        ## Populate esx/vcenter to setup values for below functions
        vm._populate_info_()

    ## 1. Detach the snapshot vmdks by reading the final out file
        if vm._detach_vmdk_(call_type):
            return 1

    ## 2. rescan at esx/vcenter, rescan as 'all' to rescan hba & vmfs both
        if vm._rescan_esx_vc_('all'):
            return 1

        return 0
    # EOF _dls_cleanup_()

    ###
    # Function to verify the ESX or Vcenter connection while executing cleanup
    ###
    def _verify_esx_vcenter_conn_(self):
        ## Import drrvmfs
        from drrvmfs import VMWARE

        # Get hostname without domain
        self._hostname_wdomain_()

        vmdk_cnt = 0
        pkg_node_name = ''
        node_uuid = self._get_node_uuid_(self.hostname_wdomain)
        pkg_node_uuid = ''
        vm = VMWARE(self.p.pkg_name, self.drr_id, self.p.pkg_array, \
             self.DEBUG, vmdk_cnt, self.hostname_wdomain, node_uuid,\
             pkg_node_name, pkg_node_uuid, self.pv_wwid_array, self.error_file)

        ## Populate esx/vcenter to setup values for below functions
        if vm._populate_info_():
            return 1
        if vm._verify_esx_vcenter_connection_('node'):
            return 1
        return 0
## EOC HOST
