#!/usr/bin/env python
####################################################################
# (C) Copyright 2017 Hewlett Packard Enterprise Development LP.
# @(#) MC DRR Package utility script.
# @(#) Product Name                :
# @(#) Product Version             : B.12.10.00
# @(#) Patch Name                  :
#
# NOTE:    This script is not configurable!  Any changes made to this
#          script will be overwritten when you upgrade to the next
#          release of Serviceguard metrocluster addons.
#
###################################################################

## All imported modules
from drrhelper import Helper
import sys, os

os.environ['PATH'] = "/usr/lib64/qt-3.3/bin:/usr/local/sbin:" + \
                         "/usr/local/bin:/usr/sbin:/usr/bin:" + \
            "/usr/local/cmcluster/bin:/opt/cmcluster/bin:/root/bin:/sbin/:/bin:"
os.environ['HOME'] = "/root/"


###
# Class Package
###
class Package(Helper):

    ###
    # INIT
    ###
    def __init__(self, pkg_name, drr_id, DEBUG, error_file):
        Helper.__init__(self, DEBUG, pkg_name, drr_id, error_file)
        self.pkg_type          = ""
        self.drr_id            = drr_id
        self.pkg_name          = pkg_name
        self.pkg_array         = {}
        self.node_type         = ""
        self.err_file          = error_file
        self.pkg_value_array   = ""
        self.pkg_cmviewcl_file = ""

        # Source /etc/cluster.conf file
        self._source_config_file_("/etc/cmcluster.conf")
        self._DEBUG_(self.SGCONF,self.SGRUN,self.SGSBIN,self.SGROOT)

        # Create package input, output & error file.
        self.in_file_name  = self.SGRUN + "/DRxR/" + drr_id + "/.mc_" + \
                                         pkg_name + '_' + drr_id + '_input_file'
        self.out_file_name = self.SGRUN + "/DRxR/" + drr_id + "/.mc_" + \
                                     pkg_name + '_' + drr_id + '_output_file'
        self.pv_wwid_file_name_in  = self.SGRUN + "/DRxR/" + drr_id + "/.mc_" + \
                                     pkg_name + '_' + drr_id + '_pv_wwid_file_in'
        self.pv_wwid_file_name_out = self.SGRUN + "/DRxR/" + drr_id + "/.mc_" +\
                                   pkg_name + '_' + drr_id + '_pv_wwid_file_out'
        self.dls_rdm_pv_info    = self.SGRUN + '/DRxR/' + drr_id + '/.mc_' +\
                                    pkg_name + '_' + drr_id + '_dls_rdm_pv_info'
        self.dls_rdm_pv_wwid    = self.SGRUN + '/DRxR/' + drr_id + '/.mc_' +\
                                    pkg_name + '_' + drr_id + '_dls_rdm_pv_wwid'
        self.dls_rdm_file       = self.SGRUN + '/DRxR/' + drr_id + '/.mc_' + \
                                                      pkg_name + '_dls_rdm_file'

        self._DEBUG_("Input filename: ", self.in_file_name)
        self._DEBUG_("Output filename: ", self.out_file_name)
        self._DEBUG_("pv_wwid_file_name_in file: ", self.pv_wwid_file_name_in)
        self._DEBUG_("pv_wwid_file_name_out file: ", self.pv_wwid_file_name_out)
        self._DEBUG_("dls_rdm_pv_info file: ", self.dls_rdm_pv_info)

    # EOF __init__()


    ###
    # Function to create the temporary file name
    # Usage : _tmp_pkg_file_(pkg_name)
    ###
    def _tmp_pkg_file_(self):
        self._DEBUG_("Inside function _tmp_pkg_file_")
        tmp_directory = self.SGRUN + "/DRxR/" + self.drr_id + "/"
        pkg_details_file  = tmp_directory + "drr_cmviewcl_p_" + self.pkg_name
        return pkg_details_file   
    # EOF _tmp_pkg_file_(pkg_name)

    ###
    # Function to get package id for a given package name
    ###
    def _get_pkg_id_(self, pkg_name):
        self._DEBUG_("Inside function _get_pkg_id_")
        self._DEBUG_("pkg_name: ", pkg_name)
        pkg_id_var     = 'package:' + pkg_name + '|id='

        ## Read cmviewcl output file
        self._tmp_pkg_cmviewcl_file_()
        #with open(self.pkg_cmviewcl_file) as cmviewcl_file:
        fh = open(self.pkg_cmviewcl_file, "r")
        cmviewcl_file =  [x.strip() for x in fh.readlines()]
        for line in cmviewcl_file:
            if ( pkg_id_var in line):
                # remove new line character from line variable
                self.pkg_id = line.rstrip()
                break
        # Read file ends here

        ## Split and get package id in variable
        self.pkg_id = self.pkg_id.split('=')[1]
        self._DEBUG_("self.pkg_id: ", self.pkg_id)

        if not self.pkg_id:
            self.ret_error = "Unable to get package id."
            return -1

        return self.pkg_id
    # EOF _get_pkg_id_()

    ###
    # Function to create pkg environment available for other functions to import
    ###
    def _cache_pkg_env_(self):
        self._DEBUG_("Inside function _cache_pkg_env_")
        pkg_details_file = self._tmp_pkg_file_()
        self._DEBUG_("pkg_details_file: ", pkg_details_file)
        cmd = self.SGSBIN + "/cmviewcl"
        value = "-v -p " + self.pkg_name + " > " + pkg_details_file
        output,p_status,err = self._cmd_exec_(name = cmd, value = value)
        if (p_status != 0):
            self.ret_error = "ERROR: Unable to get the package", self.pkg_name, "details."
            return 1
        return 0
    # EOF _cache_pkg_env_(pkg_name)


    ###
    # Function to create pkg environment available for other functions to import
    ###
    def _cache_pkg_cmviewcl_env_(self):
        self._DEBUG_("Inside function _cache_pkg_cmviewcl_env_")
        self.pkg_cmviewcl_file = self._tmp_pkg_cmviewcl_file_()
        self._DEBUG_("pkg_cmviewcl_file: ", self.pkg_cmviewcl_file)
        value = "-vf line -s config > " + self.pkg_cmviewcl_file
        cmd = self.SGSBIN + "/cmviewcl"
        output, p_status, err= self._cmd_exec_(name = cmd, value = value)
        if (p_status != 0):
            self.ret_error = "ERROR: Unable to get the cmviewcl -vf line information."
            return 1
        return 0
    # EOF _cache_pkg_cmviewcl_env_(pkg_name)


    ###
    # Function to check if a package exist
    ###
    def _check_if_pkg_exist(self):
        self._DEBUG_("Inside function _check_if_pkg_exist")
        RC = self._cache_pkg_env_()
        if (RC != 0):
            self.ret_error = "ERROR: Package", self.pkg_name, "doesn't exist."
            return 1
        return 0
    # EOF _check_if_pkg_exist(pkg_name)


    ###
    # Function to get package current status
    # Note: Execute this after /tmp/drr_cmviewcl will be created.
    ###
    def _get_pkg_current_status_(self):
        self._DEBUG_("Inside function _get_pkg_current_status_")
        pkg_details_file = self._tmp_pkg_file_()
        value = pkg_details_file + " | grep -w " + self.pkg_name + \
                                                           " | awk '{print $3}'"
        output, rcode, err = self._cmd_exec_(name = "cat", value = value)
        if ( output == "running" ):
            return output, rcode
        else :
            # will return 2, in-case package current status is not running
            rcode = 2
            return output, rcode
    # EOF _get_pkg_current_status_(pkg_name)


    ###
    # Function to get package node name
    # Note: Execute this after /tmp/drr_cmviewcl will be created.
    ###
    def _get_pkg_node_name_(self):
        self._DEBUG_("Inside function _get_pkg_node_name_")
        output, rcode = self._get_pkg_current_status_()
        if ( rcode == 0 ):
            pkg_details_file = self._tmp_pkg_file_()
            value = pkg_details_file + " | grep -w " + self.pkg_name + \
                                                           " | awk '{print $5}'"
            output, rcode, err = self._cmd_exec_(name = "cat", value = value)
            return output, rcode
        else :
            output = "Unable to detect the pkg node name as its not running."
            return output, rcode
    # EOF _get_pkg_node_name_(pkg_name)


    ###
    # Function to create the temporary file name for cmviewcl output
    # Usage : _tmp_pkg_cmviewcl_file_()
    ###
    def _tmp_pkg_cmviewcl_file_(self):
        self._DEBUG_("Inside function _tmp_pkg_cmviewcl_file_")
        tmp_directory = self.SGRUN + "/DRxR/" + self.drr_id + "/"
        self.pkg_cmviewcl_file = tmp_directory + "drr_cmviewcl_" + self.pkg_name
        return self.pkg_cmviewcl_file
    # EOF _tmp_pkg_cmviewcl_file_()


    ###
    # Function to get the package details in an array
    ###
    def _get_pkg_details_(self):
        self._DEBUG_("Inside function _get_pkg_details_")
        cmd_val = self.SGSBIN + "/cmgetpkgenv " + self.pkg_name
        output, p_status, err = self._cmd_exec_(name = cmd_val, value=" ")
        if ( p_status != 0):
            return False
        pkg_big_array = {}
        pkg_big_array = output.splitlines()
        self._DEBUG_("pkg_big_array length: ", len(pkg_big_array))
        #global self.pkg_array
        self.pkg_array = {}
        pkg_val_array = {}
        for array_element in pkg_big_array :
            self._DEBUG_("array_element: ", array_element)
            pkg_val_array = array_element.split("=")
            pkg_val_key = pkg_val_array[0]
            #string = string[1:-1] 
            #---- To remove first & last inverted commas from a string
            pkg_val_value = pkg_val_array[1][1:-1]
            #self._DEBUG_("pkg_val_key: ", pkg_val_key)
            #self._DEBUG_("pkg_val_value: ", pkg_val_value)
            self.pkg_array[pkg_val_key] = pkg_val_value
            pkg_val_array = {}
        self._DEBUG_("pkg_array: ", self.pkg_array)
        return self.pkg_array
    # EOF _get_pkg_details_(pkg_name)


    ###
    # Function to count the no of VG's/VMDK/e.t.c exists for a given package
    ###
    def _pkg_val_occurances_count_(self, value):
        self._DEBUG_("Inside function _pkg_val_occurances_count_ for-", value)
        aa=filter(lambda x:value in x, self.pkg_array)
        self.pkg_value_array = list(aa)
        self._DEBUG_("pkg_value_array- ", self.pkg_value_array)
        self._DEBUG_("pkg_value_array len- ", len(self.pkg_value_array))
        return len(self.pkg_value_array)
    # EOF _pkg_val_occurances_count_(value)


    ###
    # Function to detect that a given package is a 3PAR/XP/EVA/SRDF package
    ###
    def _get_pkg_type_(self):
        self._DEBUG_("Inside function _get_pkg_type_")
        #self._cache_pkg_cmviewcl_env_()
        mc3par_module = "dts/mc3parrc"
        mcxp_module   = "dts/mcxp"
        mceva_module  = "dts/mceva"
        mcsrdf_module = "dts/mcsrdf"
        self._pkg_val_occurances_count_("SG_MODULE_NAME[")
        for pkg_value in self.pkg_value_array:
            if (self.pkg_array[pkg_value] == mc3par_module):
                self.pkg_type = "MC3PAR";
                return 0
            elif (self.pkg_array[pkg_value] == mcxp_module):
                self.pkg_type = "MCXP";
                return 0
            elif (self.pkg_array[pkg_value] == mceva_module):
                self.pkg_type = "MCEVA";
                return 0
            elif (self.pkg_array[pkg_value] == mcsrdf_module):
                self.pkg_type = "MCSRDF";
                return 0
        return 1
    # EOF _pkg_val_occurances_count_(value)


# EOC Package()
