#!/usr/bin/env python
####################################################################
# (C) Copyright 2017 Hewlett Packard Enterprise Development LP.
# @(#) MC DRR vmfs utility script.
# @(#) Product Name                :
# @(#) Product Version             : B.12.10.00
# @(#) Patch Name                  :
#
# NOTE:    This script is not configurable!  Any changes made to this
#          script will be overwritten when you upgrade to the next
#          release of Serviceguard metrocluster addons.
#
###################################################################
## All imported modules
from drrhelper import Helper
from drrpkgutils import Package
import os, sys, shutil

os.environ['PATH'] = "/usr/lib64/qt-3.3/bin:/usr/local/sbin:" + \
                         "/usr/local/bin:/usr/sbin:/usr/bin:" + \
            "/usr/local/cmcluster/bin:/opt/cmcluster/bin:/root/bin:/sbin/:/bin:"
os.environ['HOME'] = "/root/"

###
# Class STORAGE_ARRAY()
###
class VMWARE(Helper):

    ###
    # INIT
    ###
    def __init__(self, pkg_name, drr_id, pkg_array, DEBUG, vmdk_count, \
             node_name, node_uuid, pkg_node_name, pkg_node_uuid, pv_wwid_array, err_file):

        ## Import Helper module
        Helper.__init__(self, DEBUG, pkg_name, drr_id, err_file)

        self.p = Package(pkg_name, drr_id, int(DEBUG), err_file)
        self.drr_id                 = drr_id
        self.esx_vc                 = ''
        self.pkg_type               = 'SLS'
        self.disk_type              = 'NA'
        self.ds_name                = ''
        self.pv_wwid                = ''
        self.pkg_array              = pkg_array
        self.hba_array              = []
        self.dls_array              = []
        self.ds_wwn_array           = []
        self.sls_dls_rdm_info       = []
        self.sls_dls_rdm_other_info = []
        self.vmdk_count             = int(vmdk_count)
        self.node_uuid              = node_uuid
        self.node_name              = node_name
        self.pkg_node_uuid          = pkg_node_uuid
        self.pkg_node_name          = pkg_node_name
        self.pkg_name               = pkg_name
        self.pv_wwid_array          = pv_wwid_array
        self.snap_wwid_arr          = []
        self.attach_detach_info_arr = []
        self.final_out_array        = []
        self.ds_wwid_head           = {}
        self.sls_snap_vmdk_array    = {}
        self.snap_ds_name_with_wwid = {}

        self.vmfsutilsbin = '/opt/drenabler/utils/drr_commonutils/drrvmfsutil'
        drr_directory    = self.p.SGRUN  + "/DRxR/" + self.drr_id + '/'
        self.vmfs_in     = drr_directory + '.mc_' + self.pkg_name + '_vmfs_in'
        self.vmfs_out    = drr_directory + '.mc_' + self.pkg_name + '_vmfs_out'
        self.vmfs_out_bk = drr_directory + '.mc_' + self.pkg_name + '_vmfs_out'
        self.vmfs_dump   = drr_directory + '.mc_' + self.pkg_name + '_vmfs_dump'
        self.vmfs_final  = drr_directory + '.mc_' + self.pkg_name + '_vmfs_final'
        self.vmfs_resig  = drr_directory + '.mc_' + self.pkg_name + '_vmfs_resig'
        self.dls_vmdk_ar = drr_directory + '.mc_' + self.pkg_name + '_dls_vmdk_info'
        sls_dls_rdm_error= drr_directory + '.mc_' + self.pkg_name + '_dls_rdm_error'
        self.roll_attach = drr_directory + '.mc_' + self.pkg_name + '_roll_attach'
        self.roll_detach = drr_directory + '.mc_' + self.pkg_name + '_roll_detach'
        self.dls_rdm_file= drr_directory + '.mc_' + self.pkg_name + '_dls_rdm_file'

    # EOF __init__()

    ###
    # Function to populate information w.r.t to esx/vcenter
    # 1. Verify connection
    # 2. Get vmdk info from package
    ###
    def _populate_info_(self):
        self._DEBUG_("Inside _populate_info_ functon")

    # 1. Verify connection
        # Get node esx/vcenter name & verify connection
        if self._get_esx_vc_name_(self.node_name): return 1
        self.node_esx_vc = self.esx_vc
        #if self._verify_esx_vcenter_connection_("node"): return 1

        ## No need to verify the connection for the pkg running node if pkg is
        #  of SLS RDM type
        ## Get pkg node esx/vcenter name & verify connection
        if self.vmdk_count > 0:
            if self._get_esx_vc_name_(self.pkg_node_name): return 1
            self.pkg_node_esx_vc = self.esx_vc
            #if self._verify_esx_vcenter_connection_("pkg"): return 1

    # 2. Get vmdk info from package
        ## If package type DLS then need to get the vmdk's information in an arr
        #  If vmdk_count is greater than 0 that means the package is of DLS type
        if self.vmdk_count > 0:
            self.pkg_type = 'DLS'
            # Populate dls_array
            RC = self._get_vmdk_info_()
            if RC: return RC

        return 0
    # EOF _populate_info_()


    ###
    # Function to read cmviewcl output file & get vmdk information
    ###
    def _get_vmdk_info_(self):
        self.dls_array = []
        for num in range(0, self.vmdk_count):
            self._DEBUG_("vmdk_num: ", self.vmdk_count)
            vmdk_arr         = {}
            vmdk_arr['DS']   = self.pkg_array['SG_DATASTORE_NAME['  + str(num) + ']']
            vmdk_arr['VMDK'] = self.pkg_array['SG_VMDK_FILE_NAME['  + str(num) + ']']
            vmdk_arr['PORT'] = self.pkg_array['SG_SCSI_CONTROLLER[' + str(num) + ']']
            self.disk_type = self.pkg_array['SG_DISK_TYPE[' + str(num) + ']']

            # Append to the array
            self._DEBUG_("vmdk_arr: ", vmdk_arr)
            self.dls_array.append(vmdk_arr)

        #Return error if the  array file was not populated
        if not self.dls_array:
            self.ret_error = "Unable to get the VMDK information from package."
            self._write_to_file_(self.ret_error, 'a')
            return 1
        return 0
    # EOF _get_vmdk_info_()


    ###
    # Function to get esx or vcenter name for a given node name
    ###
    def _get_esx_vc_name_(self, node_name):
        self._DEBUG_("node_name: ", node_name)
        esx_var     = 'node:' + node_name + '|esx_name='
        vcenter_var = 'vcenter_name='

        ## Read cmviewcl output file
        self.p._tmp_pkg_cmviewcl_file_()        
        #with open(self.p.pkg_cmviewcl_file) as cmviewcl_file:
        fh = open(self.p.pkg_cmviewcl_file, "r")
        cmviewcl_file =  [x.strip() for x in fh.readlines()]
        for line in cmviewcl_file:
            if (esx_var in line) or (vcenter_var in line):
                # remove new line character from line variable
                self.esx_vc = line.rstrip()
                break
        # Read file ends here

        ## Split and get esx or vsphere name into the variable
        self._DEBUG_("esx_vc Before: ", self.esx_vc)
        self.esx_vc = self.esx_vc.split('=')[1]
        self._DEBUG_("esx_vc After: ", self.esx_vc)
        
        if not self.esx_vc:
            self.ret_error = "Unable to get ESX or Vsphere details."
            self._write_to_file_(self.ret_error, 'a')
            return 172

        return 0
    # EOF _get_esx_vc_name_()


    ###
    # Function to verify the ESX host or Vcenter connection
    ###
    def _verify_esx_vcenter_connection_(self, node_option):
        ##vmwutil option
        option = "connect"

        if node_option == 'node':
            parms = "-H " + self.node_esx_vc
        else:
            parms = "-H " + self.pkg_node_esx_vc
        self._DEBUG_("parms: ", parms)

        RC = 0
        RC = self._call_vmwutil_(option, parms)
        if RC:
            if node_option == 'node':
                self.ret_error = "Unable to communicate to esx/vcenter:-" + \
                                                                self.node_esx_vc
                self._write_to_file_(self.ret_error, 'a')
            else:
                self.ret_error = "Unable to communicate to esx/vcenter:-" + \
                                                            self.pkg_node_esx_vc
                self._write_to_file_(self.ret_error, 'a')
            return 1
        return 0
    # EOF _verify_esx_vcenter_connection_()


    ###
    # Function to get ESX host wwpn & iscsi details
    ###
    def _get_wwpn_iscsi_(self):
        self._DEBUG_("Inside function _get_wwpn_iscsi_")
        self._DEBUG_("self.vmfs_in: ", self.vmfs_in)
        option = "query"

        fw = open(self.vmfs_in, 'w')
        fw.write('UUID:' + self.node_uuid + '\n')
        fw.write('ADAPTER:all' + '\n')
        fw.close()

        parms = "-H " + self.node_esx_vc
        self._DEBUG_("parms: ", parms)
        RC = 0
        RC = self._call_vmwutil_(option, parms)
        if RC:
            self.ret_error = "Unable to get the wwpn & iscsi information " +\
                                    "from esx/vcenter:-" + self.node_esx_vc
            self._write_to_file_(self.ret_error, 'a')
            return 1
      
        ## Parse output & get wwpn & iscsi details
        if self._parse_esx_output_('HBA'): 
            return 1

        return 0
    # EOF _get_wwpn_iscsi_()


    ###
    # Function to get SLS information
    ###
    def _get_sls_info_(self):
        self._DEBUG_("Inside function _get_sls_info_")
        option = "query"

        self._DEBUG_("self.vmfs_in: ", self.vmfs_in)
        fw = open(self.vmfs_in, 'w')
        fw.write('UUID:' + self.node_uuid + '\n')
        fw.close()

        parms = "-H " + self.node_esx_vc
        self._DEBUG_("parms: ", parms)

        RC = 0
        RC = self._call_vmwutil_(option, parms)
        if RC:
            self.ret_error = "Unable to get the required DS, VMDK & slot " + \
                                     " information from the esx/vcenter:-" + \
                                                           self.node_esx_vc
            self._write_to_file_(self.ret_error, 'a')
            return 1

        ## Parse DS VMDK, SLOT for SLS configuration
        for self.pv_wwid in self.pv_wwid_array:
            if self._parse_esx_output_('SLS_INFO'): 
                return 1
        return 0
    # EOF _get_sls_info_()


    ###
    # Function to get DLS information
    ###
    def _get_dls_info_(self):
        self._DEBUG_("Inside function _get_dls_info_")
        option = "query"

        self._DEBUG_("self.vmfs_in: ", self.vmfs_in)
        fw = open(self.vmfs_in, 'w')
        fw.write('UUID:' + self.pkg_node_uuid + '\n')

        # write all vmdk information to the input file
        for dls_val in self.dls_array:
            fw.write('VMDK:' + dls_val['VMDK'] + '\n')
            fw.write('DS:' + dls_val['DS'] + '\n')
            fw.write('SLOT:' + dls_val['PORT'] + '\n')

        # Close the file
        fw.close()

        parms = "-H " + self.pkg_node_esx_vc
        self._DEBUG_("parms: ", parms)

        RC = 0
        RC = self._call_vmwutil_(option, parms)
        if RC:
            self.ret_error = "Unable to get the required DS, VMDK & slot " + \
                               " information from the esx/vcenter:-" + \
                                                           self.pkg_node_esx_vc
            self._write_to_file_(self.ret_error, 'a')
            return 1

        ## Parse output & get DS associated wwid's
        old_ds_name = []
        for dls_info in self.dls_array:

            self._DEBUG_("dls_info: ", dls_info)
            self.ds_name = dls_info['DS']
            self._DEBUG_("ds_name: ", self.ds_name)

            if self.ds_name in old_ds_name:
                pass
            else:
                old_ds_name.append(self.ds_name)
                if self._parse_esx_output_('DLS_VMFS'):
                    return 1

            ## For DLS RDM
            #"Datastore:DS_NAME|File:XYZ.vmdk|wwid="
            if self._check_file_exist_(self.dls_rdm_file):
                vmdk = dls_info['VMDK']
                cmd = "cat " + self.vmfs_dump
                val = "| grep \"Datastore:" + dls_info['DS'] + '|File:' + vmdk + \
                                                                         "|wwid=\""
                val1= "| grep \"Datastore:" + dls_info['DS'] + '|File:' + vmdk + \
                                                     "|Slot\" | grep \"|name=\""
                output, p_status, err = self._cmd_exec_(name = cmd, value = val)
                output1, p_status, err = self._cmd_exec_(name = cmd, value = val1)

                if output:
                    output = output.split("=")
                    output1= output1.split("=")
                    fw = open(self.dls_vmdk_ar, 'a')
                    #XYZ.vmdk=vml.WWID=SLOT
                    fw.write(vmdk + '=' + output[1] + '=' + output1[1] + '\n')
                    fw.close()

        self._DEBUG_("dls_vmdk_ar file-", self.dls_vmdk_ar)

        ## write DS head wwid info to the vmfs_resig file to be used later-on
        #  Open the file to write
        fw = open(self.vmfs_resig, 'w')
        
        # Iterate through the array self.ds_wwid_head
        for ds_head_val in self.ds_wwid_head:
            fw.write(ds_head_val + '=' + self.ds_wwid_head[ds_head_val] + '\n')

        # Close the file
        fw.close()

        return 0
    # EOF _get_dls_info_()


    ###
    # Function to resginature a device & get the created DS name
    # As in the DLS VMFS|RDM, DS can have more than disks,so read the vmfs_resig
    # file which has just one(first disk from query) DS disk wwid. We need to
    # resignature the disk which has the 'head' meta tag.
    ###
    def _resignature_dev_ds_(self, snap_pv_wwid_array):
        self._DEBUG_("Inside function _resignature_dev_ds_")
        option = "resignature"

        self._DEBUG_("self.vmfs_in: ", self.vmfs_in)
        self._DEBUG_("self.vmfs_resig: ", self.vmfs_resig)
        self._DEBUG_("snap_pv_wwid_array: ", snap_pv_wwid_array)

        ## read the vmfs output file to get required information to create vmdk
        file_arr = []     
        #with open(self.vmfs_resig) as fout:
        fh = open(self.vmfs_resig, "r")
        
             #file_arr = fout.readlines()
        #file_arr = [x.strip() for x in file_arr]
        file_arr = [x.strip() for x in fh.readlines()]

        fw = open(self.vmfs_in, 'w')
        fw.write('UUID:' + self.node_uuid + '\n')

        # Initialize an array to have snapshotpvwwid:originalvmdkname to be used
        snap_pv_orig_ds = {}

        # Go through the file line by line
        for line in file_arr:
            line = line.split('=')
            self._DEBUG_('line: ', line)

            ds_name = line[0]
            ds_wwid = line[1]
            self._DEBUG_("ds_name: ", ds_name)
            self._DEBUG_("ds_wwid: ", ds_wwid)

            try:
                snap_wwid = snap_pv_wwid_array[ds_wwid]
                snap_pv_orig_ds[ds_name] = snap_wwid.lower()
                self._DEBUG_("snap_pv_orig_ds: ", snap_pv_orig_ds[ds_name])

            #except Exception as e:
            except Exception:
                _, e, _ = sys.exc_info()
                self.ret_error = "Unable to get the matching snapshot wwid"
                self._write_to_file_(self.ret_error, 'a')
                return 1

            # write the information to the input file
            fw.write('WWID:' + snap_wwid.upper() + '\n')

        # Close the file
        fw.close()

        parms = "-H " + self.node_esx_vc
        self._DEBUG_("parms: ", parms)

        RC = 0
        RC = self._call_vmwutil_(option, parms)
        if RC:
            self.ret_error = "Unable to create new DS with the snapshot " + \
                                    "volumes at-" + self.node_esx_vc
            self._write_to_file_(self.ret_error, 'a')
            return 1

        ## Parse output & get newly created DS name after resinaturing is done
        for snap_pv_wwid in snap_pv_wwid_array:
            self.pv_wwid = snap_pv_wwid_array[snap_pv_wwid.lower()]
            self._DEBUG_("snap_pv_wwid_before: ", snap_pv_wwid)            
            self.pv_wwid = self.pv_wwid.lower()
            self._DEBUG_("snap_pv_wwid_after: ", self.pv_wwid)
            self._DEBUG_("snap_pv_wwid_val: ",  snap_pv_wwid)

            if self._parse_esx_output_('DLS_RESIGNATURE'): 
                return 1

        ## Write to the vmfs_final file to be used later-on
        # Note:- writing to the final output file to use the same approach as 
        # used for SLS RDM detach
        fw = open(self.vmfs_out, 'w')
        fwfinal = open(self.vmfs_final, 'w')
        self._DEBUG_("self.dls_array: ", self.dls_array)

        for dls_val in self.dls_array:

            self._DEBUG_("dls_val: ", dls_val)
            self._DEBUG_("dls_val['DS']: ", dls_val['DS']) 
            self._DEBUG_("snap_pv_orig_ds: ", snap_pv_orig_ds) 

            DS = self.snap_ds_name_with_wwid[snap_pv_orig_ds[dls_val['DS']]]
            self._DEBUG_('DS :', DS)

            # write to the file
            fw.write('DLS:' + self.disk_type + '|WWID:NA|DS:' + DS + \
                              '|VMDK:'+ dls_val['VMDK'] + '|SLOT:' + \
                              dls_val['PORT'] + '\n')
            fwfinal.write('DLS:' + self.disk_type + '|WWID:NA|DS:' + DS + \
                                   '|VMDK:'+ dls_val['VMDK'] + '|SLOT:' + \
                                   dls_val['PORT'] + '\n')

        # Close the files
        fw.close()
        fwfinal.close()
        return 0
    # EOF _resignature_dev_ds_()


    ###
    # Function to query ESX host or Vcenter & get the required information
    ###
    def _query_esx_vc_(self):
        self._DEBUG_("Inside function _query_esx_vc_")

        ##vmwutil option
        option = "query"
        
        ## get esx wwpn & iscsi details
        if self._get_wwpn_iscsi_(): return 1

        if self.pkg_type == 'DLS':
            if self._get_dls_info_(): return 1
        else:
            if self._get_sls_info_(): return 1

        return 0    
    # EOF _query_esx_vc_()


    ###
    # Function to rescan the esx host as STORAGE_TYPE:hba/vmfs/all 
    ###
    def _rescan_esx_vc_(self, rescan_type):
        self._DEBUG_("Inside function _rescan_esx_vc_")

        # populate info to setup environment variables
        if self._populate_info_(): return 1

        ##vmwutil option
        option = "rescan"
        
        self._DEBUG_("self.vmfs_in: ", self.vmfs_in)
        fw = open(self.vmfs_in, 'w')
        fw.write('UUID:' + self.node_uuid + '\n')

        ## rescan esx_vcenter as per the parameter value
        if rescan_type == 'hba': 
            fw.write('STORAGE_TYPE:hba \n')

        elif rescan_type == 'vmfs': 
            fw.write('STORAGE_TYPE:vmfs \n')

        else:
            # take default as all
            fw.write('STORAGE_TYPE:all \n')

        fw.close()

        parms = "-H " + self.node_esx_vc
        self._DEBUG_("parms: ", parms)

        RC = 0
        RC = self._call_vmwutil_(option, parms)
        if RC:
            error= "Unable to perform " + rescan_type+ " rescan at the esx/vcenter-" +\
                    self.node_esx_vc + ". Please verify manually if its reachable"
            self._write_to_file_(error, 'a')
            return 1

        return 0    
    # EOF _rescan_esx_vc_()

    ###
    # Function to detach vmdk at the esx from vm's
    ###
    def _attach_vmdk_(self, attach_type):
        self._DEBUG_("Inside function _attach_vmdk_")

        ## vmwutil option
        option = "attach"

        # Condition for start rehearsal
        if (attach_type == 'START'):
            # vmfs_out file will be used to start the rehearsal
            # Copy the files to the roll_attach
            try:
                shutil.copy(self.vmfs_out, self.roll_detach)
            except EnvironmentError:
                self._DEBUG_("Unable to create a detach backup file for rollback")
            else:
                pass
            
            ## Verify if the file got copied, as recently we have noticed with
            #  multiple pkgs that this file was not copied.
            if not self._check_file_exist_(self.roll_detach):
                self.ret_error = "Rehearsal failed while creating a required backup" +\
                                 " file"
                self._write_to_file_(self.ret_error, 'a')
                return 1

        # Read the final output file for the cleanup or roll back
        elif (attach_type == 'STOP'):
            self.vmfs_out = self.vmfs_out_bk

        # If attach_type is 'roll_back' then use the roll_attach to attach vmdks
        elif attach_type == 'roll_back':
            self.vmfs_out = self.roll_attach

        self._DEBUG_("self.vmfs_in: ", self.vmfs_in)
        self._DEBUG_("self.vmfs_out: ", self.vmfs_out)

        ## open input file to write
        fw = open(self.vmfs_in, 'w')
        fw.write('UUID:' + self.node_uuid + '\n')

        ## read the vmfs output file to get required information to create vmdk
        file_arr = []
        #with open(self.vmfs_out) as fout:
        #    file_arr = fout.readlines()
        fh = open(self.vmfs_out, "r")
        #file_arr = [x.strip() for x in file_arr]
        file_arr =  [x.strip() for x in fh.readlines()]

        # Go through the file line by line
        self.attach_detach_info_arr = []
        for line in  file_arr:
            if 'SLS:' in line or 'DLS:' in line:
                line = line.split('|')
                self._DEBUG_('line: ', line)
                wwid = line[1].split(':')[1]
                ds   = line[2].split(':')[1]
                vmdk = line[3].split(':')[1]
                slot = line[4].split(':')[1] + ':' + line[4].split(':')[2]

                # write the information to the input file
                fw.write('VMDK:' + vmdk + '\n')
                fw.write('DS:'   + ds   + '\n')
                fw.write('SLOT:' + slot + '\n')

                # Append value
                append_Val = vmdk + '|' + ds + '|' + slot + '|' + wwid
                self.attach_detach_info_arr.append(append_Val)
        ## close the file
        fw.close()

        ## create parameter for vmwutil
        #  -g operation is required for the attach related operation
        parms = "-H " + self.node_esx_vc + ' -g'
        self._DEBUG_("parms: ", parms)

        RC = 0
        RC = self._call_vmwutil_(option, parms)
        if RC:
            ## Return code 8 is for attach error.
            #  We shouldn't call roll_back in-case of any other errors
            if RC == 8:
                error = "Unable to attach vmdk at esx/vcenter- " + self.node_esx_vc 
                self._write_to_file_(error, 'a')

                if attach_type == 'START':
                    # Parse the output & use for roll_back
                    self._vm_roll_back_parse_('snapshot_attach')

                if attach_type == 'STOP' or attach_type == 'roll_back':
                    # Parse the output & use for roll_back
                    self._vm_roll_back_parse_('original_attach')

            ## get other rc & related error messages
            else:
                self._vm_return_code_(RC)
                self._write_to_file_(self.ret_error, 'a')
            return RC
        return 0
    # EOF _attach_vmdk_()

    ###
    # Function to detach vmdk at the esx from vm's
    ###
    def _detach_vmdk_(self, detach_type):
        self._DEBUG_("Inside function _detach_vmdk_")

        ## vmwutil option
        option = "detach"

        # Condition for start rehearsal
        if (detach_type == 'START'):
            # vmfs_out file will be used to start the rehearsal
            # Copy the files to the roll_attach file.
            try:
                shutil.copy(self.vmfs_out, self.roll_attach)
            except EnvironmentError:
                self._DEBUG_("Unable to create a attach backup file for rollback")
            else:
                pass
            
            ## Verify if the file got copied, as recently we have noticed with
            #  multiple pkgs that this file was not copied.
            if not self._check_file_exist_(self.roll_attach):
                self.ret_error= "Rehearsal failed while creating a required backup"+\
                                 " file"
                self._write_to_file_(self.ret_error, 'a')
                return 1

        # Read the final output file for the cleanup or roll back
        elif (detach_type == 'STOP'):
            self.vmfs_out = self.vmfs_final

        # If attach_type is 'roll_back' then use the roll_attach to detach vmdks
        elif detach_type == 'roll_back':
            self.vmfs_out = self.roll_detach

        self._DEBUG_("self.vmfs_in: ", self.vmfs_in)
        self._DEBUG_("self.vmfs_out: ", self.vmfs_out)

        ## open input file to write
        fw = open(self.vmfs_in, 'w')
        fw.write('UUID:' + self.node_uuid + '\n')

        ## read the vmfs output file to get required information to create vmdk
        file_arr = []       
        #with open(self.vmfs_out) as fout:
        #    file_arr = fout.readlines()
        #file_arr = [x.strip() for x in file_arr]
        fh = open(self.vmfs_out, "r")
        file_arr =  [x.strip() for x in fh.readlines()]

        ## Return as success if file_arr is blank, as it will delete all attached
        #  vmdks. Note- vmwutil use to detach all attached vmdks if the input file
        #  won't have any vmdks details.
        if not file_arr and (detach_type == 'START'):
            self.ret_error = "Rehearsal failed while detaching the vmdks as the"+\
                             " required input file for util was empty"
            self._write_to_file_(self.ret_error, 'a')
            return 1
        elif not file_arr:
            return 0

        # Go through the file line by line
        self.attach_detach_info_arr = []
        for line in  file_arr:
            if 'SLS:' in line or 'DLS:' in line:
                line = line.split('|')
                self._DEBUG_('line: ', line)
                wwid = line[1].split(':')[1]
                ds   = line[2].split(':')[1]
                vmdk = line[3].split(':')[1]
                slot = line[4].split(':')[1] + ':' + line[4].split(':')[2]

                # write the information to the input file
                fw.write('VMDK:' + vmdk + '\n')
                fw.write('DS:'   + ds   + '\n')
                fw.write('SLOT:' + slot + '\n')

                # Append value
                append_Val = vmdk + '|' + ds + '|' + slot + '|' + wwid
                self.attach_detach_info_arr.append(append_Val)
        ## close the file
        fw.close()

        ## create parameter for vmwutil
        parms = "-H " + self.node_esx_vc
        self._DEBUG_("parms: ", parms)

        RC = 0
        RC = self._call_vmwutil_(option, parms)
        if RC:
            ## Return code 9 is for detach errors.
            #  We shouldn't call roll_back in-case of any other errors
            if RC == 9:
                error = "Unable to detach vmdk at esx/vcenter- " + self.node_esx_vc
                self._write_to_file_(error, 'a')

                if detach_type == 'START':
                    # Parse the output & use for roll_back
                    self._vm_roll_back_parse_('original_detach')

                if detach_type == 'STOP' or detach_type == 'roll_back':
                    # Parse the output & use for roll_back
                    self._vm_roll_back_parse_('snapshot_detach')

            ## get other rc & related error messages
            else:
                self._vm_return_code_(RC)
                self._write_to_file_(self.ret_error, 'a')
            return RC
        return 0
    # EOF _detach_vmdk_()



    ###
    # Function to destroy detached vmdks at the esx from vm's
    ###
    def _destroy_vmdk_(self):
        self._DEBUG_("Inside function _destroy_vmdk_")

        ## vmwutil option
        option = "destroy"

        # Read the final output file to get the required informations for the 
        # vmdks, which will be destroyed.
        self.vmfs_out = self.vmfs_final

        self._DEBUG_("self.vmfs_in: ", self.vmfs_in)
        self._DEBUG_("self.vmfs_out: ", self.vmfs_out)

        ## open input file to write
        fw = open(self.vmfs_in, 'w')
        fw.write('UUID:' + self.node_uuid + '\n')

        ## read the vmfs output file to get required information to create vmdk
        file_arr = []       
        #with open(self.vmfs_out) as fout:
        #    file_arr = fout.readlines()
        #file_arr = [x.strip() for x in file_arr]
        fh = open(self.vmfs_out, "r")
        file_arr =  [x.strip() for x in fh.readlines()]

        # Go through the file line by line
        for line in  file_arr:
            if 'SLS:' in line or 'DLS:' in line:
                line = line.split('|')
                self._DEBUG_('line: ', line)
                wwid = line[1].split(':')[1]
                ds   = line[2].split(':')[1]
                vmdk = line[3].split(':')[1]
                slot = line[4].split(':')[1] + ':' + line[4].split(':')[2]

                # write the information to the input file
                fw.write('VMDK:' + vmdk + '\n')
                fw.write('DS:'   + ds   + '\n')
                fw.write('SLOT:' + slot + '\n')
        ## close the file
        fw.close()

        ## create parameter for vmwutil
        parms = "-H " + self.node_esx_vc
        self._DEBUG_("parms: ", parms)

        RC = 0
        RC = self._call_vmwutil_(option, parms)
        if RC:
            error = "Unable to destroy vmdk at esx/vcenter- " + self.node_esx_vc
            self._write_to_file_(error, 'a')
            return RC

        return 0
    # EOF _destroy_vmdk_()


    ###
    # Function to create new vmdk at the esx for sls rdm
    # 1.0. Verify if its a DLS RDM pkg & perform some required operation
    # 1.1. Will create & attach all vmdks
    # 2. Will parse the output & get the new vmdk names
    # 3. Write into the final output file to be used while cleanup
    ###
    def _create_vmdk_(self, snap_pv_wwid_array):
        self._DEBUG_("Inside function _create_vmdk_")
    # 1.0. Verify if its a DLS RDM pkg & perform some required operation
        if self._check_file_exist_(self.dls_rdm_file):
            fh = open(self.dls_vmdk_ar, "r")
            pv_slot_arr = {}
            file_arr = [x.strip() for x in fh.readlines()]
            self._DEBUG_(file_arr)
            for line in file_arr:
                #XYZ.vmdk=vml.WWID=SLOT
                self._DEBUG_("line-",line)
                self._DEBUG_("snap_pv_wwid_array-",snap_pv_wwid_array)
                for pv_id in snap_pv_wwid_array:
                    self._DEBUG_("pv_id-", pv_id)
                    if pv_id.lower() in line:
                          self._DEBUG_(pv_id)
                          self._DEBUG_(line.split("=")[2])
                          pv_slot_arr[line.split("=")[2]]=pv_id
            self._DEBUG_(pv_slot_arr)

    # 1.1 Will create & attach all vmdks

        ## vmwutil option
        option = "create"
        
        self._DEBUG_("self.vmfs_in: ", self.vmfs_in)
        self._DEBUG_("self.vmfs_out: ", self.vmfs_out)

        ## Copy the output file to roll_detach file for rollback
        try:
            self._DEBUG_("self.roll_detach: ", self.roll_detach)
            try:
                shutil.copy(self.vmfs_out, self.roll_detach)
            except EnvironmentError:
                self._DEBUG_("Unable to create a detach backup file for rollback")
            else:
                pass
            
            ## Verify if the file got copied, as recently we have noticed with
            #  multiple pkgs that this file was not copied.
            if not self._check_file_exist_(self.roll_detach):
                self.ret_error = "Rehearsal failed while creating a required backup" +\
                                 " file"
                self._write_to_file_(self.ret_error, 'a')
                return 1
        except Exception:
            _, e, _ = sys.exc_info()
            pass

        ## open input file to write
        fw = open(self.vmfs_in, 'w')
        fw.write('UUID:' + self.node_uuid + '\n')

        ## read the vmfs output file to get required information to create vmdk
        file_arr = []       
        #with open(self.vmfs_out) as fout:
        #    file_arr = fout.readlines()
        #file_arr = [x.strip() for x in file_arr]
        fh = open(self.vmfs_out, "r")
        file_arr =  [x.strip() for x in fh.readlines()]

        # Go through the file line by line
        for line in  file_arr:
            if 'SLS:RDM' in line or 'DLS:RDM' in line:
                line = line.split('|')
                self._DEBUG_('line: ', line)
                ds   = line[2].split(':')[1]
                slot = line[4].split(':')[1] + ':' + line[4].split(':')[2]
                
                if self._check_file_exist_(self.dls_rdm_file):
                    wwid = pv_slot_arr[slot]
                else:
                    wwid = line[1].split(':')[1]

                # Get the value corresponding to the wwid from the array
                self._DEBUG_("snap_pv_wwid_array: ", snap_pv_wwid_array)
                self._DEBUG_("wwid :", wwid)
                try:
                    snap_wwid = snap_pv_wwid_array[wwid.upper()]
                #except Exception as e:
                except Exception:
                    _, e, _ = sys.exc_info()
                    self.ret_error = "Unable to get the matching snapshot wwid"
                    self._write_to_file_(self.ret_error, 'a')
                    return 1

                # write the information to the input file
                fw.write('WWID:' + snap_wwid.lower() + '\n')
                fw.write('DS:'   + ds   + '\n')
                fw.write('SLOT:' + slot + '\n')

                # append snap_wwid value to the array to be used later-on
                self.snap_wwid_arr.append(snap_wwid.lower())
                final_out = 'SLS:RDM|WWID:' + snap_wwid.lower() + '|DS:' + ds +\
                             '|VMDK:NA|SLOT:' + slot
                self.final_out_array.append(final_out)

        ## close the file
        fw.close()

        ## create parameter for vmwutil
        parms = "-H " + self.node_esx_vc
        self._DEBUG_("parms: ", parms)

        RC = 0
        RC = self._call_vmwutil_(option, parms)
        if RC:
            ## Return code 20 is for create vmdk error.
            #  We shouldn't call roll_back in-case of any other errors
            if RC == 20:
                self.ret_error = "Unable to create & attach vmdk at esx/" +\
                                                  "vcenter- " + self.node_esx_vc
                self._write_to_file_(self.ret_error, 'a')

                # Parse the output & use for roll_back
                self._vm_roll_back_parse_('snapshot_attach')

            ## get other rc & related error messages
            else:
                self._vm_return_code_(RC)
                self._write_to_file_(self.ret_error, 'a')
            return RC

    # 2. Will parse the output & get the new vmdk names
        ## Parse output & get DS associated wwid's
        if self._parse_esx_output_('SLS_VMDK'):
            # change return code later-on
            return 1

    # 3. Write into the final output file to be used while cleanup or end of reh
        # open the final output file to write
        fw = open(self.vmfs_final, 'w')
 
        # Iterate through the create final output array & replace the vmdk name 
        # with the created new vmdk names
        for final in self.final_out_array:
            final = final.split('|')

            # Get the wwid from the splitted line to get the vmdk from the array
            wwid_to_be_used = final[1].split(':')[1]
            self._DEBUG_("wwid_to_be_used: ", wwid_to_be_used)
            vmdk_new = self.sls_snap_vmdk_array[wwid_to_be_used]
            ds_new   = final[2].split(':')[1]
            slot_new = final[4].split(':')[1] + ':' + final[4].split(':')[2]
            self._DEBUG_('vmdk_new_name: ', vmdk_new)
            self._DEBUG_('ds_new: ', ds_new)
            self._DEBUG_('slot_new: ', slot_new)

            # Write all the enteries to the file format detach function wll work
            fw.write('SLS:RDM|WWID:' + wwid_to_be_used + '|DS:' + ds_new + \
                            '|VMDK:' + vmdk_new + '|SLOT:' + slot_new + '\n')

        ## close the file
        fw.close()

        return 0    
    # EOF _create_vmdk_()


    ###
    # Function to execute vmwutil with parameters
    ###
    def _call_vmwutil_(self, option, parms):
        jar = self.p.SGSBIN + '/vmwutil'
        cmd = 'java -jar ' + jar
        self._DEBUG_("Jar file: ", cmd)

        # Query can be done using the UUID only
        #if (option == 'query'):
        #    value = ' -o query ' + parms + ' &> ' + self.vmfs_dump

        # Connect doesn't need any parameter
        if (option == 'connect'):
            value = ' -o connect ' + parms

        # For all others
        else:
            file_parm_w_val = '-F ' + self.vmfs_in
            self._DEBUG_("-F with parameter: ", file_parm_w_val)

            value = ' -o ' + option + ' ' + parms + ' ' + file_parm_w_val + \
                                                         ' &> ' + self.vmfs_dump

        self._DEBUG_('command complete value: ', value)

        ## There are some scenarios in which ESX/vsphere took long time (more than
        #  10 mins to complete. Hence increasing the timeout to 20 mins for all cmds
        output, p_status, err = self._cmd_exec_(name = cmd, value = value, timeout=1200)
        if p_status:
            return p_status

        return 0
    # EOF _call_vmwutil_()


    ###
    # Function to parse vmwutil output
    # 1. type==HBA
    # 2. type==DLS_VMFS
    # 3. type==DLS_RESIGNATURE
    # 4. type==SLS_VMDK
    # 5. type==SLS_DLS
    # 6. type==SLS_CREATE
    # 7. type==SLS_DLS_DETACH_ATTACH
    ###
    def _parse_esx_output_(self, type):
        self._DEBUG_("Inside function _parse_esx_output_")

        ## Read the output file
        file_arr = []
        #with open(self.vmfs_dump) as fr:
        #    file_arr = fr.readlines()
        #file_arr = [x.strip() for x in file_arr]
        fh = open(self.vmfs_dump, "r")
        file_arr =  [x.strip() for x in fh.readlines()]

    # 1. type==HBA
        ## Get HBA information
        if type == 'HBA':
            for line in file_arr:
                if 'wwpn' in line or 'iqn' in line:
                    line = line.split('|')
                    self._DEBUG_('line: ', line)
                    self.hba_array.append(line[2].split('=')[1])

            # return non-zero value if the self.hba_array is empty
            self._DEBUG_("hba_array: ", self.hba_array)
            if not self.hba_array:
                self.ret_error = 'Unable to get the ESX wwpn/iscsi details'
                self._write_to_file_(self.ret_error, 'a')
                return 1
            else: return 0

    # 2. type==DLS_VMFS
        ## Get DLS DS disk wwid's
        if type == 'DLS_VMFS':
            dls_wwid_head_flag = True
            for line in file_arr:
                search_val = self.ds_name + '|wwn='

                if search_val in line:
                    self._DEBUG_("search_val: ", search_val)

                    # Split the line to convert to an array
                    line = line.split('|')
                    self._DEBUG_('line: ', line)

                    # Split to get the pv wwid
                    ds_wwid_val = line[2].split('=naa.')[1]
                    self._DEBUG_('ds_wwid_val: ', ds_wwid_val)

                    # Append ds_wwid_val to the array
                    self.ds_wwn_array.append(ds_wwid_val)

                    ## write only the first matched DS wwid to the array to be 
                    # used for re-signature. If a DS has multiple pv attached
                    # then head information will be only with the first pv.
                    if dls_wwid_head_flag:
                        self.ds_wwid_head[self.ds_name] = ds_wwid_val
                        dls_wwid_head_flag = False

            # return non-zero value if the self.ds_wwn_array is empty
            self._DEBUG_("ds_wwn_array: ", self.ds_wwn_array)
            if not self.ds_wwn_array:
                self.ret_error = 'Unable to get the DS wwid details'
                self._write_to_file_(self.ret_error, 'a')
                return 1
            else: return 0

    # 3. type==DLS_RESIGNATURE
        ## Get DLS DS name after resinaturing is done
        if type == 'DLS_RESIGNATURE':
            dls_wwid_head_flag = True
            search_val = '|Disk:' + self.pv_wwid.lower() + '|ds_name='
            self._DEBUG_("search_val: ", search_val)
            for line in file_arr:
                search_val = '|Disk:' + self.pv_wwid.lower() + '|ds_name='

                if search_val in line:
                    self._DEBUG_("search_val: ", search_val)

                    # Split the line to convert to an array
                    line = line.split('|')
                    self._DEBUG_('line: ', line)

                    # Split to get the snap_ds_name
                    snap_ds_name = line[2].split('=')[1]
                    self._DEBUG_('snap_ds_name: ', snap_ds_name)

                    # Append ds_wwid_val to the array
                    self.snap_ds_name_with_wwid[self.pv_wwid] = snap_ds_name
					
                ## for errors
                error_search_val = '|Disk:' + self.pv_wwid.lower() + '|error='
                if error_search_val in line:
                    line  = line.split("=")
                    error = line[1]
                    self._write_to_file_(error, 'a')

            # return non-zero value if the self.snap_ds_name_with_wwid is empty
            self._DEBUG_("snap_ds_name_with_wwid: ",self.snap_ds_name_with_wwid)
            if not self.snap_ds_name_with_wwid:
                self.ret_error = 'Unable to get the snapshot DS name'
                self._write_to_file_(self.ret_error, 'a')
                return 1
            else: return 0

    # 4. type==SLS_VMDK
        ## Get SLS vmdk attached information
        if type == 'SLS_VMDK':
            # Iterate through self.snap_wwid_arr array to get the vmdks values
            for snap_wwid in self.snap_wwid_arr:
                search_val_vmdk = '|Disk:' + snap_wwid + '|file_name='
                self._DEBUG_("search_val_vmdk: ", search_val_vmdk)

                for line in file_arr:
                    if search_val_vmdk in line:
                        line = line.split('|')
                        self._DEBUG_('line: ', line)
                        self.sls_snap_vmdk_array[snap_wwid] = \
                                                line[2].split('=')[1].split()[1]
                        
            # return non-zero value if the self.sls_snap_vmdk_array is empty
            self._DEBUG_("sls_snap_vmdk_array: ", self.sls_snap_vmdk_array)
            if not self.sls_snap_vmdk_array:
                self.ret_error= 'Unable to get the created snapshot vmdk details'
                self._write_to_file_(self.ret_error, 'a')
                return 1
            else: return 0

    # 5. type==SLS_INFO
        ## Get DS Name, VMDK name & slot info for the SLS configuration
        if type == 'SLS_INFO':
            ds_vmdk_flag = False
            sls_slot_flag= False
            availability_flag = False
            action_flag = False
            attached_flag = False
            attached_sharing = False

            ## get ds & vmdk information
            for line in file_arr:
                if '|Datastore:' in line:
                    search_val = self.pv_wwid.lower()
                    if search_val in line:
                        self._DEBUG_("search_val: ", search_val)
                        line = line.split('|')
                        self._DEBUG_('line: ', line)

                        # Get DS name by splitting ':' in the 1st element
                        ds = line[1].split(':')[1]
                        self._DEBUG_('datastore name=', ds)

                        # Get vmdk name by splitting ':' in the 2nd element
                        vmdk = line[2].split(':')[1]
                        self._DEBUG_('vmdk name=', vmdk)
                        ds_vmdk_flag = True
                        break

            if not ds_vmdk_flag:
                self.ret_error = "Unable to find the rdm device " +\
                                                      "in the query output file"
                self._write_to_file_(self.ret_error, 'a')
                return 1

            ## get slot & availability information for respective vmdk 
            #  attached to the host
            for line in file_arr:
                ## 1. Get slot number & Availability
                search_val_slot = '|Datastore:' + ds + '|File:' + vmdk + '|Slot'
                search_val_act= '|Datastore:' + ds + '|File:' + vmdk + '|action'
                search_val_sharing = '|Datastore:' + ds + '|File:' + vmdk + \
                                                      '|controller_sharing_mode'
                search_val_attached = '|Datastore:' + ds + '|File:' + vmdk + \
                                                      '|attached'

                if search_val_act in line:
                    self._DEBUG_("search_val_act: ", search_val_act)

                    line = line.split('|')
                    self._DEBUG_('line: ', line)
                    action = line[3].split('=')[1]
                    self._DEBUG_('action: ', action)
                    action_flag = True

                if search_val_sharing in line:
                    self._DEBUG_("search_val_sharing: ", search_val_sharing)

                    ## Check for name word in the line
                    line = line.split('|')
                    self._DEBUG_('line: ', line)
                    controller_sharing_mode = line[3].split('=')[1]
                    self._DEBUG_('controller_sharing_mode: ', controller_sharing_mode)
                    attached_sharing = True

                if search_val_attached in line:

                    self._DEBUG_("search_val_attached: ", search_val_attached)
                    ## Check for name word in the line

                    line = line.split('|')
                    self._DEBUG_('line: ', line)
                    attached = line[3].split('=')[1]
                    self._DEBUG_('attached: ', attached)
                    attached_flag = True

                if search_val_slot in line:
                    self._DEBUG_("search_val_slot: ", search_val_slot)
                    ## Check for name word in the line
                    if 'name' in line:
                        line = line.split('|')
                        self._DEBUG_('line: ', line)
                        slot = line[4].split('=')[1]
                        self._DEBUG_('slot number:', slot)
                        sls_slot_flag = True

                    ## Check for availability word in the line
                    if 'availability' in line:
                        line = line.split('|')
                        self._DEBUG_('line: ', line)
                        availability = line[4].split('=')[1]
                        self._DEBUG_('availability:', availability)
                        availability_flag = True

                    # Break if all flags are True
                    if action_flag and attached_sharing and attached_flag \
                                        and availability_flag and sls_slot_flag:
                        break
            
            if not sls_slot_flag or not availability_flag:
                self.ret_error = "Unable to get the attached slot " + \
                                               "or the availability information"
                self._write_to_file_(self.ret_error, 'a')
                return 1

            
            if self.pkg_type == 'SLS':
                val_to_be_appended = 'SLS:RDM|WWID:' + self.pv_wwid + '|DS:' + \
                                      ds + '|VMDK:' + vmdk + '|SLOT:' + slot + \
                                 '|AVAILABILITY:' + availability + '|action:' +\
                                  action + '|controller_sharing_mode:' + \
                                  controller_sharing_mode + '|attached:'+ \
                                  attached
            else:
                val_to_be_appended = 'DLS:' + self.disk_type + '|WWID:' + \
                                  self.pv_wwid + '|DS:' + ds + '|VMDK:' + \
                                  vmdk + '|SLOT:' + slot + '|AVAILABILITY:' + \
                                  availability + '|action:' + action + \
                                  '|controller_sharing_mode:' + \
                                  controller_sharing_mode + '|attached:'+ \
                                  attached
            self.sls_dls_rdm_info.append(val_to_be_appended)

    # 6. type==SLS_CREATE
        ## Get SLS created vmdks information
        # sls_dls_rdm_info array will contain a list of ACTION=SUCCEED vmdks
        # sls_dls_rdm_other_info array will contain a list of other vmdks
        if type == 'SLS_CREATE':
            self.sls_dls_rdm_info  = []

            # Iterate through self.snap_wwid_arr array to get the vmdks values
            for snap_wwid in self.snap_wwid_arr:
                cmd  = 'sh ' + self.vmfsutilsbin
                parm = '-create ' +  self.vmfs_dump + ' ' + snap_wwid
                output, p_status, err = self._cmd_exec_(name= cmd, value = parm)

                #WWID|DS|SLOT|ACTION|DS XYZ.VMDK|ERROR
                output = output.split("|")
                self._DEBUG_("output: ", output)
                action    = output[3]
                vmdk_name = output[4]

                # Get the created and attached vmdk information
                if action == 'SUCCEED':
                    vmdk_name = output[4].split()[1]
                    val_to_be_appended = self.pkg_type + ':' + self.disk_type +\
                                    '|WWID:' + output[0] + '|DS:' + output[1] +\
                                    '|VMDK:' + vmdk_name + '|SLOT:' + output[2] 
                    self.sls_dls_rdm_info.append(val_to_be_appended)

                # Capture all the failure messages in an Array.
                elif action == 'FAILED':
                    error = output[5]
                    self._DEBUG_("ERROR MSG:- ", error)
                    self._write_to_file_(error, 'a')

                    val_to_be_appended = self.pkg_type + ':' + self.disk_type +\
                                    '|WWID:' + output[0] + '|DS:' + output[1] +\
                                    '|VMDK:' + vmdk_name + '|SLOT:' + output[2] 
                    self.sls_dls_rdm_other_info.append(val_to_be_appended)
                else:
                    val_to_be_appended = self.pkg_type + ':' + self.disk_type +\
                                    '|WWID:' + output[0] + '|DS:' + output[1] +\
                                    '|VMDK:' + vmdk_name + '|SLOT:' + output[2] 
                    self.sls_dls_rdm_other_info.append(val_to_be_appended)
                    # 

            self._DEBUG_("sls_dls_rdm_info: ",  self.sls_dls_rdm_info)
            self._DEBUG_("sls_dls_rdm_other_info: ",  self.sls_dls_rdm_other_info)

    # 7. type==SLS_DLS_DETACH_ATTACH
        ## Get SLS or DLS created vmdks information
        # sls_dls_rdm_info array will contain a list of ACTION=SUCCEED vmdks
        # sls_dls_rdm_other_info array will contain a list of other vmdks
        if type == 'SLS_DLS_DETACH_ATTACH':
            self.sls_dls_rdm_info  = []

            # Iterate through self.attach_detach_info_arr array to get the 
            # vmdks, ds, slot values
            for attach_detach_info in self.attach_detach_info_arr:
                # Split the attach_detach_info_arr to get the req values
                attach_detach_info = attach_detach_info.split("|")
                vmdk = attach_detach_info[0]
                ds   = attach_detach_info[1]
                slot = attach_detach_info[2]
                wwid = attach_detach_info[3]

                # Execute script to get the required information
                cmd  = 'sh ' + self.vmfsutilsbin
                parm ='-attach_detach ' +  self.node_uuid + ' ' + vmdk + ' ' + ds
                output, p_status, err = self._cmd_exec_(name= cmd, value = parm)

                #XYZ.vmdk|ATTACHED|ERROR|ACTION
                output = output.split("|")
                self._DEBUG_("output: ", output)
                action    = output[3]
                vmdk_name = output[0]

                # Get the created and attached vmdk information
                if action == 'SUCCEED':
                    val_to_be_appended = self.pkg_type + ':' + self.disk_type +\
                                     '|WWID:' + wwid + '|DS:' + ds + '|VMDK:' +\
                                     vmdk_name + '|SLOT:' + slot
                    self.sls_dls_rdm_info.append(val_to_be_appended)

                # Capture all the failure messages in an Array.
                elif action == 'FAILED':
                    error = output[2]
                    self._write_to_file_(error, 'a')
                    val_to_be_appended = self.pkg_type + ':' + self.disk_type +\
                                    '|WWID:' + output[0] + '|DS:' + output[1] +\
                                    '|VMDK:' + vmdk_name + '|SLOT:' + output[2] 
                    self.sls_dls_rdm_other_info.append(val_to_be_appended)

                else:
                    val_to_be_appended = self.pkg_type + ':' + self.disk_type +\
                                    '|WWID:' + output[0] + '|DS:' + output[1] +\
                                    '|VMDK:' + vmdk_name + '|SLOT:' + output[2] 
                    self.sls_dls_rdm_other_info.append(val_to_be_appended)

            self._DEBUG_("sls_dls_rdm_info: ",  self.sls_dls_rdm_info)
            self._DEBUG_("sls_dls_rdm_other_info: ",  self.sls_dls_rdm_other_info)

        return 0
    # EOF _parse_esx_output_()


    ###
    # Function to setup ESX related env
    # 1. _populate_info_: To populate all required information for ESX/Vcenter
    # 2. _query_esx_vc_ : Query ESX/Vcenter
    # 3. Write all required information to the output
    ###
    def _setup_esx_(self):
    # 1. _populate_info_: To populate all required information for ESX/Vcenter
        if self._populate_info_(): return 1

    # 2. _query_esx_vc_ : Query ESX/Vcenter
        if self._query_esx_vc_(): return 1

    # 3. Write all required information to the output
        fw = open(self.vmfs_out, 'w')

        for sls_dls_rdm_info in self.sls_dls_rdm_info:
            fw.write(sls_dls_rdm_info + '\n')

        fw.close()

        return 0
    # EOF _setup_()


    ###
    # Function _roll_back_parsing_()- is helpful in-case of a failure. It will roll
    # back complete steps 
    # 1. type == 'original_detach'
    # 2. type == 'snapshot_attach'
    # 3. type == 'snapshot_detach'
    # 4. type == 'original_attach'
    ###
    def _vm_roll_back_parse_(self, type):
        ## Roll back called from original detach function
        if type == 'original_detach':
            # For original vmdk detach
            # Parsing will be done using the vmfs_dump file & it will return
            # an array. 
            self._parse_esx_output_('SLS_DLS_DETACH_ATTACH')

            #SLS:RDM|WWID:XXX|DS:XXX|VMDK:XXX|SLOT:XXX
            fw = open(self.roll_attach, 'w')
            for sls_dls_rdm_info in self.sls_dls_rdm_info:
                fw.write(sls_dls_rdm_info + '\n')
            fw.close()

        ## Roll back called from snapshot attach (create & attach)function
        if type == 'snapshot_attach':
            # For snapshot vmdk attach
            # Parsing will be done using the vmfs_dump file & it will return
            # an array. 
            self._parse_esx_output_('SLS_CREATE')

            #SLS:RDM|WWID:XXX|DS:XXX|VMDK:XXX|SLOT:XXX
            fw = open(self.roll_detach, 'w')
            for sls_dls_rdm_info in self.sls_dls_rdm_info:
                fw.write(sls_dls_rdm_info + '\n')
            fw.close()

        ## Roll back called from snapshot detach function
        if type == 'snapshot_detach':
            # For snapshot vmdk detach
            # Parsing will be done using the vmfs_dump file & it will return
            # an array. 
            self._parse_esx_output_('SLS_DLS_DETACH_ATTACH')

            #SLS:RDM|WWID:XXX|DS:XXX|VMDK:XXX|SLOT:XXX
            fw = open(self.roll_detach, 'w')
            for sls_dls_rdm_info in self.sls_dls_rdm_info:
                fw.write(sls_dls_rdm_info + '\n')
            fw.close()

        ## Roll back called from original attach function
        if type == 'original_attach':
            # For original vmdk attach
            # Parsing will be done using the vmfs_dump file & it will return
            # an array. 
            self._parse_esx_output_('SLS_DLS_DETACH_ATTACH')

            # SLS:RDM|WWID:XXX|DS:XXX|VMDK:XXX|SLOT:XXX
            # Iterate through sls_dls_rdm_other_info array to maintain a list of
            # the not attached original vmdks
            fw = open(self.roll_attach, 'w')
            for sls_dls_rdm_other_info in self.sls_dls_rdm_other_info:
                fw.write(sls_dls_rdm_other_info + '\n')
            fw.close()
        return 0
    # EOF _vm_roll_back_parse_()


    ###
    # vmwutil related error messages
    ###
    def _vm_return_code_(self, code):
        if code == '1':
            self.ret_error ="Usage error"

        elif code == '2':
            self.ret_error ="Unable to parse the input file correctly."

        elif code == '3':
            self.ret_error ="Given UUID doesn't exist."

        elif code == '4':
            self.ret_error ="Configuration issue for the used input file details."

        elif code == '5':
            self.ret_error ="Failed to populate the object"

        elif code == '6':
            self.ret_error ="Added ESX/Vcenter credentials is wrong"

        elif code == '7':
            self.ret_error ="ESX/Vcenter is not reachable"

        elif code == '8':
            self.ret_error ="Failed to attached the disk"

        elif code == '9':
            self.ret_error ="Failed to detach the disk"

        elif code == '10':
            self.ret_error ="Incorrect Jave version."

        elif code == '11':
            self.ret_error ="Input file is corrupted"

        elif code == '13':
            self.ret_error ="No Vcenter configured to perform the rehearsal."

        elif code == '14':
            self.ret_error ="Given targer host is not eligible."

        elif code == '18':
            self.ret_error ="Unable to rescan the esx_host or venter."

        elif code == '19':
            self.ret_error ="Failed to resignature the datastore."

        elif code == '20':
            self.ret_error ="Failed to create SLS RDM vmdk's."

        else:
            self.ret_error ="Other configuration issue with ESX/Vcenter"
    # EOF _vm_return_code_(code)

# EOC VMWARE()
