#!/usr/bin/env python
#####################################################################
# (C) Copyright 2017 Hewlett Packard Enterprise Development LP.
# @(#) MC DRR 3PAR script.
# @(#) Product Name                :
# @(#) Product Version             : B.12.10.00
# @(#) Patch Name                  :
#
# NOTE:    This script is not configurable! Any changes made to this
#          script will be overwritten when you upgrade to the next
#          release of Serviceguard metrocluster addons.
#
#####################################################################

## Import Helper 
from drrhelper import Helper
import sys, os

###
# 3PAR CLASS
###
class MC3PAR(Helper):

    ###
    # INIT
    ###
    def __init__(self, pkg_name, drr_id, pkg_array, pkg_node_name, DEBUG, err_file):
        Helper.__init__(self, DEBUG, pkg_name, drr_id, err_file)
        self._source_config_file_("/etc/cmcluster.conf")
        ## Standby array 
        self.os_version            = "NA"
        self.s_nodelist            = ""
        self.s_storage_name        = ""
        self.s_storage_user_name   = ""
        self.s_storage_rcvg_name   = ""
        self.s_storage_target_name = ""

        ## Primary array
        self.p_nodelist            = ""
        self.p_storage_name        = ""
        self.p_storage_user_name   = ""
        self.p_storage_rcvg_name   = ""
        self.p_storage_target_name = ""

        ## DC1 & DC2 Flag
        self.drr_dc1_flag          = False
        self.drr_dc2_flag          = False

        ## standby & primary array cli or ssh variable
        self.s_cli_ssh             = ""
        self.p_cli_ssh             = ""

        ## Other variables
        self.sv_wwid               = ""
        self.rcvg_vv_array         = []
        self.pkg_name              = pkg_name
        self.pkg_array             = pkg_array
        self.pkg_node_name         = pkg_node_name
        self.vm_vv_wwid_array      = {}
        self.orig_vv_replicated_vv_array = {}

    # EOF __init__()
    
    
    ###
    # Function to get all array related details from a given rcvg
    ###
    def _get_rcvg_details_(self):
        ## Getting RCVG & other details required from Package.
        self.RC3PAR_DC1_NODE_LIST=self.pkg_array['RC3PAR_DC1_NODE_LIST'].split()
        self._DEBUG_("RC3PAR_DC1_NODE_LIST: ", \
                                                      self.RC3PAR_DC1_NODE_LIST)
        for DC1_node in self.RC3PAR_DC1_NODE_LIST :
           if ( DC1_node == self.pkg_node_name ):
               self.drr_dc1_flag = True
               break

        self.RC3PAR_DC2_NODE_LIST=self.pkg_array['RC3PAR_DC2_NODE_LIST'].split()
        self._DEBUG_("RC3PAR_DC2_NODE_LIST: ", \
                                                      self.RC3PAR_DC2_NODE_LIST)
        if not self.drr_dc1_flag :
           for DC2_node in self.RC3PAR_DC2_NODE_LIST :
               if ( DC2_node == self.pkg_node_name ):
                   self.drr_dc2_flag = True
                   break
                   
        if self.drr_dc1_flag:
            # DC2 details will be used as standby & DC1 as primary array.
            self.s_nodelist              = self.RC3PAR_DC2_NODE_LIST
            self.s_storage_name          = \
                                self.pkg_array['RC3PAR_DC2_STORAGE_SYSTEM_NAME']
            self.s_storage_user_name     = \
                                self.pkg_array['RC3PAR_DC2_STORAGE_SYSTEM_USER']
            self.s_storage_rcvg_name     = \
                                    self.pkg_array['RC3PAR_DC2_RC_VOLUME_GROUP']
            self.s_storage_target_name   = \
                                  self.pkg_array['RC3PAR_DC2_RC_TARGET_FOR_DC1']
            # Primary details
            self.p_nodelist            = self.RC3PAR_DC1_NODE_LIST
            self.p_storage_name        = \
                                self.pkg_array['RC3PAR_DC1_STORAGE_SYSTEM_NAME']
            self.p_storage_user_name   = \
                                self.pkg_array['RC3PAR_DC1_STORAGE_SYSTEM_USER']
            self.p_storage_rcvg_name   = \
                                self.pkg_array['RC3PAR_DC1_RC_VOLUME_GROUP']
            self.p_storage_target_name = \
                                self.pkg_array['RC3PAR_DC1_RC_TARGET_FOR_DC2']
        elif self.drr_dc2_flag:
            # DC1 details will be used as standby & DC2 details as primary array.
            self.s_nodelist              = self.RC3PAR_DC1_NODE_LIST
            self.s_storage_name          = \
                                self.pkg_array['RC3PAR_DC1_STORAGE_SYSTEM_NAME']
            self.s_storage_user_name     = \
                                self.pkg_array['RC3PAR_DC1_STORAGE_SYSTEM_USER']
            self.s_storage_rcvg_name     = \
                                self.pkg_array['RC3PAR_DC1_RC_VOLUME_GROUP']
            self.s_storage_target_name   = \
                                self.pkg_array['RC3PAR_DC1_RC_TARGET_FOR_DC2']
            # Primary array details
            self.p_nodelist            = self.RC3PAR_DC2_NODE_LIST
            self.p_storage_name        = \
                                self.pkg_array['RC3PAR_DC2_STORAGE_SYSTEM_NAME']
            self.p_storage_user_name   = \
                                self.pkg_array['RC3PAR_DC2_STORAGE_SYSTEM_USER']
            self.p_storage_rcvg_name   = \
                                self.pkg_array['RC3PAR_DC2_RC_VOLUME_GROUP']
            self.p_storage_target_name = \
                                self.pkg_array['RC3PAR_DC2_RC_TARGET_FOR_DC1']
        else:
            self.ret_error = "Unable to get the current RCVG details for the"+\
                            " mc3par package " + '"' + self.pkg_name + '"'
            return 112

    # EOF _get_rcvg_details_()


    ###
    # Function to check if CLI/SSH connection for both the arrays as per there
    # InForm OS version
    ###
    def _get_cli_ssh_(self):
        self._DEBUG_("Inside function _check_cli_")
        trySshFlag1 = True
        trySshFlag2 = True
        versioncmd  = 'showversion -s'
        DC1_sys     = self.s_storage_name
        DC2_sys     = self.p_storage_name
        DC1_user    = self.s_storage_user_name
        DC2_user    = self.p_storage_user_name

        ## Condition for RH5
        RH5_Flag = False
        self._get_os_version_()
        if self.os_version == '5':
            RH5_Flag = True

        # Check if cli binary exist
        if (self._check_file_exist_('/usr/bin/cli')):
            # check if both pwf file's are present
            pwf_dir = self.SGCONF + "/3parrc/"
            # check if 3parrc directory exist
            if (self._check_dir_exist_(pwf_dir)):
                # check if pwf file's exist in the directory
                DC1_pwf  = pwf_dir + DC1_user + "_" + DC1_sys + ".pwf"
                DC2_pwf  = pwf_dir + DC2_user + "_" + DC2_sys + ".pwf"
                self._DEBUG_("DC1_pwf: ", DC1_pwf)
                #/usr/local/cmcluster/conf/3parrc/mc3par_15.213.97.208.pwf
                self._DEBUG_("DC2_pwf: ", DC2_pwf)
                #/usr/local/cmcluster/conf/3parrc/metrouser2_15.213.161.42.pwf

                if (self._check_file_exist_(DC1_pwf)):
                    DC1_pwf_path = '/usr/bin/cli' + " " + "-sys " + DC1_sys + \
                                                              " -pwf " + DC1_pwf
                    self._DEBUG_("DC1_pwf_path: ", DC1_pwf_path)
                    #/usr/bin/cli -sys 15.213.97.208 -pwf /usr/local/cmcluster/
                    #conf/3parrc/mc3par_15.213.97.208.pwf
                    output1, rcode1, err1 = self._cmd_exec_\
                                       (name = DC1_pwf_path, value = versioncmd)
                    if ( rcode1 != 0 ):
                        trySshFlag1 = True
                        rcode1 = 1
                    else :
                        trySshFlag1 = False

                if (self._check_file_exist_(DC2_pwf)):
                    DC2_pwf_path = '/usr/bin/cli' + " " + "-sys " + DC2_sys + \
                                                              " -pwf " + DC2_pwf
                    self._DEBUG_("DC2_pwf_path: ", DC2_pwf_path)
                    #/usr/bin/cli -sys 15.213.161.42 -pwf /usr/local/cmcluster/
                    #conf/3parrc/metrouser2_15.213.161.42.pwf
                    output2, rcode2, err2 = self._cmd_exec_\
                                       (name = DC2_pwf_path, value = versioncmd)
                    if ( rcode2 != 0 ):
                        trySshFlag2 = True
                        rcode2 = 1
                    else :
                        trySshFlag2 = False
            
                #return rcode1, rcode2, output1, output2
                #else :
                    #print "pwf file doesn't exist"
            #else :
                #print "directory doesn't exist"

        # Try SSH to connect to the array
        if ( trySshFlag1 ):
            DC1_SSH_CMD = "ssh" + " " + DC1_user + "@" + DC1_sys
            output1, rcode1, err1 = self._cmd_exec_(name = DC1_SSH_CMD, \
                                                             value = versioncmd)        
        if ( trySshFlag2 ):
            DC2_SSH_CMD = "ssh" + " " + DC2_user + "@" + DC2_sys
            output2, rcode2, err1 = self._cmd_exec_(name = DC2_SSH_CMD, \
                                                             value = versioncmd)

        # Compare & decided SSH or CLI has to be used
        self._DEBUG_("DC1: ", output1)
        self._DEBUG_("DC2: ",output2)
        #3.3.1
        #3.2.2

        if (output1 >= "3.3.1") and not RH5_Flag:
            output1 = "ssh -l " + DC1_user + " " + DC1_sys
        else :
            output1 = "/usr/bin/cli -sys " + DC1_sys + " -pwf " + DC1_pwf

        if (output2 >= "3.3.1") and not RH5_Flag:
            output2 = "ssh -l " + DC2_user + " " + DC2_sys
        else :
            output2 = "/usr/bin/cli -sys " + DC2_sys + " -pwf " + DC2_pwf

        self._DEBUG_("rcode1: ", rcode1, "rcode2: ", \
                                rcode2, "output1: ", output1, \
                                "output2: ", output2)
        #0 0 ssh -l mc3par 15.213.97.208 /usr/bin/cli -sys 15.213.161.42 -pwf 
        # /usr/local/cmcluster/conf/3parrc/metrouser2_15.213.161.42.pwf
        return rcode1, rcode2, output1, output2
    # EOF _get_cli_ssh_()


    ###
    # Function to check if a virtual volume is part of a volume set
    ###
    def _is_vvset_(self, vv_name, cli_ssh):
        self._DEBUG_("Inside function _is_vvset_")
        value="showvvset " + vv_name + " -csvtable | awk -F\",\" \'{print $3}\'"
        #value = "showvvset " + vv_name
        self._DEBUG_("value: ", value)
        output, p_status, err = self._cmd_exec_(name=cli_ssh , value=value)
        if not output:
            self._DEBUG_("No vv set listed")
            return 1
        output = self._parse_array_(output)
        self._DEBUG_("output: ", output)
        return 0
    # EOF _is_vvset_()


    ###
    # Function to get the virtual volume w.r.t to the vlun 
    ###
    def _get_vv_info_(self, vv_detail, cli_ssh):
        self._DEBUG_("Inside function _get_vv_info_")
        self._DEBUG_("vv_detail: ", vv_detail)
        global vv_details_arr
        vv_details_arr = {}
        global vv_detail_array
        vv_detail_array = vv_detail
        vv_name = vv_detail_array[1]
        # [Lun,VVName,HostName,Port,Type]

        vv_details_arr['STEPS_COMPLETED']  = 1
        vv_details_arr['LUN_ID']   = vv_detail_array[0]
        vv_details_arr['VV_NAME']  = vv_detail_array[1]
        vv_details_arr['HOST']     = vv_detail_array[2]
        vv_details_arr['HOST_SET'] = "NA"
        vv_details_arr['PORT']     = vv_detail_array[3]

        # Check for different presentation type
        if (vv_detail_array[4] == "host set"):
            vv_details_arr['PRES_TYPE'] = "host_set"
        elif (vv_detail_array[4] == "port"):
            vv_details_arr['PRES_TYPE'] = "port_set"
        elif (vv_detail_array[4] == "matched set"):
            vv_details_arr['PRES_TYPE'] = "matched_set"
        elif (vv_detail_array[4] == "host"):
            vv_details_arr['PRES_TYPE'] = "host_sees"
        else :
            vv_details_arr['PRES_TYPE'] = "NA"

        # Check if its a vv set or just a vv
        if self._is_vvset_(vv_detail_array[1], cli_ssh):
            vv_details_arr['VV_SET'] = "NA"
        else:
            vv_details_arr['VV_SET'] = vv_name

        self._DEBUG_("vv_details_arr: ", vv_details_arr)
        return vv_details_arr
    # EOF _get_vv_info_()


    ###
    # Function to parse array.
    # Note: It will remove the first & last two elements of the given array
    ###
    def _parse_array_(self, array):
        try:
            del array[0]
            del array[-1]
            del array[-1]
        #except KeyError as e:
        except KeyError:
            _, e, _ = sys.exc_info()
            pass
        return array
    # EOF _parse_array_()


    ###
    # Function to get RCVG vv details associated to the host pv's wwid
    # Note- Incase the vv has been exported to the multiple vhosts, then it will
    # append the latest vv_wwid, vlun_id, vhost_name, vhost_wwn at the end for 
    # vv_name element
    ###
    def _get_rcvg_vv_details_(self, pv_wwid_array, host_wwid_array):
        self._DEBUG_("Inside function _get_rcvg_vv_details_")
        RC = self._get_rcvg_details_()
        if RC: return RC
        storage_rcvg_name = self.s_storage_rcvg_name
        rc1, rc2, self.s_cli_ssh, self.p_cli_ssh = self._get_cli_ssh_()
        if rc1 or rc2:
            self.ret_error = "Unable to get the CLI or SSH details for DC1,DC2"
            return 113

        ## Get VV into an array
        #  communicate with array to get the VV names
        value = "showvv -p -rcopygroup " + storage_rcvg_name + \
                                              " -showcols Name,VV_WWN -csvtable"

        output, p_status, err = self._cmd_exec_(name = self.s_cli_ssh, \
                                                                  value = value)
        self._DEBUG_("value: ", value, "output: ", output)
        vv_wwid_array = {}
        vv_wwid_array_new = {}
        vv_wwid_array = output.split("\n")
        vv_wwid_array = self._parse_array_(vv_wwid_array)
        self._DEBUG_("vv_wwid_array: ", vv_wwid_array)
        for vv_wwid_val in vv_wwid_array:
            # print vv_wwid_val
            # VV_NAME,VV_WWID
            vv_wwid_val = vv_wwid_val.split(",")
            vv_name = vv_wwid_val[0] 
            vv_val = vv_wwid_val[1]
            for pv_wwid in pv_wwid_array:
                if (pv_wwid.lower() == vv_val.lower()):
                    vv_wwid_array_new[vv_name] = vv_val
        self._DEBUG_("vv_wid_array_new:",vv_wwid_array_new)
        # {'VV_NAME.0': 'VV_WWID'}
        if not vv_wwid_array_new:
            self.ret_error = "Unable to get any matching virtual volume " + \
                                         "details w.r.t to package PV's."
            return 129
		
        # Get vlun details for all vv's with the vhost name & wwid
        concat_string = " -v "
        for vv_name in vv_wwid_array_new:
            self._DEBUG_("vv_name: ", vv_name)
            concat_string += vv_name + ","
        concat_string = concat_string[:-1]
        self._DEBUG_("concat_string: ", concat_string)
        # concat_string:   -v VV1,VV2

        # communicate with array to get all vhost & vlun id details for all  
        # VV's of a given RCVG 
        vlun_array     = {}
        vlun_array_new = {}
        vlun           = ""
        vlun_val       = ""
        value          = "showvlun -a" + concat_string + \
                            " -showcols Lun,VVName,HostName,Port,Type -csvtable"
        self._DEBUG_("value: ", value)
        # showvlun -a -v VV1,VV2 -showcols Lun,
        # VVName,HostName,Port,Type -csvtable

        output, p_status, err = self._cmd_exec_(name = self.s_cli_ssh, \
                                                                  value = value)

        if p_status:
            self.ret_error = "Unable to get the active vlun details"
            return 130

        vlun_array = output.split('\n')

        #Lun,VVName,HostName,Port,Type
        #0,VV1,HOST_NAME,---,host
        #2,VV2,HOST_NAME,---,host
        #-----------------------------------------
        #2,total,,,

        vlun_array = self._parse_array_(vlun_array)

        iter = 0
        self._DEBUG_("vlun_array: ", vlun_array)
        if not vlun_array:
            self.ret_error = "Unable to get the vlun details"
            return 128
        # vlun_array:  ['0,VV_NAME,HOST_NAME,---,host']

        vv_name_to_check = ""
        host_flag        = False

        for vlun in vlun_array:
            vlun_array_new = vlun.split(",")

            self._DEBUG_("vlun_array_new: ",vlun_array_new)
            # vlun_array_new:  ['0', 'VV_NAME', 'HOST_NAME', '---', 'host']

            ## Let's keep only unique VV's to the rcvg_vv_array array. Hence 
            # array will have only 1 unique set of PV & array details
            if vlun_array_new[1] in vv_name_to_check:
                continue

            self._get_vv_info_(vlun_array_new, self.s_cli_ssh)
            vv_details_arr['VV_WWID'] = \
                                    vv_wwid_array_new[vv_details_arr['VV_NAME']]

            ## Get the host wwid 
            # Note: A host can have more than one wwid
            # If presentation type is "host_set", then only we should check this
            host_set_array = []
            if (vv_details_arr['PRES_TYPE'] != "host_set"):
                value = "showhost -d " + vlun_array_new[2] + " -csvtable  " + \
                            " | grep -v Id,Name | awk -F '"','"' \'{print $4}\'"
            else:
                value = "showhostset " + vv_details_arr['HOST'] + \
                  " | grep -v Members | grep " + '"' + vv_details_arr['HOST'] +\
                  ' "' + " | awk \'{print $3}\'"
                output, p_status, err = self._cmd_exec_\
                                          (name = self.s_cli_ssh, value = value)
                host_set_array.append(output) 
                value = "showhostset " + vv_details_arr['HOST'] + \
                                    " | grep -v Members | grep -v " + '"' + \
                                    vv_details_arr['HOST'] + ' "' + \
                                    " | awk \'{print $1}\' | head -n-2" 
                output, p_status, err = self._cmd_exec_\
                                          (name = self.s_cli_ssh, value = value)
                for val in output.split('\n'):
                    host_set_array.append(val)
                concat_string = ""
                for host_name in host_set_array:
                    concat_string += " -e "
                    concat_string += '"' + host_name + ' "'
                value ="showhost -d|grep " + concat_string + \
                                                         " | awk \'{print $4}\'"
            #Get the wwid of HOST/HOST_SET in an array
            output, p_status, err = self._cmd_exec_(name = self.s_cli_ssh,\
                                                                  value = value)
            host_array = output.split('\n')
            self._DEBUG_("host_array: ", host_array)
            self._DEBUG_("host_wwid_array: ", host_wwid_array)
            # host_array:  ['10000090FA269AA4', '10000090FA269AA5']

            # Need to iterate through the complete vv array, because one vv can 
            # be exported to multiple vhost. So we need to get the complete list
            for iter in vv_wwid_array_new:
                #Continue only if vv_wwid_array_new iter has VV name
                if(iter == vv_details_arr['VV_NAME']):
                    # Need to get the active vlun details to compare 
                    # with others
                    value = "showvlun -a -v " + vv_details_arr['VV_NAME'] +\
                                                           " -showcols Host_WWN"
                    output,p_status,err= self._cmd_exec_\
                                            (name= self.s_cli_ssh, value= value)
                    active_wwid_array = output.split('\n')
                    active_wwid_array= self._parse_array_(active_wwid_array)

                    active_host_wwid = ""
                    for host in host_array:
                        self._DEBUG_("host: ", host)
                        for host_val_wwid in host_wwid_array:
                            self._DEBUG_("host_val_wwid: ", host_val_wwid)
                            if(host_val_wwid.lower() == host.lower()):
                                for active_wwid in active_wwid_array:
                                    self._DEBUG_("active_wwid: ", active_wwid_array)
                                    if(host_val_wwid.lower() == active_wwid.lower()):
                                        host_flag = True
                                        active_host_wwid = active_wwid
                                        break
                                if host_flag:
                                    break
                        if host_flag:
                            break
                    # We need to consider vluns only exported to the DRR node.
                    # Ignore the vluns exported to any other host.
                    if host_flag:
                        self._DEBUG_("active_host_wwid: ", active_host_wwid)
                        vv_details_arr['HOST_VAL'] = active_host_wwid
                        vv_details_arr['ACTIVE_HOST_VAL'] = active_host_wwid
                        # Get host alias name of a wwid if HOST_SET != NA
                        if (vv_details_arr['HOST_SET'] != "NA"):
                            value = "showhost -d | grep -i " + host + \
                                                        " | awk \'{print $2}\'"
                            output, p_status, err= self._cmd_exec_\
                                            (name= self.s_cli_ssh, value= value)
                            vv_details_arr['HOST'] = output

                        # Get VV alias name of a wwid if VV_SET != NA
                        if (vv_details_arr['VV_SET'] != "NA"):
                            value = "showvv -d | grep -i " + \
                                            vv_details_arr['VV_NAME'] + \
                                            " | awk \'{print $2}\'"
                            output, p_status, err= self._cmd_exec_\
                                            (name= self.s_cli_ssh, value= value)
                            vv_details_arr['VV_NAME'] = output

                        ## Append to rcvg_vv_array to make a set
                        self.rcvg_vv_array.append(vv_details_arr)
                        vv_name_to_check += vv_details_arr['VV_NAME']
                        break
                    else:
                        #print "ERROR:Unable to get the matching host wwn/iscsi"
                        vv_details_arr['HOST_VAL'] = "NA"
                        self.ret_error = "Unable to get the matching exposed" +\
                                         " host name."

        self._DEBUG_("rcvg_vv_array: ",self.rcvg_vv_array)
        # rcvg_vv_array:  [{'HOST_VAL': 'host_wwid', 'HOST_SET': 'NA', 
        # 'VV_WWID': 'vv_wwid', 'PRES_TYPE': 'host_sees', 
        # 'LUN_ID': '0', 'HOST': 'host_name', 'VV_SET': 'NA', 'PORT': '---', 
        # 'VV_NAME': 'vv_name'}

        ## If host found flag is True then return sucess otherwise return failure
        # to be caught in the next step
        if host_flag:
            return 0
        else:
            return 1
    # EOF _get_rcvg_details_()


    ###
    # Funtion to create snapshot of a given volume at the array.
    # Usage: _create_snapshot_(v_name, h_name, snap_vv_name)
    ###
    def _create_snapshot_(self, v_name, h_name, snap_vv_name):
        self._DEBUG_("Inside function _create_snapshot_")
        create_snap_cmd   = 'createsv ' + snap_vv_name + " " + v_name
        self._DEBUG_("create_snap_cmd: ", create_snap_cmd)
        command = "showvv " + snap_vv_name
        output, p_status, err = self._cmd_exec_(name = self.s_cli_ssh, \
                                                      value = command)
        if 'CopyOf' in output :
            self.ret_error = "Snapshot volume with name '" + \
                       snap_vv_name + "' already exist. Please delete it first."
            return 121
        output,p_status,err= self._cmd_exec_(name = self.s_cli_ssh, \
                                            value = create_snap_cmd)
        return p_status
    # EOF _create_snapshot_()


    ###
    # Funtion to remove snapshot vv.
    # Usage: _remove_snapshot_vv_(snap_vv_name)
    ###
    def _remove_snapshot_vv_(self, snap_vv_name):
        self._DEBUG_("Inside function _remove_snapshot_vv_")
        remove_snap_vv_cmd   = 'removevv -f ' + snap_vv_name
        self._DEBUG_("remove_snap_vv_cmd: ", remove_snap_vv_cmd)
        command = "showvv " + snap_vv_name
        output, p_status, err = self._cmd_exec_(name = self.s_cli_ssh, \
                                                      value = command)
        if 'no vv listed' in output :
            self.ret_error= "WARNING: Snapshot volume with name",snap_vv_name + \
                                                              " doesn't exist."
            return 0
        output,p_status,err= self._cmd_exec_(name = self.s_cli_ssh, \
                                            value = remove_snap_vv_cmd)
        return p_status
    # EOF _create_snapshot_()



    ###
    # Funtion to create vlun of a given volume at the array.
    # Usage: _create_vlun_cmd_(virt_vol_array, "ORIGINAL")
    ###
    def _create_vlun_cmd_(self, virt_vol_array, vv_type):
        self._DEBUG_("Inside function _create_vlun_cmd_")
        # Create vlun command to expose the disk.
        if (vv_type == "ORIGINAL"):
            if (virt_vol_array['VV_SET'] != "NA"):
                vv_name = virt_vol_array['VV_SET']
            else:
                vv_name = virt_vol_array['VV_NAME']

            # Create remove vlun command to un-expose the disk.
            if (virt_vol_array['PRES_TYPE'] == "port_set"):
                create_vlun_cmd = 'createvlun ' + vv_name + " " +  \
                                  virt_vol_array['LUN_ID'] + " " + \
                                  virt_vol_array['PORT']
            elif (virt_vol_array['PRES_TYPE'] == "matched_set"):
                create_vlun_cmd = 'createvlun ' + vv_name + " " +  \
                                  virt_vol_array['LUN_ID'] + " " + \
                                 virt_vol_array['HOST'] + virt_vol_array['PORT']
            elif (virt_vol_array['PRES_TYPE'] == "host_set"):
                create_vlun_cmd = 'createvlun ' + vv_name + " " +  \
                                  virt_vol_array['LUN_ID'] + " " + \
                             virt_vol_array['HOST_SET'] + virt_vol_array['PORT']
            else:
                create_vlun_cmd = 'createvlun ' + vv_name + " " +  \
                                  virt_vol_array['LUN_ID'] + " " + \
                                  virt_vol_array['HOST']
        else:
            create_vlun_cmd = 'createvlun -f ' + virt_vol_array['SV_NAME'] + \
                                               " auto " + virt_vol_array['HOST']
        # Only if i am appending the variable then its working, otherwise 
        # createvlun is giving error.
        self._DEBUG_("create_vlun_cmd: ", create_vlun_cmd)
        cli_ssh = virt_vol_array['ARRAY_CON']
        cli_ssh += " " + create_vlun_cmd

        output, p_status, err= self._cmd_exec_(name = cli_ssh, value = " ")
        if p_status:
            if (vv_type == "ORIGINAL"):
                self.ret_error= "Unable to expose virtual volume-"+\
                               vv_name + " to the host-" + virt_vol_array['HOST']
            else:
                self.ret_error = "Unable to expose snapshot vol-"+ \
                                 virt_vol_array['SV_NAME'] + " to the host-" + \
                                 virt_vol_array['HOST']
            return 124
        return p_status
    # EOF _create_vlun_cmd_()


    ###
    # Funtion to remove vlun of a given volume at the array.
    # Usage: _remove_vlun_cmd_(virt_vol_array, "ORIGINAL")
    ###
    def _remove_vlun_cmd_(self, virt_vol_array, vv_type):
        self._DEBUG_("Inside function _remove_vlun_cmd_")
        self._DEBUG_("virt_vol_array: ", virt_vol_array)

        ## Remove a original virtual volume command
        if (vv_type == "ORIGINAL"):

            if (virt_vol_array['VV_SET'] != "NA"):
                vv_name = virt_vol_array['VV_SET']
            else:
                vv_name = virt_vol_array['VV_NAME']

            # Create remove vlun command to un-expose the disk.
            if (virt_vol_array['PRES_TYPE'] == "port_set"):
                remove_vlun_cmd = 'removevlun -f ' + vv_name + " " +  \
                                     virt_vol_array['LUN_ID'] + " " + \
                                     virt_vol_array['PORT']
            elif (virt_vol_array['PRES_TYPE'] == "matched_set"):
                remove_vlun_cmd = 'removevlun -f ' + vv_name + " " +  \
                                     virt_vol_array['LUN_ID'] + " " + \
                                 virt_vol_array['HOST'] + virt_vol_array['PORT']
            elif (virt_vol_array['PRES_TYPE'] == "host_set"):
                remove_vlun_cmd = 'removevlun -f ' + vv_name + " " +  \
                                  virt_vol_array['LUN_ID'] + " " +    \
                             virt_vol_array['HOST_SET'] + virt_vol_array['PORT']
            else:
                remove_vlun_cmd = 'removevlun -f ' + vv_name + " " +  \
                                    virt_vol_array['LUN_ID'] + " " +  \
                                    virt_vol_array['HOST']

        ## Remove a snapshot volume command
        else:
            remove_vlun_cmd = 'removevlun -f ' + virt_vol_array['SV_NAME'] + \
               " " +  virt_vol_array['SV_LUN_ID'] + " " + virt_vol_array['HOST']

        self._DEBUG_("remove_vlun_cmd: ", remove_vlun_cmd)
        cli_ssh = virt_vol_array['ARRAY_CON']
        # Only if i am appending the variable then its working, otherwise 
        # removevlun is giving error.
        cli_ssh += " " + remove_vlun_cmd

        output, p_status, err= self._cmd_exec_(name = cli_ssh, value = " ")
        if p_status:
            if (vv_type == "ORIGINAL"):
                self.ret_error = "Unable to un-expose vv-", vv_name +\
                                     " from the host- " + virt_vol_array['HOST']
            else:
                self.ret_error = "Unable to un-expose snapshot vol- "  + \
                          virt_vol_array['SV_NAME'] + " from the host- "  + \
                          virt_vol_array['HOST']
            return 125
        return p_status
    # EOF _remove_vlun_cmd_()


    ###
    # Function to list the exported vlun details for a given volume
    ###
    def _show_vlun_(self, virt_vol_array, snap_vv_name):
        #value = "showvlun | grep " + snap_vv_name + \
        #                           " | grep -v '\-\-\-\-\-\-\-\-\-\-\-\-\-'"
        value = "showvlun -a -v " + snap_vv_name + \
                                 " -showcols Lun,VVName,HostName,Port -csvtable"
        cli_ssh = virt_vol_array['ARRAY_CON']
        self._DEBUG_("cli_ssh: ", cli_ssh)

        output, p_status, err= self._cmd_exec_(name= cli_ssh, value= value)
        snap_array = {}
        snap_vlun_array = {}
        snap_vlun_array = output.split()
        snap_vlun_array = self._parse_array_(snap_vlun_array)
        if not snap_vlun_array:
            return 1
        snap_vlun_array = snap_vlun_array[0].split(",")
        virt_vol_array['SV_LUN_ID'] = snap_vlun_array[0]
        virt_vol_array['SV_HOST']   = snap_vlun_array[2]
        virt_vol_array['SV_PORT']   = snap_vlun_array[3]     
        return p_status
    # EOF _show_vlun_()


    ###
    # Funtion to get the virtual volume wwid from the storage array.
    # Usage: _get_vv_wwid_(vv_name, cli_ssh)
    ###
    def _get_vv_wwid_(self, cli_ssh, vv_name):
        self._DEBUG_("Inside function _get_vv_wwid_")
        # Create command to get the wwid details.
        get_vv_wwid = 'showvv -showcols VV_WWN ' + vv_name

        # Only if i am appending the variable then its working, otherwise 
        # createvlun is giving error.
        #cli_ssh = virt_vol_array['ARRAY_CON']
        cli_ssh += " " + get_vv_wwid
        output, p_status, err= self._cmd_exec_(name = cli_ssh, value = " ")
        if (p_status != 0):
            return 1
        output = output.split('\n')
        self.sv_wwid = self._parse_array_(output)[0]

        if self.sv_wwid:
            return 0
        else:
            self.ret_error = "Unable to get snapshot " + vv_name + \
                                                         " wwid." + self.sv_wwid
            return 126
    # EOF _get_vv_wwid_()


    ###
    # Function to remove a virtual volume from 3par array
    ###
    def _remove_vv_(self, vv_name, cli_ssh):
        value = "removevv -f " + vv_name
        output,p_status,err= self._cmd_exec_(name = cli_ssh, value = value)
    # EOF _remove_vv_()


    ###
    # Function to get all vv's with wwid associated to a RCVG
    ###
    def _get_rcvg_vv_wwid_(self, rcvg_name, cli_ssh, type):
        cmd = cli_ssh + ' showvv'
        value='-p -rcopygroup ' + rcvg_name + ' -showcols Name,VV_WWN -csvtable'
        output, p_status, err = self._cmd_exec_(name = cmd, value = value)
        if p_status:
            self.ret_error = "Unable to get the vv details for rcvg"
            return 127

        # Split using ',' & append to an array to be use later on
        vv_wwid_array = {}
        self.vm_vv_wwid_array = {}

        # Parse the output
        vv_wwid_array = self._parse_array_(output.split('\n'))

        #vv_wwid_array = self._parse_array_(vv_wwid_array)
        self._DEBUG_("vv_wwid_array: ", vv_wwid_array)

        # Iterate through array to make a set of vv & wwid
        for vv_wwid_val in vv_wwid_array:
            vv_wwid_val = vv_wwid_val.split(",")
            vv_name = vv_wwid_val[0] 
            vv_wwid = vv_wwid_val[1]
            self._DEBUG_("vv_name :", vv_name)
            self._DEBUG_("vv_wwid :", vv_wwid)

            ## If type is 'snap' then create vv_name:vv_wwid array
            if type == 'snap':
                self.vm_vv_wwid_array[vv_name] = vv_wwid
            else:
                self.vm_vv_wwid_array[vv_wwid] = vv_name

        self._DEBUG_("vm_vv_wwid_array:",self.vm_vv_wwid_array)
        # {'VV_NAME': 'VV_WWID'}

        return 0

    # EOF _get_rcvg__vv_wwid_()

    ###
    # Function to get all vv's with wwid associated to a RCVG
    ###
    def _get_rcvg_sync_vv_nam_(self, rcvg_name, cli_ssh):
        self._DEBUG_("Inside function _get_rcvg_sync_vv_nam_")

        cmd = cli_ssh + ' showrcopy groups ' + rcvg_name
        value = "-csvtable | awk NF | awk '/,LastSyncTime/{y=1;next}y'"
        output, p_status, err = self._cmd_exec_(name = cmd, value = value)
        if p_status:
            self.ret_error= "Unable to get the vv details for rcvg-" + rcvg_name
            return 127

        # Split the output using the new line character
        output = output.split('\n')

        # Iterate through the output array & split the value using ',' 
        for out_val in output:
            out_val = out_val.split(',')
            orig_vv = out_val[1]
            snap_vv = out_val[3]
            self.orig_vv_replicated_vv_array[orig_vv] = snap_vv

        self._DEBUG_("orig_vv_replicated_vv: ",self.orig_vv_replicated_vv_array)
        if not self.orig_vv_replicated_vv_array:
            self.ret_error= "Unable to get the original & replicated virtual"+\
                                                               " volume mapping"
            return 1
        return 0

    # EOF _get_rcvg_sync_vv_nam_()

# EOC 3par()
